#!/usr/bin/python

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

def main(max=2, bound=1000):
    """Generate system with num sensors and filters, plus
    property with upper time bound set to bound.
    Returns (m_name, f_name, p_name)"""

    slim = """system implementation Sensors.Impl
  subcomponents
    sensor1: system Sensor in modes (Mode1);
	SUBS
  connections
    data port sensor1.output -> output in modes (Mode1);
	CONS
  modes
    Mode1: activation mode;
	MODES
  transitions
	TRANSS
end Sensors.Impl;

system implementation Filters.Impl
  subcomponents
    filter1: system Filter in modes (Mode1);
	SUBF
  connections
	CONF
    data port input -> filter1.input in modes (Mode1);
    data port filter1.output -> output in modes (Mode1);
  modes
    Mode1: activation mode;
	MODEF
  transitions
	TRANSF
end Filters.Impl;
"""

    fis = """<ns0:association>
    <ns0:componentPath>sensors.sensorNUM</ns0:componentPath>
    <ns0:errorModel>
      <ns0:packageName>__default__</ns0:packageName>
      <ns0:implementationName>SensorFailures.Impl</ns0:implementationName>
    </ns0:errorModel>
    <ns0:faultEffectsSpecification>
      <ns0:faultEffects errorState="Dead">
        <ns0:effect>output := 15</ns0:effect>
      </ns0:faultEffects>
    </ns0:faultEffectsSpecification>
  </ns0:association>"""

    fif = """<ns0:association>
    <ns0:componentPath>filters.filterNUM</ns0:componentPath>
    <ns0:errorModel>
      <ns0:packageName>__default__</ns0:packageName>
      <ns0:implementationName>FilterFailures.Impl</ns0:implementationName>
    </ns0:errorModel>
    <ns0:faultEffectsSpecification>
      <ns0:faultEffects errorState="Dead">
        <ns0:effect>output := 0</ns0:effect>
      </ns0:faultEffects>
    </ns0:faultEffectsSpecification>
  </ns0:association>"""

    SUBS = ""
    CONS = ""
    MODES = ""
    TRANSS = ""
    SUBF = ""
    CONF = ""
    MODEF = ""
    TRANSF = ""

    for i in range(2,max+1):
        s = str(i)
        s2 = str(i-1)
        SUBS += "sensor%s: system Sensor in modes (Mode%s);\n" % (s, s)
        CONS += "data port sensor%s.output -> output in modes (Mode%s);\n" % (s, s)
        MODES += "Mode%s: mode;\n" % (s)
        TRANSS += "Mode%s -[switch]-> Mode%s;\n" % (s2, s)
        SUBF += "filter%s: system Filter in modes (Mode%s);\n" % (s,s)
        CONF += "data port filter%s.output -> output in modes (Mode%s);\n" % (s,s)
        CONF += "data port input -> filter%s.input in modes (Mode%s);\n" % (s,s)
        MODEF += "Mode%s: mode;\n" % (s)
        TRANSF += "Mode%s -[switch]-> Mode%s;\n" % (s2, s)

    FI = ""
    for i in range(1,max+1):
        FI += fif.replace("NUM", str(i)) + "\n";
        FI += fis.replace("NUM", str(i)) + "\n";

    slim = slim.replace("SUBS", SUBS)
    slim = slim.replace("CONS", CONS)
    slim = slim.replace("MODES", MODES)
    slim = slim.replace("TRANSS", TRANSS)
    slim = slim.replace("SUBF", SUBF)
    slim = slim.replace("CONF", CONF)
    slim = slim.replace("MODEF", MODEF)
    slim = slim.replace("TRANSF", TRANSF)

    propxml = """<?xml version='1.0' encoding='UTF-8'?>
<ns0:properties xmlns:ns0="http://compass.informatik.rwth-aachen.de/properties">
  <ns0:property>
    <ns0:name>probabilisticExistence 0</ns0:name>
    <ns0:description> The probability that proposition (sensors.sensorNUM.error = error:Dead) will eventually hold within timebound [ 0 , BOUND ]. </ns0:description>
    <ns0:cslPatternInstance patternId="probabilisticExistence">
      <ns0:stateFormulaInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/csl" placeholderId="Phi">
        <ns0:atomicProposition>(sensors.sensorNUM.error = error:Dead)</ns0:atomicProposition>
      </ns0:stateFormulaInstance>
      <ns0:timeBoundInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/csl" placeholderId="Time1">
        <ns0:value>0</ns0:value>
      </ns0:timeBoundInstance>
      <ns0:timeBoundInstance xmlns:ns0="http://compass.informatik.rwth-aachen.de/csl" placeholderId="Time2">
        <ns0:value>BOUND</ns0:value>
      </ns0:timeBoundInstance>
    </ns0:cslPatternInstance>
  </ns0:property>
</ns0:properties>"""

    fixml = """<?xml version='1.0' encoding='UTF-8'?>
<ns0:faultInjection xmlns:ns0="http://compass.informatik.rwth-aachen.de/fault">
INJ
</ns0:faultInjection>"""

    PXML =  propxml.replace("NUM", str(max))
    PXML =  PXML.replace("BOUND", str(bound))
    FXML = fixml.replace("INJ", FI)

    slim_fname = "sensorfilter-" + str(max) + ".slim"
    f = open(slim_fname, "w");
    f.write(slim)
    f.close()

    fi_fname = "sensorfilter-" + str(max) + ".fixml"
    f = open(fi_fname, "w");
    f.write(FXML)
    f.close()

    props_fname = "sensorfilter-" + str(max) + ".propxml"
    f = open(props_fname, "w");
    f.write(PXML)
    f.close()
    
    return (slim_fname, props_fname, fi_fname)

# launcher and options handler
if __name__ == "__main__": 
    import sys
    try: bits = int(sys.argv[1])
    except: 
        sys.stderr.write("""Usage: gen.py <num> 
Where <num> is the number of sensors and filters.
""")
        sys.exit(1)
        pass

    if bits < 2:
        sys.stderr.write("""The number of sensors/filters has to be greater than 1.
""")
        sys.exit(1)
        pass

    main(bits)
