# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



import gobject

class DelayedCall(object):
    """This class is used to call functions after a certain delay.
    For example this is useful when you want to show a tooltip after nothing
    happened for 500 ms"""
    def __init__(self, delay, callback):
        self._delay = delay
        self._callback = callback
        self._id = 0
    
    def change_color_and_balloon(self, entry, error_lst):
        """This method perfoms the delayed call"""
        #The name is a bit misleading but it is intuitive in the context it is
        #used in. The delayed call is used to display a balloon and change
        #an entry's color after some delay
        if self._id != 0:
            gobject.source_remove(self._id)
        if entry:
            self._id = gobject.timeout_add(self._delay, 
                                           self._timeout, entry, error_lst)
            
    def immediately_change_color_and_balloon(self, entry, error_lst):
        """This method makes the call without delay"""
        if self._id != 0:
            gobject.source_remove(self._id)
        if entry:
            self._timeout(entry, error_lst)
            
    def reset_delay(self):
        if self._id != 0:
            gobject.source_remove(self._id)
    
    def _timeout(self, entry, error_lst):
        self._id = 0
        self._callback(entry, error_lst)
        return False
