# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import os.path
import gtk
import gobject

import options
from gtkmvc.model import Model

from src.models.activity import ActivityModel

import property_management.src.api as api
from property_management.src.patterns import *
from property_management.src.constants import Logic
from property_management.src.constants import PlaceholderType
from property_management.src.properties import *
from property_management.src.formulae import AtomicProposition
from slim_compiler.src.model_extension.core import InvalidFaultInjectionDocumentError
from threading import Lock  # fix for ticket #485


class PropertyModel(object):
    def __init__(self, pattern_instance, name, description):
        self.property = Property.create(pattern_instance, name, description)
        self.pattern_instance = pattern_instance
        self.name = name
        self.description = description
        #self.instance_description = instance_description
        pass    

class PatternModel(object):
    def __init__(self, pattern):
        self.pattern = pattern
        self.logic = pattern.get_logic()
        self.story = pattern.get_story()
        self.id = pattern.id
        #TODO remove name, ==> refactor Controller and View
        self.name = pattern.id
        self.refresh()
        
    def __str__(self):
        return self.name
            
    def refresh(self):
        self.text_buffer = gtk.TextBuffer()
        self.anchors = []
        self.placeholders = []
        last_pos = self.text_buffer.get_end_iter()
        for s in self.story:
            if isinstance(s, PlaceholderInfo):
                anchor = self.text_buffer.create_child_anchor(last_pos)
                #the name is used to store the TYPE of the placeholder
                self.anchors.append((anchor, str(s.type)))
                self.placeholders.append(s)
            else:
                self.text_buffer.insert(last_pos, " " + s + " ")

class MyTreeStore(gtk.TreeStore):
    def __init__(self, *kwargs):
        gtk.TreeStore.__init__(self, *kwargs)

    def __iter__(self):
        self._outer_iterator = None
        self._inner_iterator = None
        return self
    
    def next(self):
        # if we have just started create an iterator for the (outer) rows
        if not self._outer_iterator:
            self._outer_iterator = (0,)
            
        # first check whether the row is a dummy row, in that case immediately
        # start iterating over the children
        try: # we have to convert our tuple path to a GtkTreeIter to check for children
            gtk_tree_iter = self.get_iter(self._outer_iterator)
        except ValueError:
            raise StopIteration
        
        if self.iter_has_child(gtk_tree_iter):
            # if we just started iteration over children create an iterator
            # for the children and return the first row
            if not self._inner_iterator:
                row_index, = self._outer_iterator
                self._inner_iterator = (row_index,0)
                return self[self._inner_iterator]
            # we have an iterator for children, try advancing
            else:
                row_index, child_index = self._inner_iterator
                if child_index + 1 < self.iter_n_children(gtk_tree_iter):
                    self._inner_iterator = (row_index, child_index + 1)
                    return self[self._inner_iterator]
                # otherwise just continue on row level
                else:
                    self._inner_iterator = None
                    # advance outer iterator one step
                    row_index, = self._outer_iterator
                    if row_index + 1 < len(self):
                        self._outer_iterator = (row_index + 1,)
                        return self.next()
                    else:
                        self._inner_iterator = None
                        self._outer_iterator = None
                        raise StopIteration
        else:
            row_index, = self._outer_iterator
            if row_index < len(self):
                self._outer_iterator = (row_index + 1,)
                return self[(row_index,)] #Yes, return the old one
            else:
                self._inner_iterator = None
                self._outer_iterator = None
                raise StopIteration
        
#TODO: maybe refactoring is needed:
# do not confuse the following PropertiesModel with
# Property or PropertyModel
class PropertiesModel(ActivityModel):

    # Columns indices
    USE = 0
    DESCRIPTION = 1
    PROPERTY = 2
    SELECT_SINGLE = 3
    SELECT_MULTI  = 4
    MC_RESULT = 5
    FT = 6
    ANALYSIS_RESULT = 7
    FDA_LIST = 8

    loaded_properties = 0
    __observables__ = ("loaded_properties", )

    def __init__(self, appl_model, slim_model, props_model = None):
        
        ActivityModel.__init__(self, appl_model, slim_model, props_model)

        self._init_properties_tree()
        self.property_counter = 0
        self.slim_model.register_observer(self)
	self.lock = Lock()
        return

    def get_properties_tree(self):
        """Returns a TreeStore containing all the properties"""
        return self.properties_tree
    
    def get_properties_list(self):
        """Returns a ListStore containing all the properties"""
        return self.properties_tree
    
    def on_use_property_toggled(self, rend, path):
        """Toggle whether a property should be used for some activity or not.
        Especially these checkboxes are disabled if a property is invalidated
        due to a reload of the SLIM model.
        """
        previously_selected = self.properties_tree[path][PropertiesModel.USE]
        # case 1: property is deselected, that's is always OK
        if previously_selected:
            # check if there are children that have to be deselected as well
            parent = self.properties_tree.get_iter(path)
            child_num = self.properties_tree.iter_n_children(parent)
            for i in xrange(child_num):
                child_iter = self.properties_tree.iter_nth_child(parent, i)
                self.properties_tree[child_iter][PropertiesModel.USE] = False
            self.properties_tree[path][PropertiesModel.USE] = False
            #if this is a child and it was the only selected one, its parent
            #should be deselected now, too
            child = self.properties_tree.get_iter(path)
            parent = self.properties_tree.iter_parent(child)
            if parent:
                #uncheck parent if all children are deselected
                child_num = self.properties_tree.iter_n_children(parent)
                selected_children_num = 0
                for i in xrange(child_num):
                    child_iter = self.properties_tree.iter_nth_child(parent, i)
                    if self.properties_tree[child_iter][PropertiesModel.USE]:
                        selected_children_num += 1
                if selected_children_num == 0:
                    self.properties_tree[parent][PropertiesModel.USE] = False
        
        # case 2: property is selected, before doing so validate property
        # against the current slim model        
        else:
            # check if there are children that have to be selected as well
            parent = self.properties_tree.get_iter(path)
            child_num = self.properties_tree.iter_n_children(parent)
            if child_num == 0:
                #we have only the property itself without children
                if self.is_valid_property(self.properties_tree[path][PropertiesModel.PROPERTY].property):
                    self.properties_tree[path][PropertiesModel.USE] = True
                    #if this is a child its parent should be selected now, too
                    child = self.properties_tree.get_iter(path)
                    parent = self.properties_tree.iter_parent(child)
                    if parent:
                        self.properties_tree[parent][PropertiesModel.USE] = True
                else:
                    return #don't select; it is invalid wrt current slim model
            else:
                #the property that is selected is a dummy, validate its children
                all_valid = True
                for i in xrange(child_num):
                    child_iter = self.properties_tree.iter_nth_child(parent, i)
                    if not self.is_valid_property(self.properties_tree[child_iter][PropertiesModel.PROPERTY].property):
                        return
                #if we are here then all properties are valid, select them and
                #their parent
                for i in xrange(child_num):
                    child_iter = self.properties_tree.iter_nth_child(parent, i)
                    self.properties_tree[child_iter][PropertiesModel.USE] = True
                self.properties_tree[path][PropertiesModel.USE] = True
        
    def toggle_property(self, path, multi):
        """Toggle selection value of a property. 

        In the properties tree there are two rows: row index 1
        contains single selection (mutex), and row index 2 contains
        multiple selections. Multiple are used e.g. by the model
        simulation to select a bunch of properties. If multi is
        true, then row 2 is set, otherwise row 1 is set.
        """
        
        #we are in multi selection mode
        if multi:
            previously_selected = self.properties_tree[path][PropertiesModel.SELECT_MULTI]
            # case 1: property is going to be deselected
            if previously_selected:
                # check if there are children that have to be deselected as well
                parent = self.properties_tree.get_iter(path)
                child_num = self.properties_tree.iter_n_children(parent)
                for i in xrange(child_num):
                    child_iter = self.properties_tree.iter_nth_child(parent, i)
                    self.properties_tree[child_iter][PropertiesModel.SELECT_MULTI] = False
                self.properties_tree[path][PropertiesModel.SELECT_MULTI] = False
                #if this is a child and it was the only selected one, its parent
                #should be deselected now, too
                child = self.properties_tree.get_iter(path)
                parent = self.properties_tree.iter_parent(child)
                if parent:
                    #uncheck parent if all children are deselected
                    child_num = self.properties_tree.iter_n_children(parent)
                    selected_children_num = 0
                    for i in xrange(child_num):
                        child_iter = self.properties_tree.iter_nth_child(parent, i)
                        if self.properties_tree[child_iter][PropertiesModel.SELECT_MULTI]:
                            selected_children_num += 1
                    if selected_children_num == 0:
                        self.properties_tree[parent][PropertiesModel.SELECT_MULTI] = False
            
            # case 2: property is going to be selected
            else:
                # check if there are children that have to be selected as well
                parent = self.properties_tree.get_iter(path)
                child_num = self.properties_tree.iter_n_children(parent)
                if child_num == 0:
                    #we have only the property itself without children
                    self.properties_tree[path][PropertiesModel.SELECT_MULTI] = True
                    #if this is a child its parent should be selected now, too
                    child = self.properties_tree.get_iter(path)
                    parent = self.properties_tree.iter_parent(child)
                    if parent:
                        self.properties_tree[parent][PropertiesModel.SELECT_MULTI] = True
                else:
                    #the property that is selected is a dummy, select its children
                    for i in xrange(child_num):
                        child_iter = self.properties_tree.iter_nth_child(parent, i)
                        self.properties_tree[child_iter][PropertiesModel.SELECT_MULTI] = True
                    self.properties_tree[path][PropertiesModel.SELECT_MULTI] = True
        #we are in single selection mode
        else:
            previously_selected = self.properties_tree[path][PropertiesModel.SELECT_SINGLE]
            # case 1: property is going to be deselected
            if previously_selected:
                # check if there are children that have to be deselected as well
                parent = self.properties_tree.get_iter(path)
                child_num = self.properties_tree.iter_n_children(parent)
                for i in xrange(child_num):
                    child_iter = self.properties_tree.iter_nth_child(parent, i)
                    self.properties_tree[child_iter][PropertiesModel.SELECT_SINGLE] = False
                self.properties_tree[path][PropertiesModel.SELECT_SINGLE] = False
                #if this is a child, its parent
                #should be deselected now, too
                child = self.properties_tree.get_iter(path)
                parent = self.properties_tree.iter_parent(child)
                if parent:
                    #uncheck parent
                    self.properties_tree[parent][PropertiesModel.SELECT_SINGLE] = False
            
            # case 2: property is going to be selected
            else:
                #since we are in single selection mode,
                #deselect all other properties
                for row in self.properties_tree:
                    row[PropertiesModel.SELECT_SINGLE] = False
                    #since we do not iterate over the header properties,
                    #we deselect them explicitly
                    parent = self.properties_tree.iter_parent(row.iter)
                    if parent:
                        #uncheck parent
                        self.properties_tree[parent][PropertiesModel.SELECT_SINGLE] = False
                    
                # check if there are children that have to be selected as well
                parent = self.properties_tree.get_iter(path)
                child_num = self.properties_tree.iter_n_children(parent)
                if child_num == 0:
                    #we have only the property itself without children
                    self.properties_tree[path][PropertiesModel.SELECT_SINGLE] = True
                    #if this is a child its parent should be selected now, too
                    child = self.properties_tree.get_iter(path)
                    parent = self.properties_tree.iter_parent(child)
                    if parent:
                        self.properties_tree[parent][PropertiesModel.SELECT_SINGLE] = True
                else:
                    #the property that is selected is a dummy,
                    #select ONE of its children, e.g. the first
                    #TODO: or forbid the selection of a header property??
                    child_iter = self.properties_tree.iter_nth_child(parent, 0)
                    self.properties_tree[child_iter][PropertiesModel.SELECT_SINGLE] = True
                    self.properties_tree[path][PropertiesModel.SELECT_SINGLE] = True
        return
    
    def set_mc_result(self, iter, result):
        """Sets the model checking result for given property. This
        happens when a property gets model checked, and a result is
        found. Then we want to associate that result to the
        corresponding property for future memory.  Param 'result'
        can be None or an instance of class McResultModel."""
        model = self.get_properties_list()
        model.set(iter, PropertiesModel.MC_RESULT, result)
        return

    def set_ft(self, iter, ft):
        """Associates a ft (a triple carrying file names) to the
        property.

        This happens when a fault tree is generated (in safety
        fault tree generation) from a property. Param 'ft' can be
        None or a triple of filenames (events_fname, gates_fname,
        xml_fname)"""
        model = self.get_properties_list()
        model.set(iter, PropertiesModel.FT, ft)
        return
    
    def set_analysis_result(self, iter, analysis_result):
        """
        @param iter: Iterator pointing to the row where the analysis result should be stored
        @type iter: gtk.TreeIter
        
        @param analysis_result: The analysis result generated during Performability Evaluation
        @type AnalysisResult: result of Performability Evaluation
        """
        model = self.get_properties_list()
        model.set(iter, PropertiesModel.ANALYSIS_RESULT, analysis_result)
        return

    def set_fda_list(self, iter, fda_list):
        """
        @param iter: Iterator pointing to the row where the graph should be stored
        @type iter: gtk.TreeIter
        
        @param fda_list: The fda list for this property (where applicable)
        @type fda_list: fda list
        """
        model = self.get_properties_list()
        model.set(iter, PropertiesModel.FDA_LIST, fda_list)
        return
    
    def _init_properties_tree(self):
        cr_text = gtk.CellRendererText()
        #cr_toggle = gtk.CellRendererToggle()
        #cr_toggle.set_radio(False) # we want a checkbox
        #cr_toggle.connect("toggled", self.on_use_property_toggled)
        self.properties_col = gtk.TreeViewColumn("Properties",cr_text, text=1)
        def foreground_color(column, cell, model, iter, user_data=None):
            if model[iter][PropertiesModel.USE]:
                cell.set_property("foreground", "black")
            else:
                cell.set_property("foreground", "gray")

        self.properties_col.set_cell_data_func(cr_text, foreground_color)
        #self.use_col = gtk.TreeViewColumn("Use",cr_toggle, active=0)
        
        # property model instance, flag for single selection, flag
        # for multiple selection, MCResultModel (None for no result),
        # associated Fault Tree (None for no FT), associated FDA list 
        # (None for no FDA)
        self.properties_tree = MyTreeStore(bool, # use
                                           str, # description
                                           gobject.TYPE_PYOBJECT, # property
                                           bool, # single selection
                                           bool, # multiple selection
                                           gobject.TYPE_PYOBJECT, # mc result
                                           gobject.TYPE_PYOBJECT, # ft generation
                                           gobject.TYPE_PYOBJECT, # performability graph
                                           gobject.TYPE_PYOBJECT  # fda list
                                           )
        pass
        
    def remove_property(self, iter):
        #check if we are a child
	self.lock.acquire() # bugfix for ticket 485
        parent = self.properties_tree.iter_parent(iter)
        self.properties_tree.remove(iter)
	self.lock.release() # bugfix for ticket 485
        if parent is not None:
            #check if we have been the last child
	    self.lock.acquire() # bugfix for ticket 485
            child_num = self.properties_tree.iter_n_children(parent)
	    self.lock.release() # bugfix for ticket 485
            if child_num == 0:
                #the dummy property has no children anymore so remove it
                self.remove_property(parent)
        pass
        
    def prepend_properties(self, m, after_sibling=None):
        """ Given an instance of the PropertyDialogModel m, inserts the properties
        into the tree
        """
	self.lock.acquire() # bugfix for ticket 485
        if len(m.get_properties_list()) == 1:
            property = m.get_properties_list()[0]
            self.properties_tree.insert_after(None, after_sibling,
                [True, property.description, property, False,False,None,None,None,None])
        elif len(m.get_properties_list()) > 1:
            props = m.get_properties_list()
            # create a header-property for the treeview and put the list of
            # properties below it
            property = props.pop(0)
            #we discard after_sibling because a component based property always
            #is added at top level
            parent = self.properties_tree.insert_after(None, None,
                        [True, m.story, property, False,False,None,None,None,None])
            #parent = None
            for property in props: # all properties except the first one (header)
                self.properties_tree.append(parent,
                    [True, property.description, property, False,False,None,None,None,None])
            pass
        else:
            assert False, "No property generated in the popup!"
        pass

	self.lock.release() # bugfix for ticket 485
    
    def get_root_impl(self):
        """TODO: unused?"""
        return self.slim_model.root_impl
    
    def is_valid_property(self, prop):
        root = self.slim_model.get_extended_root_impl()
        if not root:
            assert False
        formula = api.create_formula_for_property(prop)
        errors = api.get_errors_in_atomic_propositions_in_formula(formula,
                                    root)
        if len(errors) > 0:
            return False
        else:
            return True
    
    # ---------------------------------------------------------------
    #        Notifications from observable properties
    # ---------------------------------------------------------------    
    def property_changed_signal_emit(self, slim_model, args):
        """ The SLIM model has changed. We have to validate all properties
        against the (new) SLIM model now. Then disable those that are invalid.
        """
        root = None
        try:
            root = slim_model.get_extended_root_impl()
        except InvalidFaultInjectionDocumentError, e:
            root = None
        finally:
            if not root:
                #this happens when a second model is loaded but the user has not
                #chosen a root impl yet or fault injections are invalid
                # ==> disable all properties
                for row in self.get_properties_list():
                    if row[PropertiesModel.USE]:
                        self.on_use_property_toggled(None, row.path)
            else:
                for row in self.get_properties_list():
                    prop = row[PropertiesModel.PROPERTY].property
                    if not self.is_valid_property(prop):
                        #if this property was selected, deselect it now
                        if row[PropertiesModel.USE]:
                            self.on_use_property_toggled(None, row.path)
                    else:
                        #if property was disabled for use, enable it again
                        if not row[PropertiesModel.USE]:
                            self.on_use_property_toggled(None, row.path)

        # as a las step is necessary to clear any previous results for
        # mc, ft, fda
        for row in self.get_properties_list():
            row[PropertiesModel.MC_RESULT] = None
            row[PropertiesModel.FT] = None
            row[PropertiesModel.FDA_LIST] = None
    
    def property_failures_value_change(self, model, old, new):
        """ A fault injection might have been removed such that a property is
        invalid now. Check that (same as for "changed" signal)
        """
        self.property_changed_signal_emit(model,None)
        
    def generate_unique_properties_basename(self):
        """ Generates a unique filename for the fault injections upon saving"""
        base = options.BASE_FILE_NAME        
        if base == "": base = "properties"
        if not os.path.exists(base + ".propxml"):
            return base
            
        num = 2    
        while os.path.exists(base + "-" + str(num) + ".propxml"):
            num += 1            
        return base + "-" + str(num)
    #end of class PropertiesModel
     
class PropertyDialogModel(Model):
    ALL = "All"
    PROPOSITIONAL = "Propositional"
    FUNCTIONAL = "Functional"
    TIMED = "Timed"
    PROBABILISTIC = "Probabilistic"
    
    def __init__(self, slim_model, old_prop=None):
        """
        @param slim_model: reference to src.models.slim
        @param old_prop: if given the property dialog displays the contents
                         of that property, this is needed for easy editing
        """
        Model.__init__(self)
        
        self.slim_model = slim_model
        self.old_prop = old_prop

        # This list stores the properties (objects of PropertyModel) that are
        # created by the user.
        self.properties = []
        
        #pattern document used to create a list store of patterns
        #leave out LTL patterns since they are subsumed by CTL patterns and
        #there is a translation function between CTL and LTL properties
        #pd = [p for p in api.get_patterns_document(Logic.LTL)]
        pd = [p for p in api.get_patterns_document(Logic.CTL)]
        
        self.PROP_PATTERN = [pd[0].id] # the first one is supposed to be "propositional"
        assert "propositional" == self.PROP_PATTERN[0], self.PROP_PATTERN[0]
        self.FUNC_PATTERN = [p.id for p in pd[1:]]
        # There are no patterns for the specified logic. PCTL
        # pd.extend([p for p in api.get_patterns_document(Logic.PCTL)])
        prob_pd = [p for p in api.get_patterns_document(Logic.CSL)]
        
        self.PROB_PATTERN = [p.id for p in prob_pd]
        
        timed_pd = [p for p in api.get_patterns_document(Logic.MITL)]
        
        self.TIMED_PATTERN = [p.id for p in timed_pd]

        pd.extend(prob_pd)
        pd.extend(timed_pd)
        self._create_pattern_list(pd)
        
        self.categories = [PropertyDialogModel.ALL,
                      PropertyDialogModel.PROPOSITIONAL,
                      PropertyDialogModel.FUNCTIONAL,
                      PropertyDialogModel.TIMED,
                      PropertyDialogModel.PROBABILISTIC]
        self._create_categories_list(self.categories)
        self.filtered_patterns = self.pattern_list.filter_new()
        self.chosen_category = PropertyDialogModel.ALL
        def _categories_filter(model, it):
            pattern = model[it][0].name
            logic = model[it][0].logic
            if self.chosen_category == PropertyDialogModel.ALL:
                return True
            elif self.chosen_category == PropertyDialogModel.PROPOSITIONAL:
                return pattern in self.PROP_PATTERN
            elif self.chosen_category == PropertyDialogModel.PROBABILISTIC:
                return pattern in self.PROB_PATTERN and logic == Logic.CSL
            elif self.chosen_category == PropertyDialogModel.FUNCTIONAL:
                return pattern in self.FUNC_PATTERN and logic in [Logic.LTL, Logic.CTL]
            elif self.chosen_category == PropertyDialogModel.TIMED:
                return pattern in self.TIMED_PATTERN and logic == Logic.MITL
            else:
                assert False, self.chosen_category + "unknown category"
        self.filtered_patterns.set_visible_func(_categories_filter)
        
    def choose_patterns(self, category):
        assert category in self.categories, category
        self.chosen_category = category
        self.filtered_patterns.refilter()
        
    def _create_pattern_list(self, pd):
        """ create list model for patterns list """
        cr = gtk.CellRendererText()
        self.pattern_column = gtk.TreeViewColumn("Patterns",cr)
        def _pattern_cell_data_func(column, cell, model, iter):
            cell_text = str(model.get_value(iter, 0))
            cell.set_property('text', cell_text)
            pass
        self.pattern_column.set_cell_data_func(cr, _pattern_cell_data_func)
        
        self.pattern_list = gtk.ListStore(gobject.TYPE_PYOBJECT)
        for pat in pd:
            self.pattern_list.append([PatternModel(pat)])
        pass
        
    def _create_categories_list(self, categories):
        cr = gtk.CellRendererText()
        self.categories_column = gtk.TreeViewColumn("Categories",cr, text=0)
        
        self.categories_list = gtk.ListStore(str)
        for cat in categories:
            self.categories_list.append([cat])
    
    def instantiate(self, pattern_model, txt_placeholders, name, slice):
        assert len(txt_placeholders) == len(pattern_model.placeholders)
        # Store the original story that is entered by the user which is then 
        # used as the header for component based properties
        self.story = slice
        for txt_placeholder in txt_placeholders:
            self.story = self.story.replace(u"\uFFFC", txt_placeholder.strip(), 1)
        # Create a list with length len(pattern_model.placeholders).
        # Each list element is then again a list which has only 1 element if
        # the corresponding placeholder is a number or an AP without templates.
        placeholder_fill = []
        str_placeholder_fill = []
        for i in pattern_model.placeholders:
            placeholder_fill.append([])
            str_placeholder_fill.append([])

        for index in xrange(len(txt_placeholders)):
            #instantiate an apt PlaceholderInstance
            type = pattern_model.placeholders[index].type
            txt_placeholders[index] = txt_placeholders[index].strip()
            if type == PlaceholderType.STATE_FORMULA:
                # 1st: generate a list of strings of the form "package::component.impl"
                # where "package::" is omitted if package = "__default__"
                impl_names = []
                
                for package in self.slim_model.compile_info.packages.values():
                    for subpackage in [package.private_subpackage, package.public_subpackage]:
                        if subpackage.package_name is "__default__":
                            impl_names.extend(subpackage.implementations.keys())
                        else:
                            impl_names.extend([subpackage.package_name + "::" + impl_name
                                               for impl_name
                                               in subpackage.implementations.keys()])
                
                # count how many component implementations the AP refers to
                # and store their names
                found_impl_names = []
                for impl_name in impl_names:
                    # We have to search for the implementation name followed by a dot
                    # in the atomic proposition string. Otherwise substrings might be
                    # found.
                    if (-1 != txt_placeholders[index].find(impl_name + ".")):
                        found_impl_names.append(impl_name)
                    if len(found_impl_names) > 1:
                        break
                
                assert len(found_impl_names) <= 1 # this should be checked
                                                  # already by the GUI ctrl
                
                root_implementation = self.slim_model.get_extended_root_impl()
                if len(found_impl_names) == 1:
                    # 2nd: find one instance of the component implementation that is
                    # reachable from root.
                    paths = self._get_instances_of(found_impl_names[0])
                    assert len(paths) > 0, paths
                    # this is the AP for the dummy (header) property
                    ap = AtomicProposition.create(pattern_model.logic,
                                                  txt_placeholders[index])
                    pi = StateFormulaPlaceholderInstance.create(
                            pattern_model.logic,
                            pattern_model.placeholders[index].id,
                            ap)
                    placeholder_fill[index].append(pi)
                    str_placeholder_fill[index].append(txt_placeholders[index])
                    # now come the instances
                    for path in paths:
                        ap = AtomicProposition.create(pattern_model.logic,
                            txt_placeholders[index].replace(found_impl_names[0]+".", path))
                        pi = StateFormulaPlaceholderInstance.create(
                                pattern_model.logic,
                                pattern_model.placeholders[index].id,
                                ap)
                        placeholder_fill[index].append(pi)
                        str_placeholder_fill[index].append(txt_placeholders[index].replace(found_impl_names[0]+".", path))
                else:
                    ap = AtomicProposition.create(pattern_model.logic,
                                                  txt_placeholders[index])
                    pi = StateFormulaPlaceholderInstance.create(
                            pattern_model.logic,
                            pattern_model.placeholders[index].id,
                            ap)
                    placeholder_fill[index].append(pi)
                    str_placeholder_fill[index].append(txt_placeholders[index])
            elif type == PlaceholderType.TIME_BOUND:
                pi = TimeBoundPlaceholderInstance.create(
                        pattern_model.logic,
                        pattern_model.placeholders[index].id,
                        txt_placeholders[index])
                placeholder_fill[index].append(pi)
                str_placeholder_fill[index].append(txt_placeholders[index])
            elif type == PlaceholderType.THRESHOLD:
                pi = ThresholdPlaceholderInstance.create(
                        pattern_model.logic,
                        pattern_model.placeholders[index].id,
                        float(txt_placeholders[index]))
                placeholder_fill[index].append(pi)
                str_placeholder_fill[index].append(txt_placeholders[index])
            else:
                assert False, "invalid Placeholder Type in story"
            assert pi is not None
        self.fill(pattern_model, name, slice, placeholder_fill, str_placeholder_fill)
        
    def fill(self, pattern_model, name, slice, placeholder_fill, str_placeholder_fill):
        """ This method generates a list of properties.
        In general a pattern allows to have several placeholders for APs. Each
        AP may be component based instead of instance based. This method walks
        through all combinations and creates the corresponding properties.
        
        @param pattern_model: The pattern model
        @type  pattern_model: PatternModel
        @param placeholder_fill: List which contains for each placeholder a
                                 list of all possible instantiations
        @type  atomic_proposition:  list(list)
        """
        max_len = 0
        for lst in placeholder_fill:
            if len(lst) > max_len:
                max_len = len(lst)
        for lst, str_lst in zip(placeholder_fill, str_placeholder_fill):
            assert len(lst) == 1  or len(lst) == max_len, lst
            while len(lst) < max_len:
                deep_copy = copy_placeholder(lst[0], lst[0].get_logic())
                lst.append(deep_copy)
                str_lst.append(str_lst[0])

        for instance_index in xrange(max_len):            
            pattern_instance = PatternInstance.create(pattern_model.logic,
                                                      pattern_model.id)
            instance_description = slice
            for placeholder_index in xrange(len(placeholder_fill)):
                pattern_instance.add_placeholder_instance(placeholder_fill[placeholder_index][instance_index])            
                instance_description = instance_description.replace(u"\uFFFC", str_placeholder_fill[placeholder_index][instance_index], 1)
            property = PropertyModel(pattern_instance, name, instance_description)
            self.properties.append(property)
    
    def get_properties_list(self):
        return self.properties
    
    def _get_instances_of(self, impl_name):
        """returns a list of qualified subcomponent names which are instances
        of a given subcomponent implementation
        """
        root = self.slim_model.root_impl
        paths = []
        if root:
            # The queue is a list of pairs
            # ("path" , ComponentImplementation object)
            queue = [("",root)]
    
            while queue != []:
                current_impl = queue[0][1]
                if current_impl.name == impl_name:
                    paths.append(queue[0][0])
                #add the subcomponents of the current subcomponent to the queue
                for id in current_impl.non_data_subcomponents.keys():
                    path = queue[0][0] + str(id) + "."
                    subcmp = current_impl.non_data_subcomponents[id].get_referenced_component_implementation()
                    queue.append((path, subcmp))
                queue.pop(0)
            
        return paths
    
# some utility methods for property management that are independent of classes

def copy_placeholder(placeholder, logic):
    """ Given a placeholder returns a deep copy of it - i.e. a newly created
    placeholder that is independent of the original but has exactly the same
    contents
    
    @param placeholder: A placeholder instance
    @type  placeholder: StateFormulaPlaceholderInstance or
                        ThresholdPlaceholderInstance or
                        TimeBoundPlaceholderInstance
    @param logic: The logic that the placeholder should have
    @type: property_management.constants.Logic
    @return: A new placeholder instance with the same contents
    @rtype: StateFormulaPlaceholderInstance or
            ThresholdPlaceholderInstance or
            TimeBoundPlaceholderInstance
    """
    if isinstance(placeholder, StateFormulaPlaceholderInstance):
        # The following code will only work if an atomic proposition is used
        # to instantiate the state formula placeholder.
        assert isinstance(placeholder.content, AtomicProposition)
        ap = AtomicProposition.create(logic, placeholder.content.text)
        deep_copy = StateFormulaPlaceholderInstance.create(
            logic,
            placeholder.placeholder_id,
            ap)
    elif isinstance(placeholder, ThresholdPlaceholderInstance):
        deep_copy = ThresholdPlaceholderInstance.create(
            logic,
            placeholder.placeholder_id,
            placeholder.content)
    elif isinstance(placeholder, TimeBoundPlaceholderInstance):
        deep_copy = TimeBoundPlaceholderInstance.create(
            logic,
            placeholder.placeholder_id,
            placeholder.content)
    else:
        assert False, "Unexpected Placeholder instance" + placeholder
    return deep_copy

def translate_property_to_logic(property_model, new_logic):
    """ Given an LTL or CTL property and the desired logic the function creates
    a new property with the same contents and the same pattern in another logic.
    None is returned if the pattern does not exist for the desired logic.
    
    @param property_model: The original property
    @type property_model: PropertyModel object
    @param new_logic: The logic that the property should be traslated to
    @type: property_management.constants.Logic
    @return: A new property in the desired logic or None on failure
    @rtype: PropertyModel or None
    """
    # make sure new_logic is LTL or CTL since these are the only logics
    # between which we can translate
    if new_logic is Logic.LTL:
        available_pattern_ids = [p.id
                                 for p in api.get_patterns_document(Logic.LTL)]
    elif new_logic is Logic.CTL:
        available_pattern_ids = [p.id
                                 for p in api.get_patterns_document(Logic.CTL)]
    else:
        return None
    
    # make sure the logic of the property is LTL or CTL since these are the
    # only logics between which we can translate
    if property_model.property.get_logic() not in (Logic.LTL, Logic.CTL):
        return None
    
    # make sure that the pattern of the current property is also available for
    # the desired logic. This check is needed for example for
    # CTLresponseExist -> LTL translation which is impossible
    pattern_id = property_model.property.pattern_instance.pattern_id
    if not pattern_id in available_pattern_ids:
        return None
    
    #OK, everything's fine, let's translate
    new_placeholders = [copy_placeholder(p, new_logic) for p in
                        property_model.property.pattern_instance]
    new_pattern_instance = PatternInstance.create(new_logic, pattern_id)
    for p in new_placeholders:
        new_pattern_instance.add_placeholder_instance(p)
    new_property_model = PropertyModel(new_pattern_instance,
                                       property_model.property.name,
                                       property_model.property.description)
    return new_property_model
