# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



import gtk
import gobject

from gtkmvc import Model
from src.models.tool_handler import Model2ImcHandlerModel, MRMCHandlerModel, NuSMVHandlerModel


class ToolHandlersModel (Model):
    """This class is used to monitor and handle processes in form
    of ToolHandlerModel instances."""
    
    def __init__(self):
        Model.__init__(self)
                
        # name (or group), and ToolHandlerModel (or None for group)
        self.tools = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        return


    def add_tool(self, name, tool, group=None):
        """group is the group of tools this tool belongs to. It can
        be None for not specifying any group. 

        name is a descriptive name for the purpose of the tool, and
        tool is a ToolHandlerModel instance"""

        if group is None:
            # no group was specified
            self.tools.append(None, (name, tool))
            return

        # group was specified, searches it first
        for row in self.tools:
            if row[0] == group and row[1] is None:
                self.tools.append(row.iter, (name, tool))
                return
            pass
            
        # here group was never added before:
        row = self.tools.append(None, (group, None))
        self.tools.append(row, (name, tool))
        return

    def set_tool_limits(self, memout, timeout):
        """Sets given limits for all handled tools. memory in MB,
        time in seconds. -1 means no limit"""
        
        def func(model, path, iter, mem_time_limits):
            if model[iter][1] is not None: 
                model[iter][1].set_mem_limit(mem_time_limits[0])
                model[iter][1].set_time_limit(mem_time_limits[1])
                pass
                
            return False

        self.tools.foreach(func, (memout,timeout))
        return


    def remove_tool(self, tool):
        """Removes the given tool instance (and possibly the row
        containing it if empty after the removal)"""
        for row in self.tools:
            if row[1] is not None:
                if row[1] == tool: 
                    self.tools.remove(row.iter)
                    return
                pass
            else:                                 
                for children in row:
                    if children[1] == tool: 
                        self.tools.remove(children.iter)
                        if len(row) == 0: self.tools.remove(row.iter)
                        return
                    pass
                pass
            pass

        return       


    def cleanup(self):
        """Removes all tools that have been killed, that are dead, etc."""
        # RC: FIXME (groups have to be taken into account)
        for row in self.tools:
            if row[1] is None or not row[1].is_alive(): self.tools.remove(row.iter)
            pass
        return            

    pass # end of class 
# ----------------------------------------------------------------------

