# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import xml.dom.minidom

from slim_compiler.src.smv import api 
import src.models.trace  #import TraceModel

#DEFAULT_TRACE_HANDLER = tr.TraceModel

# This module contains classes and functions to be used as helpers
# in the interaction with nusmv

class NuSMVTraceReader :

    def __init__(self):
        return
    
    from utils.log import log_icd
    @log_icd
    def parse(self, filename, trace_handler = None ):
        """Parses the trace in xml format contained in the given file.
        @filename the name of the xml file to parse
        @ param trace_handler: a class that can handle a single trace. 
        It must be a traceModel or a derived class. 
        @return a new filled TraceModel instance, or None if no trace is
        found"""
		# for testing purposes

        root = xml.dom.minidom.parse(filename)
        node = root.documentElement
        if node.tagName != "counter-example": return None

        #
        if not trace_handler:
            trace = src.models.trace.TraceModel()
        else:
            trace = trace_handler()

        steps = node.getElementsByTagName("node")
        loops = node.getElementsByTagName("loops")

        # collects steps info
        names = {}
        names_list = { 'state' : [], 
                       'combinatorial' : [],
                       'input' : [],
                       }  # map of strip type, and list of names
        for step in steps:
            for field in names_list: 
                values = step.getElementsByTagName(field)
                for val in values:
                    for var in val.getElementsByTagName("value"):
                        name = var.getAttribute("variable")
                        if name not in names:
                            names[name] = []
                            names_list[field].append(name)
                            pass
                        
                        val = ""
                        for text_child in var.childNodes:
                            if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                                val += text_child.data
                                pass
                            pass
                        
                        names[name].append(val)
                        pass
                    pass
                pass
            pass

        for field in names_list:
            for name in names_list[field]:
                stripe_container = trace.get_stripe_container()
                stripe = stripe_container(name, field)
                slim_name = api.convert_smv_id_to_slim(stripe.get_name(), 
                                                       can_be_enum_literal=False)

                boolean = True
                blist = ['true','false'] #list of valid boolean values

                for step in names[name]:
                    to_add = api.convert_smv_const_to_slim(step)
                    if to_add and to_add.lower() not in blist: boolean = False
                    stripe.add_value(to_add)
                    pass
                #set if the stripe is a boolean
                stripe.set_boolean(boolean)
                # NAMING convention to decide wether or not a stripe is an error stripe
                if slim_name and src.models.trace.TraceModel.ERROR_STRIPE in slim_name:
                    stripe.set_error(True)
                else:
                    stripe.set_error(False)
                    pass
                
                iter = trace.append(None)
                trace.set_value(iter, 0, stripe)
                trace.set_value(iter, 1, slim_name)           
                
                #DT
                if ('data_#delta' == stripe.get_name()
                    or 't_#delta' == stripe.get_name()):
                    #slim_name = 'delta'
                    trace.set_delta(iter)
                #end
                pass
            pass

        # handling of loops
        loops_strs = ""
        for loop in loops:
            val = ""
            for text_child in loop.childNodes:
                if text_child.nodeType == xml.dom.Node.TEXT_NODE:
                    val += text_child.data
                    pass
                pass            

            val = val.strip()
            if val: loops_strs += val + " "
            pass

        loops_strs = loops_strs.replace(",", " ").strip()
        for loop in loops_strs.strip().split(" "):
            if loop != "": trace.add_loop_at_step(int(loop))
            pass
        
        pass
                
        return trace

    pass # end of class
