# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

from optparse import OptionParser
import os

import options
from utils import log

# This classs is a simplified version of the runscript parser
class GUIOptions(object):
    def __init__(self):
        self.opt_parser = OptionParser("usage: %prog [options] [slim-files]",
                                       version="%prog " + ".".join(map(str, options.PROG_VER)))

        self.register_options(self.opt_parser)       # provide possible options

    def parse(self):
        self.parse_options(options)                  # now it is time to parse the options
        self.check_options(self.opt_parser, options) # check option values

    def register_options(self, opt_parser):
        opt_parser.add_option("-f", "--file", dest="BASE_FILE_NAME",
                              help="Write output to files with BASE_FILE_NAME base name. Default value is the first input file without extension")

        opt_parser.add_option("-d", "--dir", dest="OUT_DIR_NAME",
                              help="Create output/logging files in the specified directory")

        opt_parser.add_option("-l", "--level", dest="LOGGING_SYS_LEVEL", 
                              help="Set minimum system logging level")

        opt_parser.add_option("-e", "--logfile", dest="LOGGING_FNAME",
                              help="Write logging output to FILE")

        opt_parser.add_option("-i", "--injection", dest="FAULT_INJECTION_FILE_NAME",
                              help="Fault injection file")

        opt_parser.add_option("-p", "--property", dest="PROPERTIES_FILE_NAME",
                              help="Properties file")

        opt_parser.add_option("-m", "--mission", dest="MISSION_FILE_NAME",
                              help="Mission file");

        opt_parser.add_option("-t", "--tfpg", dest="TFPG_FILE_NAME",
                              help="Tfpg file");

        opt_parser.add_option("-a", "--slim_assoc", dest="TFPG_ASSOC_FILE_NAME",
                              help="Slim associations file");

        def __check_wordsize(option, opt_str, value, parser):
            if value <= 0: parser.error("Word size must be greater than 0.")
            if value > 64: parser.error("Word size must be less than or equal to 64.")
            setattr(parser.values, option.dest, value)
            return

        opt_parser.add_option("-w", "--wordsize", 
                              action="callback", callback=__check_wordsize, 
                              dest="SMV_WORD_SIZE", type="int",
                              help="Size of words used to represent int type")

    def parse_options(self, options):
        # sets the given options inside module options
        (opts, args) = self.opt_parser.parse_args()
        for opt,val in opts.__dict__.iteritems():
            if val is not None:
                if hasattr(options, opt):
                    setattr(options, opt, val)
                pass
            pass

        # the input filenames/positional arguments (alias)
        options.SLIM_FILE_NAMES = args
        return

    def check_options(self, opt_parser, options):
        # set the path to output files and logging files
        if options.OUT_DIR_NAME:
            if not os.path.exists(options.OUT_DIR_NAME):
                opt_parser.error("Fatal Error: directory for output files does not exist : " +
                                 options.OUT_DIR_NAME)
                pass
            pass

        # Note: not checking for existing files, GUI will display message to user

        # Make sure logging level is valid
        try: log.get_level_from_string(options.LOGGING_SYS_LEVEL)
        except ValueError as e: opt_parser.error(str(e))

        return
