# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by GMV Aerospace and Defence  
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import gtk
import os.path
from gtkmvc import View
import src.utils.globals
from balloon import Balloon


class ConfigurationsView (View):
    """Represents the View of Configurations."""

    glade = os.path.join(src.utils.globals.GLADE_DIR, "mission_configurations.glade")
    top = "frame_conf"

pass # end of ConfigurationsView class
# ----------------------------------------------------------------------


class ConfigurationDialogView (View):
    """Represents the Dialog Class of Configuration"""

    glade = os.path.join(src.utils.globals.GLADE_DIR, "mission_configurations.glade")
    top = "confirm_configuration"


    def __init__(self, loading_configurations=False):
        """Constructor Method"""
        View.__init__(self)
        self.loading_configurations = loading_configurations
        #This window is used to display errors in a tooltip-like fashion
        self.balloon = Balloon(self[ConfigurationDialogView.top])
        if self.loading_configurations:
            self.get_top_widget().set_property("visible", False)
        pass
    

    def run(self):
        """Runs the dialog, destroys the window and returns the
           response. You can use this method ony once per instance.
        """
        res = self.get_top_widget().run()
        self.get_top_widget().destroy()
        return res


    def change_color_and_balloon(self, entry, error_lst):
        """Manages the entries colors and displays a balloon with 
           error messages adjacent to the currently edited entry.
           @param   entry:  The reference to an entry that contains the input.
           @type    entry:  gtk.Entry
           @param   error_lst: List of errors, empty when there are none.
           @param   error_lst: list
        """
        if self.loading_configurations:
            return
        self.balloon.change_color_and_balloon(entry, error_lst)
        pass

pass # end of ConfigurationDialogView class
# ----------------------------------------------------------------------
