# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


# The view for slim models and fault injection

import os.path
import gtk
import gobject

from gtkmvc import View

from balloon import Balloon
import src.utils.globals

from property_management.src.constants import PlaceholderType

class PropertiesView (View):
    glade = os.path.join(src.utils.globals.GLADE_DIR, "properties_v2.glade")

    # This name has to be changed after editing accordingly the
    # glade file:
    top = "top_level_properties"
    
    def __init__(self):
        View.__init__(self)

        self.ctrl = None
        
    pass # end of class

class PropertyDialogView (View):   
    glade = os.path.join(src.utils.globals.GLADE_DIR, "properties_v2.glade")
    top = "edit_property_dialog"
    
    def __init__(self, properties_model, loading_properties=False, parent=None):
        View.__init__(self)
        
        self.properties_model = properties_model
        self.loading_properties = loading_properties
        
        # this window is used to display errors in a tooltip-like fashion
        self.balloon = Balloon(self[PropertyDialogView.top])
        
        #set the base color of the textview such that it looks non-editable
        #to the user, i.e. grey (of course, it IS non-editable)
        base = self[PropertyDialogView.top].get_style().bg
        # does 2 correspond to gtk.STATE_PRELIGHT or is it a magic number
        # that accidently gives the right color in this case?
        self['textview'].modify_base(gtk.STATE_NORMAL, base[2])
        
        if self.loading_properties:
            self.get_top_widget().set_property("visible",False)

        if  parent:             
            topWindow = parent.get_top_widget().get_ancestor(gtk.Window)
            self.get_top_widget().set_transient_for(topWindow)
            
    def run(self):
        """Runs the dialog, destroys the window and returns the
        response. You can use this method ony once per instance"""
        res = self.get_top_widget().run()
        self.get_top_widget().destroy()
        return res
    
    def property_view_story(self, pattern):
        self.balloon.hide_all()
        self.entries = {} # dict to store pairs of the form
                          # entry : (bool, None or implementation name)
                          # indicating whether the content of an entry is
                          # syntactically valid
        # The attribute self.pattern is set such that the CONTROLLER knows what
        # pattern we are dealing with right now.
        # This is important for instatiating patterns in order to obtain 
        # properties - that is what "SAVE" does
        self.pattern = pattern
        self['property_confirm'].set_sensitive(False)
        #suggest a default property name and make sure the entry is editable
        self['name_entry'].set_text(str(pattern) +" "+\
                                    str(self.properties_model.property_counter))
        self['name_entry'].set_sensitive(True)
        self['name_entry'].connect("activate", self.ctrl.on_entry_activated)
        self['name_entry'].connect("changed", self.ctrl.on_name_entry_changed)
        self.entries[self['name_entry']] = (True, None)
        #show apt story
        pattern.refresh()
        self['textview'].set_buffer(pattern.text_buffer)
        assert self.ctrl is not None
        for placeholder, (anchor, type) in zip(pattern.placeholders, pattern.anchors):
            input = gtk.Entry()
            #the name is used to store the TYPE of the placeholder which will
            #be entered in this entry
            input.set_name(placeholder.id+"."+type)
            input.set_has_frame(False)
            if int(type) is PlaceholderType.STATE_FORMULA:
                input.set_width_chars(30)
            else:
                input.set_width_chars(8)
            input.connect("changed", self.ctrl.on_entry_changed)
            self.entries[input] = (False, None)
            self['textview'].add_child_at_anchor(input, anchor)
            input.connect("activate", self.ctrl.on_entry_activated)
        #TODO: grab focus does not work
        pattern.anchors[0][0].get_widgets()[0].grab_focus()
        self['textview'].show_all()
        pass

    def change_color_and_balloon(self, entry, error_lst):
        """
        This method manages the entries' colors and displays a balloon with 
        error messages adjacent to the currently edited entry.
        
        @param   entry:  The reference to an entry that contains the input.
        @type    entry:  gtk.Entry
        @param   error_lst: List of errors, empty when there are none.
        @param   error_lst: list
        """
        if self.loading_properties:
            return
        self.balloon.change_color_and_balloon(entry, error_lst)
        pass
    
    pass # end of class
