# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by GMV Aerospace and Defence  
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

class PhaseAlreadyExistsError(Exception):
    """Represents the PhaseAlreadyExistsError Class"""

    def __init__(self, name):
        """Constructor Method.
           @param   name:  The name of the Phase.
	   @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.name:  The name of the Phase.
	   @rtype    self.name:  string
        """
        return "A Phase with the name \"" + self.name + "\" already exists!"

pass # end of PhaseAlreadyExistsError class
# -----------------------------------------------------------------------


class OpModeAlreadyExistsError(Exception):
    """Represents the OpModeAlreadyExistsError Class"""

    def __init__(self, name):
        """Constructor Method.
           @param   name:  The name of the OpMode.
	   @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.name:  The name of the OpMode.
	   @rtype    self.name:  string
        """
        return "A Op-Mode with the name \"" + self.name + "\" already exists!"

pass # end of OpModeAlreadyExistsError class
# -----------------------------------------------------------------------


class ConfigurationAlreadyExistsError(Exception):
    """Represents the ConfigurationAlreadyExistsError Class"""

    def __init__(self, configuration_id):
        """Constructor Method.
           @param   configuration_id:  The id of the Configuration.
	   @type    configuration_id:  string
        """
        Exception.__init__(self)
        self.configuration_id = configuration_id
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.configuration_id:  The id of the Configuration.
	   @rtype    self.configuration_id:  string
        """
        return "A Configuration with the id \"" + self.configuration_id + "\" already exists!"

pass # end of ConfigurationAlreadyExistsError class
# -----------------------------------------------------------------------


class PhaseOpModeCombinationAlreadyExistsError(Exception):
    """Represents the PhaseOpModeCombinationAlreadyExistsError Class"""

    def __init__(self, phase_name, op_mode_name):
        """Constructor Method.
           @param   phase_name:  The name of the Phase.
	   @type    phase_name:  string
           @param   op_mode_name:  The name of the OpMode.
	   @type    op_mode_name:  string
        """
        Exception.__init__(self)
        self.phase_name = phase_name
        self.op_mode_name = op_mode_name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.phase_name:  The name of the Phase.
	   @rtype    self.phase_name:  string
           @return   self.op_mode_name:  The name of the OpMode.
	   @rtype    self.op_mode_name:  string
        """
        return "A Phase/Op-Mode Combination with \"" + self.phase_name + "\" ad Phase and \"" + self.op_mode_name + "\" as Op-Mode already exists!"

pass # end of PhaseOpModeCombinationAlreadyExistsError class
# -----------------------------------------------------------------------


class TargetAlreadyExistsError(Exception):
    """Represents the TargetAlreadyExistsError Class"""

    def __init__(self, target_id):
        """Constructor Method.
           @param   target_id:  The id of the Target.
	   @type    target_id:  string
        """
        Exception.__init__(self)
        self.target_id = target_id
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.target_id:  The id of the Target.
	   @rtype    self.target_id:  string
        """
        return "A Target with the id \"" + self.target_id + "\" already exists!"

pass # end of TargetAlreadyExistsError class
# -----------------------------------------------------------------------


class PhaseNotExistError(Exception):
    """Represents the PhaseNotExistError Class"""

    def __init__(self, phase_name):
        """Constructor Method.
           @param   phase_name:  The phase_name.
	   @type    phase_name:  string
        """
        Exception.__init__(self)
        self.phase_name = phase_name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.phase_name:  The phase_name.
	   @rtype    self.phase_name:  string
        """
        return "There is no Phase with name \"" + self.phase_name + "\"."

pass # end of PhaseNotExistError class
# -----------------------------------------------------------------------


class OpModeNotExistError(Exception):
    """Represents the OpModeNotExistError Class"""

    def __init__(self, op_mode_name):
        """Constructor Method.
           @param   op_mode_name:  The op_mode_name.
	   @type    op_mode_name:  string
        """
        Exception.__init__(self)
        self.op_mode_name = op_mode_name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.op_mode_name:  The op_mode_name.
	   @rtype    self.op_mode_name:  string
        """
        return "There is no Op-Mode with name \"" + self.op_mode_name + "\"."

pass # end of OpModeNotExistError class
# -----------------------------------------------------------------------


class InvalidNameError(Exception):
    """Represents the InvalidNameError Class"""

    def __init__(self, name):
        """Constructor Method.
           @param   name:  The name.
	   @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.name:  The name.
	   @rtype    self.name:  string
        """
        return "This name \"" + self.name + "\" is invalid."

pass # end of InvalidNameError class
# -----------------------------------------------------------------------


class NameAlreadyExistsError(Exception):
    """Represents the NameAlreadyExistsError Class"""

    def __init__(self, name):
        """Constructor Method.
           @param   name:  The name.
	   @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.name:  The name.
	   @rtype    self.name:  string
        """
        return "This name \"" + self.name + "\" already exists."

pass # end of NameAlreadyExistsError class
# -----------------------------------------------------------------------


class ExpressionNotCorrectError(Exception):
    """Represents the ExpressionNotCorrectError Class"""

    def __init__(self, expr):
        """Constructor Method.
           @param   expr:  The expression.
	   @type    expr:  string
        """
        Exception.__init__(self)
        self.expr = expr
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.expr:  The expression.
	   @rtype    self.expr:  string
        """
        return "The expression \"" + self.expr + "\" isn't correct.\n"

pass # end of ExpressionNotCorrectError class
# -----------------------------------------------------------------------


class FewAssociatedOpModesError(Exception):
    """Represents the FewAssociatedOpModesError Class"""

    def __init__(self, configuration_id):
        """Constructor Method.
           @param   configuration_id:  The configuration_id.
	   @type    configuration_id:  string
        """
        Exception.__init__(self)
        self.configuration_id = configuration_id
        return

    def __str__(self):
        """Gets an error Message in string format.
           @return   self.configuration_id:  The configuration_id.
	   @rtype    self.configuration_id:  string
        """ 
        return "The Configuration \"" + self.configuration_id + \
            "\" must have at least 1 associated Op-mode.\n"

pass # end of FewAssociatedOpModesError class
# -----------------------------------------------------------------------
