# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


class InvalidNameError(Exception):
    """Represents the InvalidNameError Class"""

    def __init__(self, name):
        """Constructor Method.
            @param   name:  The name.
            @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.name:  The name.
            @rtype    self.name:  string
        """
        return "This TFPG name \"" + self.name + "\" is invalid."

pass # end of InvalidNameError class
# -----------------------------------------------------------------------


class NameAlreadyExistsError(Exception):
    """Represents the NameAlreadyExistsError Class"""

    def __init__(self, name):
        """Constructor Method.
            @param   name:  The name.
            @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.name:  The name.
            @rtype    self.name:  string
        """
        return "This name \"" + self.name + "\" already exists."

pass # end of NameAlreadyExistsError class
# -----------------------------------------------------------------------


class ExpressionNotCorrectError(Exception):
    """Represents the ExpressionNotCorrectError Class"""

    def __init__(self, expr):
        """Constructor Method.
            @param   expr:  The expression.
            @type    expr:  string
        """
        Exception.__init__(self)
        self.expr = expr
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.expr:  The expression.
            @rtype    self.expr:  string
        """
        return "The expression \"" + self.expr + "\" is not correct.\n"

pass # end of ExpressionNotCorrectError class
# -----------------------------------------------------------------------


class FMNotCorrectError(Exception):
    """Represents the FMNotCorrectError Class"""

    def __init__(self, name):
        """Constructor Method.
            @param   name:  The name of the Node.
            @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.name:  The name of the Node.
            @rtype    self.name:  string
        """
        return "The Failure Mode Node \"" + self.name + "\" is not correct because it has the attribute isMonitored = True.\n"

pass # end of FMNotCorrectError class
# -----------------------------------------------------------------------


class TimeNotCorrectError(Exception):
    """Represents the TimeNotCorrectError Class"""

    def __init__(self, name, tMin, tMax):
        """Constructor Method.
            @param   name:  The name of the Edge.
            @type    name:  string
            @param   tMin:  The initial time of the Edge.
            @type    tMin:  float
           @param   tMax:  The final time of the Edge.
            @type    tMax:  float
        """
        Exception.__init__(self)
        self.name = name
        self.tMin = tMin
        self.tMax = tMax
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.name:  The name of the Edge.
            @rtype    self.name:  string
            @return   self.tMin:  The initial time of the Edge.
            @rtype    self.tMin:  float
            @return   self.tMax:  The final time of the Edge.
            @rtype    self.tMax:  float
        """
        return "In the Edge \"" + self.name + "\" the final time \"" + str(self.tMax) + "\" must be greater than the initial time \"" + str(self.tMin) + "\" or -1 (unbounded).\n"

pass # end of TimeNotCorrectError class
# -----------------------------------------------------------------------


class NodeNotInEdge(Exception):
    """Represents the NodeNotInEdge Class"""

    def __init__(self, node_name, edge_name):
        """Constructor Method.
            @param   node_name:  The name of the node.
            @type    node_name:  string
            @param   edge_name:  The name of the Edge.
            @type    edge_name:  string
        """
        Exception.__init__(self)
        self.node_name = node_name
        self.edge_name = edge_name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.node_name:  The name of the node.
            @rtype    self.node_name:  string
            @return   self.edge_name:  The name of the Edge.
            @rtype    self.edge_name:  string
        """
        return "The Node \"" + self.node_name + "\" is not a Node of the Edge \"" + self.edge_name + "\".\n"

pass # end of NodeNotInEdge class
# -----------------------------------------------------------------------


class NodeBigActivationTime(Exception):
    """Represents the NodeBigActivationTime Class"""

    def __init__(self, node):
        """Constructor Method.
            @param   node:  The name of the Node.
            @type    node:  string
        """
        Exception.__init__(self)
        self.node = node
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.node:  The name of the Node.
            @rtype    self.node:  string
        """
        return "The activation time of the Node \"" + self.node + "\" is greater than the global time"

    pass # end of NodeBigActivationTime class
# -----------------------------------------------------------------------


class EdgeBigActivationTime(Exception):
    """Represents the EdgeBigActivationTime Class"""

    def __init__(self, edge):
        """Constructor Method.
            @param   edge:  The name of the Edge.
            @type    edge:  string
        """
        Exception.__init__(self)
        self.edge = edge
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.edge:  The name of the Edge.
            @rtype    self.edge:  string
        """
        return "The activation time of the Edge \"" + self.edge + "\" is greater than the global time"

pass # end of EdgeBigActivationTime class
# -----------------------------------------------------------------------


class NodeNegativeActivationTime(Exception):
    """Represents the NodeNegativeActivationTime Class"""

    def __init__(self, node):
        """Constructor Method.
            @param   node:  The name of the Node.
            @type    node:  string
        """
        Exception.__init__(self)
        self.node = node
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.node:  The name of the Node.
            @rtype    self.node:  string
        """
        return "Negative activation time for node \"" + self.node + "\"!"

pass # end of NodeNegativeActivationTime class
# -----------------------------------------------------------------------


class EdgeNegativeActivationTime(Exception):
    """Represents the EdgeNegativeActivationTime Class"""

    def __init__(self, edge):
        """Constructor Method.
            @param   edge:  The name of the Edge.
            @type    edge:  string
        """
        Exception.__init__(self)
        self.edge = edge
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.edge:  The name of the Edge.
            @rtype    self.edge:  string
        """
        return "Negative activation time for edge \"" + self.edge + "\"!"

pass # end of EdgeNegativeActivationTime class
# -----------------------------------------------------------------------


class FMNodeIsNotDestNode(Exception):
    """Represents the FMNodeIsNotDestNode Class"""

    def __init__(self, node_name, edge_name):
        """Constructor Method.
            @param   node_name:  The name of the node.
            @type    node_name:  string
            @param   edge_name:  The name of the Edge.
            @type    edge_name:  string
        """
        Exception.__init__(self)
        self.node_name = node_name
        self.edge_name = edge_name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.node_name:  The name of the node.
            @rtype    self.node_name:  string
            @return   self.edge_name:  The name of the Edge.
            @rtype    self.edge_name:  string
        """
        return "The Edge \"" + self.edge_name + "\" can not have the FM Node \"" + self.node_name + "\" as a Destination Node.\n"

pass # end of FMNodeIsNotDestNode class
# -----------------------------------------------------------------------


class NodeLoopError(Exception):
    """Represents the NodeLoopError Class"""

    def __init__(self, name):
        """Constructor Method.
            @param   name:  The name of the node.
            @type    name:  string
        """
        Exception.__init__(self)
        self.name = name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.name:  The name of the node.
            @rtype    self.name:  string
        """
        return "The Node \"" + self.name + "\" can not have self-loops.\n"

pass # end of NodeLoopError class
# -----------------------------------------------------------------------


class DiscrepancyWithoutIncomingEdge(Exception):
    """Represents the DiscrepancyWithoutIncomingEdge Class"""

    def __init__(self, discrepancy):
        """Constructor Method.
            @param   discrepancy:  The name of the discrepancy.
            @type    discrepancy:  string
        """
        Exception.__init__(self)
        self.discrepancy = discrepancy
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.discrepancy:  The name of the discrepancy.
            @rtype    self.discrepancy:  string
        """
        return "The Discrepancy \"" + self.discrepancy + "\" must have at least one incoming edge.\n"

pass # end of DiscrepancyWithoutIncomingEdge class
# -----------------------------------------------------------------------


class NotExistingModes(Exception):
    """Represents the NodeNotInEdge Class"""

    def __init__(self, modes_list):
        """Constructor Method.
           @param   modes_list:  The list of unknown modes.
	   @type    modes_list:  list of strings
        """
        Exception.__init__(self)
        self.modes_list = modes_list
        return

    def __str__(self):
        """Gets an error Message in string format.
        """
        return "The following values are not contained in the ModesList: %s" %(','.join(self.modes_list))

pass # end of NotExistingModes class
# -----------------------------------------------------------------------


class NotModesInTheEdge(Exception):
    """Represents the NotModesInTheEdge Class"""

    def __init__(self, edge_name):
        """Constructor Method.
            @param   edge_name:  The name of the Edge.
            @type    edge_name:  string
        """
        Exception.__init__(self)
        self.edge_name = edge_name
        return

    def __str__(self):
        """Gets an error Message in string format.
            @return   self.edge_name:  The name of the Edge.
            @rtype    self.edge_name:  string
        """
        return "At least one Activation Mode for the Edge \"" + self.edge_name + "\" must be specified.\n"

pass # end of NotModesInTheEdge class
# -----------------------------------------------------------------------


class NotExistingNode(Exception):
    """Represents the NotExistingNode Class"""

    def __init__(self, node_name, node_role):
        """Constructor Methods.
            @param  node_name: The name of the Node.
            @type   node_name: string
        """
        Exception.__init__(self)
        self.node_name = node_name
        self.node_role = node_role
        return


    def get_node_role(self):
        """Returns the role of the node, i.e. whether it is 
            a source node ('src') or a destination node ('dst'). 
            If nothing has been specified, it returns None
        """
        if self.node_role.lower() == "src":
            return "src"
        elif self.node_role.lower() == "dst":
            return "dst"
        else:
            return None


    def __str__(self):
        """Gets an error message in a string format.
           @return  self.node_name: the name of the Node.
           @type    self.node_name: string
        """
        return "The Node " + self.node_name + " does not exist.\n"

pass # end of NotExistingNode class
# -----------------------------------------------------------------------


class InvalidModeName(Exception):
    """Represents the InvalidModeName Class"""

    def __init__(self, mode_name, isNode=True):
        """Constructuor method.
           @param  mode_name: the name of the incorrect mode
           @type   mode_name: string
           @param  isNode: boolean saying whether another node has the same name
                           (if False, it means that another edge has the same name)
           @type   isNode: bool
        """
        Exception.__init__(self)
        self.mode_name = mode_name
        self.isNode = isNode
        return

    def __str__(self):
        """Gets and error meessage in a string format.
           @return  self.mode_name: the name of the mode (additionally, it says
                                    whether it conflicts with a node or with an edge
           @type    self.mode_name: string
        """
        return "There cannot be a mode called " + self.mode_name + " since there is " \
            "already %s with the same name." %("a node" if self.isNode else "an edge") 
