# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


"""This module provides some common constants for the property management
tool.
"""

import os.path


class Logic(object):
    """This class provides constants to identify a logic (like LTL, CTL, ...).
    """
    LTL = 1
    CTL = 2
    PCTL = 3
    CSL = 4
    MITL = 5
    
class PlaceholderType(object):
    """This class provides constants to identify the type of a placeholder.
    """
    STATE_FORMULA = 1
    PATH_FORMULA = 2
    TIME_BOUND = 3
    THRESHOLD = 4
    COMPARISON_OPERATOR = 5

# Define constants for the required XML namespace URIs.
FORMULAE_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/formulae"
PATTERNS_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/patterns"
PROPERTIES_NAMESPACE_URI="http://compass.informatik.rwth-aachen.de/properties"
LTL_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/ltl"
CTL_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/ctl"
MITL_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/mitl"
PCTL_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/pctl"
CSL_NAMESPACE_URI = "http://compass.informatik.rwth-aachen.de/csl"

# Define constants for the strings that can be used in XML tags to indicate
# the namespace.
FORMULAE_NAMESPACE = "{" + FORMULAE_NAMESPACE_URI + "}"
PATTERNS_NAMESPACE = "{" + PATTERNS_NAMESPACE_URI + "}"
PROPERTIES_NAMESPACE = "{" + PROPERTIES_NAMESPACE_URI + "}"
LTL_NAMESPACE = "{" + LTL_NAMESPACE_URI + "}"
CTL_NAMESPACE = "{" + CTL_NAMESPACE_URI + "}"
MITL_NAMESPACE = "{" + MITL_NAMESPACE_URI + "}"
PCTL_NAMESPACE = "{" + PCTL_NAMESPACE_URI + "}"
CSL_NAMESPACE = "{" + CSL_NAMESPACE_URI + "}"

# Define a dictionary to associate each logic with the corresponding XML
# namespace
logic_to_xml_namespace_dict = {Logic.LTL: LTL_NAMESPACE,
                               Logic.CTL: CTL_NAMESPACE,
                               Logic.PCTL: PCTL_NAMESPACE,
                               Logic.CSL: CSL_NAMESPACE,
                               Logic.MITL: MITL_NAMESPACE}

# Define the string that is used to represent the special value infinity for
# time bounds in logical formulae. This must be the same as defined in the
# XML Schema Definitions for formulae.
INFINITY = "infinity"

# Define the paths to the required XML schema documents.
PATH_TO_PROPERTY_MANAGEMENT_TOOL = os.path.abspath(
        os.path.join(os.path.dirname(__file__), ".."))
PROPERTIES_XSD_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                   "xml", "xsd", "properties.xsd")
FORMULAE_XSD_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                 "xml", "xsd", "formulae.xsd")
PATTERNS_XSD_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                 "xml", "xsd", "patterns.xsd")

# Define the path to the documents containing the pattern definitions.
CTL_PATTERNS_DOCUMENT_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                          "xml", "dwyer-ctl-patterns.xml")
LTL_PATTERNS_DOCUMENT_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                          "xml", "dwyer-ltl-patterns.xml")
MITL_PATTERNS_DOCUMENT_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                          "xml", "timed-ltl-patterns.xml")
CSL_PATTERNS_DOCUMENT_PATH = os.path.join(PATH_TO_PROPERTY_MANAGEMENT_TOOL,
                                          "xml", "grunske-csl-patterns.xml")

# Specify the logics that are supported by RAT.
SUPPORTED_RAT_LOGICS = [Logic.LTL]
