# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


"""This module contains exception classes related to formula translation.
"""

class FormulaTranslationError(Exception):
	"""The base class for all exceptions that are related to formula
	translation.
	"""
	pass


class UnexpectedXmlTagError(FormulaTranslationError):
	"""An error object indicating that the XML representation of the formula
	contains an XML element with an unexpected tag.
	
	This error might occur if the formula contains operators that are not
	supported by MRMC or NuSMV.
	"""
	
	def __init__(self, tag):
		"""Constructs an UnexpectedXmlTagError object indicating that the
		specified tag occurred unexpectedly in the XML representation of the
		formula.
		
		@param   tag: The tag that occurred unexpectedly.
		@type    tag: string
		"""
		self.tag = tag
		
	def __str__(self):
		"""Returns a human-readable string representation of the error.
		
		@return: A human-readable error message.
		@rtype:  string
		"""
		return ('The following XML tag occurred unexpectedly in the XML ' +
				'representation of the formula: %s\n' +
				'The XML element might represent a logical operator that ' +
				'is not supported by the target of the translation (MRMC ' +
				'or NuSMV).') % self.tag


class ViolatedRestrictionError(FormulaTranslationError):
	"""An error object indicating that a certain formula could not be
	translated from XML to the desired target format (MRMC or NuSMV) because
	it does not conform to the restrictions imposed by MRMC or NuSMV.
	"""
	pass
