import sys
sys.path.append('../..')

import unittest
import property_management.src.properties as properties_module
import property_management.src.api as properties_api

class Test(unittest.TestCase):
    def getFormulaeDocument(self,filename):
        properties_document = properties_module.parse_properties_document(filename)
        
        errors = properties_module.get_syntax_errors_in_document(properties_document)
        if len(errors) > 0:
            error_strings = map(str, errors)
            error_message = ("Errors in properties document:\n" +
                             "\n".join(error_strings))
            raise Exception(error_message)
        
        return properties_api.create_formulae_document_for_properties(properties_document)

    def testFormulaAsIntermediateString(self):
        formulaDocument = self.getFormulaeDocument('../../examples/fault_trees/sensorfilter_ft_properties.xml')
        formulasAsTree = map(properties_api.get_formula_as_intermediate_string,formulaDocument)
        self.assertTrue(formulasAsTree[0] == '((PU, 10, 50), (tt) (Top_Level_Event))')
        self.assertTrue(formulasAsTree[1] == '((PU, 0, 50), (tt) (&& (Top_Level_Event) (! (E6))))')
        self.assertTrue(formulasAsTree[2] == '((PU, 20, 100), (E2) (fault_cfg_4))')
        self.assertTrue(formulasAsTree[3] == '((PU, 0, 100), (tt) (! (fault_cfg_2)))')
        self.assertTrue(formulasAsTree[4] == '((PU, 0, 100), (! (fault_cfg_3)) (! (|| (! (fault_cfg_4)) (fault_cfg_3))))')
        self.assertTrue(formulasAsTree[5] == '((PU), (tt) (! (|| (! (E5)) ((PU, 0, 10), (tt) (Top_Level_Event)))))')
        formulaDocument = self.getFormulaeDocument('../../examples/fault_trees/sensorfilter_ft_lra_et_properties.xml')
        formulasAsTree = map(properties_api.get_formula_as_intermediate_string,formulaDocument)
        self.assertTrue(formulasAsTree[0] == '((ET), (&& (|| (E6) (E4)) (! (Top_Level_Event))))')
        self.assertTrue(formulasAsTree[1] == '((LRA), (&& (|| (E6) (E4)) (! (Top_Level_Event))))')

if __name__ == "__main__":
    unittest.main()