%==============================================================================
% PROSYD -- (prosyd.cls) Class  file
% A LaTeX class for creating PROSYD deleiverable documents
% Author: Marco Roveri <roveri@irst.itc.it>
%
% Copyright (C) 2004-2005 by Marco Roveri <roveri@irst.itc.it>
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the Perl Artistic License or the
% GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any
% later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% If you do not have a copy of the GNU General Public License write to
% the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
% MA 02139, USA.
%
% CVSId : $Id: prosyd.cls,v 1.1 2006/12/27 09:01:08 nusmv Exp $
%==============================================================================
\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{prosyd}[16/03/2005 PROSYD Class v0.9 by M. Roveri]
\typeout{PROSYD v0.9 Author: Marco Roveri roveri@irst.itc.it.}
\typeout{PROSYD v0.9 class built on top of standard class ``article.cls''.}
\typeout{So you can use all the class option of ``article.cls''.}
\typeout{Paper-size defaults to US-letter: use a4paper as option for A4 paper-size.}

% Required package for conditional customization
\RequirePackage{ifthen}

%
% PROSYD Class options
%
% draft : add DRAFT word in the title and in the page head
%
% plainbib : use \bibliographystyle{plain}, this is a default
%
% alphabib : use \bibliographystyle{alpha}
%
\newboolean{@pdraft}
\setboolean{@pdraft}{false}
\DeclareOption{draft}{\setboolean{@pdraft}{true}}
\newcommand{\prosydbibstyle}{plain}
\DeclareOption{plainbib}{\renewcommand{\prosydbibstyle}{plain}}
\DeclareOption{alphabib}{\renewcommand{\prosydbibstyle}{alpha}}
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{article}}
\ProcessOptions

% Loading the class article with forced options
\LoadClass[12pt,notitlepage,twoside,final]{article}

% %%%%%%%%%%%%%%%%%%%%
% Packages loaded
% %%%%%%%%%%%%%%%%%%%%
% Useful packages
\RequirePackage{amssymb}
\RequirePackage{xspace}
\RequirePackage[final]{graphicx}
\RequirePackage{url}
\RequirePackage{colortbl}
\RequirePackage{setspace}

% Packages to beautify PDF
\RequirePackage{times}
\RequirePackage[OT1]{fontenc}
\RequirePackage{type1cm}


% Force use of PS fonts but for mathematical symbols that appears to
% be broken.

% Similar for times.sty
\renewcommand{\sfdefault}{phv}
\renewcommand{\rmdefault}{ptm}
\renewcommand{\ttdefault}{pcr}

% Math
\DeclareSymbolFont{operators}   {OT1}{ptmcm}{m}{n}
\DeclareSymbolFont{letters}     {OML}{ptmcm}{m}{it}
\DeclareSymbolFont{symbols}     {OMS}{pzccm}{m}{n}
\DeclareSymbolFont{largesymbols}{OMX}{psycm}{m}{n}
\DeclareSymbolFont{bold}        {OT1}{ptm}{bx}{n}
\DeclareSymbolFont{italic}      {OT1}{ptm}{m}{it}

% An \omicron command, to fill the gap.
\DeclareMathSymbol{\omicron}{0}{operators}{`\o}

% Reduce the space around math operators

\thinmuskip=2mu
\medmuskip=2.5mu plus 1mu minus 1mu
\thickmuskip=4mu plus 1.5mu minus 1mu

% Larger scripts.

\DeclareMathSizes{5}{5}{5}{5}
\DeclareMathSizes{6}{6}{5}{5}
\DeclareMathSizes{7}{7}{5}{5}
\DeclareMathSizes{8}{8}{6}{5}
\DeclareMathSizes{9}{9}{7}{5}
\DeclareMathSizes{10}{10}{7.4}{6}
\DeclareMathSizes{10.95}{10.95}{8}{6}
\DeclareMathSizes{12}{12}{9}{7}
\DeclareMathSizes{14.4}{14.4}{10.95}{8}
\DeclareMathSizes{17.28}{17.28}{12}{10}
\DeclareMathSizes{20.74}{20.74}{14.4}{12}
\DeclareMathSizes{24.88}{24.88}{17.28}{14.4}

\DeclareFontSubstitution{OMS}{pzccm}{m}{n}

%%%% OT1pcr.fd
%
\DeclareFontFamily{OT1}{pcr}{\hyphenchar \font =-1}%
%
\DeclareFontShape{OT1}{pcr}{b}{n}{%
   <-> pcrb7t%
}{}%
%
\DeclareFontShape{OT1}{pcr}{b}{sc}{%
   <-> pcrbc7t%
}{}%
%
\DeclareFontShape{OT1}{pcr}{b}{sl}{%
   <-> pcrbo7t%
}{}%

%%%% pslatex's condenced courier
\DeclareFontShape{OT1}{pcr}{m}{n}{%
   <-> pcrr7tn%
}{}%
%
\DeclareFontShape{OT1}{pcr}{m}{sc}{%
   <-> pcrrc7t%
}{}%
%
\DeclareFontShape{OT1}{pcr}{m}{sl}{%
   <-> pcrro7t%
}{}%

%
\DeclareFontShape{OT1}{pcr}{bx}{n}{<->ssub * pcr/b/n}{}%
\DeclareFontShape{OT1}{pcr}{bx}{sc}{<->ssub * pcr/b/sc}{}%

\DeclareFontShape{OT1}{pcr}{bx}{sl}{<->ssub * pcr/b/sl}{}%
\DeclareFontShape{OT1}{pcr}{b}{it}{<->ssub * pcr/b/sl}{}%
\DeclareFontShape{OT1}{pcr}{bx}{it}{<->ssub * pcr/b/it}{}%
\DeclareFontShape{OT1}{pcr}{m}{it}{<->ssub * pcr/m/sl}{}%
%%%% OT1phv.fd
\DeclareFontFamily{OT1}{phv}{}

\DeclareFontShape{OT1}{phv}{b}{n}{
   <-> s * [.9] phvb7t
}{}

\DeclareFontShape{OT1}{phv}{b}{sc}{
   <-> s * [.9] phvbc7t
}{}

\DeclareFontShape{OT1}{phv}{b}{sl}{
   <-> s * [.9] phvbo7t
}{}

\DeclareFontShape{OT1}{phv}{bc}{n}{
   <-> s * [.9] phvbrn7t
}{}

\DeclareFontShape{OT1}{phv}{bc}{sc}{
   <-> s * [.9] phvbcn7t
}{}

\DeclareFontShape{OT1}{phv}{bc}{sl}{
   <-> s * [.9] phvbon7t
}{}

\DeclareFontShape{OT1}{phv}{m}{n}{
   <-> s * [.9] phvr7t
}{}

\DeclareFontShape{OT1}{phv}{m}{sc}{
   <-> s * [.9] phvrc7t
}{}

\DeclareFontShape{OT1}{phv}{m}{sl}{
   <-> s * [.9] phvro7t
}{}

\DeclareFontShape{OT1}{phv}{mc}{n}{
   <-> s * [.9] phvrrn7t
}{}

\DeclareFontShape{OT1}{phv}{mc}{sc}{
   <-> s * [.9] phvrcn7t
}{}

\DeclareFontShape{OT1}{phv}{mc}{sl}{
   <-> s * [.9] phvron7t
}{}

\DeclareFontShape{OT1}{phv}{bx}{n}{<->ssub * phv/b/n}{}
\DeclareFontShape{OT1}{phv}{bx}{sc}{<->ssub * phv/b/sc}{}
\DeclareFontShape{OT1}{phv}{bx}{sl}{<->ssub * phv/b/sl}{}
\DeclareFontShape{OT1}{phv}{b}{it}{<->ssub * phv/b/sl}{}
\DeclareFontShape{OT1}{phv}{bx}{it}{<->ssub * phv/b/it}{}
\DeclareFontShape{OT1}{phv}{bc}{it}{<->ssub * phv/bc/sl}{}
\DeclareFontShape{OT1}{phv}{m}{it}{<->ssub * phv/m/sl}{}
\DeclareFontShape{OT1}{phv}{mc}{it}{<->ssub * phv/mc/sl}{}

%%%% OT1ptm.fd
\DeclareFontFamily{OT1}{ptm}{}

\DeclareFontShape{OT1}{ptm}{b}{n}{
   <-> ptmb7t
}{}

\DeclareFontShape{OT1}{ptm}{b}{sc}{
   <-> ptmbc7t
}{}

\DeclareFontShape{OT1}{ptm}{b}{it}{
   <-> ptmbi7t
}{}

\DeclareFontShape{OT1}{ptm}{b}{sl}{
   <-> ptmbo7t
}{}

\DeclareFontShape{OT1}{ptm}{m}{n}{
   <-> ptmr7t
}{}

\DeclareFontShape{OT1}{ptm}{m}{sc}{
   <-> ptmrc7t
}{}

\DeclareFontShape{OT1}{ptm}{m}{it}{
   <-> ptmri7t
}{}

\DeclareFontShape{OT1}{ptm}{m}{sl}{
   <-> ptmro7t
}{}

\DeclareFontShape{OT1}{ptm}{bx}{n}{<->ssub * ptm/b/n}{}
\DeclareFontShape{OT1}{ptm}{bx}{sc}{<->ssub * ptm/b/sc}{}
\DeclareFontShape{OT1}{ptm}{bx}{it}{<->ssub * ptm/b/it}{}
\DeclareFontShape{OT1}{ptm}{bx}{sl}{<->ssub * ptm/b/sl}{}

%%%% T1pcr.fd
%
\DeclareFontFamily{T1}{pcr}{\hyphenchar \font =-1}%
%
\DeclareFontShape{T1}{pcr}{b}{n}{%
   <-> pcrb8t%
}{}%
%
\DeclareFontShape{T1}{pcr}{b}{sc}{%
   <-> pcrbc8t%
}{}%
%
\DeclareFontShape{T1}{pcr}{b}{sl}{%
   <-> pcrbo8t%
}{}%
%
%%%% pslatex's condensed courier
\DeclareFontShape{T1}{pcr}{m}{n}{%
   <-> pcrr8tn%
}{}%
%
\DeclareFontShape{T1}{pcr}{m}{sc}{%
   <-> pcrrc8t%
}{}%
%
\DeclareFontShape{T1}{pcr}{m}{sl}{%
   <-> pcrro8t%
}{}%
%
\DeclareFontShape{T1}{pcr}{bx}{n}{<->ssub * pcr/b/n}{}%
\DeclareFontShape{T1}{pcr}{bx}{sc}{<->ssub * pcr/b/sc}{}%
\DeclareFontShape{T1}{pcr}{bx}{sl}{<->ssub * pcr/b/sl}{}%
\DeclareFontShape{T1}{pcr}{b}{it}{<->ssub * pcr/b/sl}{}%
\DeclareFontShape{T1}{pcr}{bx}{it}{<->ssub * pcr/b/it}{}%
\DeclareFontShape{T1}{pcr}{m}{it}{<->ssub * pcr/m/sl}{}%
%
%%%% T1phv.fd

\DeclareFontFamily{T1}{phv}{}

\DeclareFontShape{T1}{phv}{b}{n}{
   <-> s * [.9] phvb8t
}{}

\DeclareFontShape{T1}{phv}{b}{sc}{
   <-> s * [.9] phvbc8t
}{}

\DeclareFontShape{T1}{phv}{b}{sl}{
   <-> s * [.9] phvbo8t
}{}

\DeclareFontShape{T1}{phv}{bc}{n}{
   <-> s * [.9] phvbrn8t
}{}

\DeclareFontShape{T1}{phv}{bc}{sc}{
   <-> s * [.9] phvbcn8t
}{}

\DeclareFontShape{T1}{phv}{bc}{sl}{
   <-> s * [.9] phvbon8t
}{}

\DeclareFontShape{T1}{phv}{m}{n}{
   <-> s * [.9] phvr8t
}{}

\DeclareFontShape{T1}{phv}{m}{sc}{
   <-> s * [.9] phvrc8t
}{}

\DeclareFontShape{T1}{phv}{m}{sl}{
   <-> s * [.9] phvro8t
}{}

\DeclareFontShape{T1}{phv}{mc}{n}{
   <-> s * [.9] phvrrn8t
}{}

\DeclareFontShape{T1}{phv}{mc}{sc}{
   <-> s * [.9] phvrcn8t
}{}

\DeclareFontShape{T1}{phv}{mc}{sl}{
   <-> s * [.9] phvron8t
}{}

\DeclareFontShape{T1}{phv}{bx}{n}{<->ssub * phv/b/n}{}

\DeclareFontShape{T1}{phv}{bx}{sc}{<->ssub * phv/b/sc}{}
\DeclareFontShape{T1}{phv}{bx}{sl}{<->ssub * phv/b/sl}{}
\DeclareFontShape{T1}{phv}{b}{it}{<->ssub * phv/b/sl}{}
\DeclareFontShape{T1}{phv}{bx}{it}{<->ssub * phv/b/it}{}
\DeclareFontShape{T1}{phv}{bc}{it}{<->ssub * phv/bc/sl}{}
\DeclareFontShape{T1}{phv}{m}{it}{<->ssub * phv/m/sl}{}
\DeclareFontShape{T1}{phv}{mc}{it}{<->ssub * phv/mc/sl}{}

%%%% T1ptm.fd

\DeclareFontFamily{T1}{ptm}{}

\DeclareFontShape{T1}{ptm}{b}{n}{
   <-> ptmb8t
}{}

\DeclareFontShape{T1}{ptm}{b}{sc}{
   <-> ptmbc8t
}{}

\DeclareFontShape{T1}{ptm}{b}{it}{
   <-> ptmbi8t
}{}

\DeclareFontShape{T1}{ptm}{b}{sl}{
   <-> ptmbo8t
}{}

\DeclareFontShape{T1}{ptm}{m}{n}{
   <-> ptmr8t
}{}

\DeclareFontShape{T1}{ptm}{m}{sc}{
   <-> ptmrc8t
}{}

\DeclareFontShape{T1}{ptm}{m}{it}{
   <-> ptmri8t
}{}

\DeclareFontShape{T1}{ptm}{m}{sl}{
   <-> ptmro8t
}{}

\DeclareFontShape{T1}{ptm}{bx}{n}{<->ssub * ptm/b/n}{}
\DeclareFontShape{T1}{ptm}{bx}{sc}{<->ssub * ptm/b/sc}{}
\DeclareFontShape{T1}{ptm}{bx}{it}{<->ssub * ptm/b/it}{}
\DeclareFontShape{T1}{ptm}{bx}{sl}{<->ssub * ptm/b/sl}{}

\normalfont

% %%%%%%%%%%%%%%%%%%%%
% Generic settings
% %%%%%%%%%%%%%%%%%%%%
%
% Figure/Tables positioning
\def\textfraction{0.0}
\def\topfraction{1.0}
\def\bottomfraction{0.0}
\def\totalnumber{1}
% default position for tables/figures
\def\fps@table{hbp}
\def\fps@figure{hbp}

\def\@xxxpt{32.99}
\def\@xviiipt{17.99}
\renewcommand\normalsize{%
   \@setfontsize\normalsize\@xipt{11.6}%
   \abovedisplayskip 11\p@ \@plus3\p@ \@minus6\p@
   \abovedisplayshortskip \z@ \@plus3\p@
   \belowdisplayshortskip 6.5\p@ \@plus3.5\p@ \@minus3\p@
   \belowdisplayskip \abovedisplayskip
   \let\@listi\@listI}
\renewcommand{\huge}{\@setfontsize\huge\@xxvpt{30}}
\renewcommand{\Huge}{\@setfontsize\Huge\@xxxpt{30}}
\newcommand{\sectionstarsize}{\@setfontsize\large\@xviiipt{20}}

% %%%%%%%%%%%%%%%%%%%%
% redefinition of the *section/*paragraph commands
% %%%%%%%%%%%%%%%%%%%%
\newcommand{\sectionfont}{\sffamily}
\newcommand{\subsectionfont}{\sffamily}
\newcommand{\subsubsectionfont}{\sffamily}
\newcommand{\paragraphfont}{\sffamily}
\newcommand{\subparagraphfont}{\sffamily}
\newenvironment{prosydquote}%
{\list{}{\setlength{\leftmargin}{1.7cm}%
\setlength{\rightmargin}{0cm}}%
\item\relax}
{\endlist}
\newcounter{prosydcnt}
\setcounter{prosydcnt}{0}
%
% SECTION
%
\let\@oldsection\section
\renewcommand{\section}[1]{%
%\refstepcounter{section}%
\stepcounter{section}%
%\numberline{\thesection}
\ifthenelse{\(\value{prosydcnt} = 1\)}
{\end{prosydquote}\vskip -3.5\@oldtopsep\end{flushright}\setcounter{prosydcnt}{0}\ifthenelse{\value{page} = 1}{}{\cleardoublepage}}{}
\sectionmark{#1}\addtocontents{toc}{\protect\contentsline{section}{{\numberline{\thesection} #1}}{\thepage}}%
\ \@oldsection*{\sectionfont \Huge \thesection\ \ #1}
\protected@edef\@currentlabel{\csname p@section\endcsname\csname thesection\endcsname}%
\setcounter{prosydcnt}{1}
\begin{flushright}\begin{prosydquote}
\vskip 5mm
}
%\newcommand{\sectionstar}{\@oldsection*}
\newcommand{\sectionstar}[1]{
\ifthenelse {\(\value{prosydcnt} = 1\)} {\end{prosydquote}\vskip -3.5\@oldtopsep\end{flushright}\setcounter{prosydcnt}{0}} {}
\addtocontents{toc}{\protect\contentsline{section}{{#1}}{\thepage}}%
\@oldsection*{\sectionstarsize \sectionfont #1}
\setcounter{prosydcnt}{1}
\begin{flushright}\begin{prosydquote}
}

%
% SUBSECTION
%
\let\@oldsubsection\subsection
\renewcommand{\subsection}[1]{%
\stepcounter{subsection}%
\ifthenelse {\(\value{prosydcnt} = 1\)} {\end{prosydquote}\vskip -3.5\@oldtopsep\end{flushright}\setcounter{prosydcnt}{0}} {}
\addtocontents{toc}{\protect\contentsline{subsection}{\numberline{\thesubsection} #1}{\thepage}}%
\ \@oldsubsection*{\subsectionfont \Large \rule{\@oldtextwidth}{0.1mm} \thesubsection\ \ #1}
\protected@edef\@currentlabel{\csname p@subsection\endcsname\csname thesubsection\endcsname}%
\setcounter{prosydcnt}{1}
\begin{flushright}\begin{prosydquote}
}
\newcommand{\subsectionstar}{\@oldsubsection*}

%
% SUBSUBSECTION
%
\let\@oldsubsubsection\subsubsection
\renewcommand{\subsubsection}[1]{%
\addtocontents{toc}{\protect\contentsline{subsubsection}{#1}{\thepage}}%
\ \@oldsubsubsection*{\subsubsectionfont \large #1}}
\newcommand{\subsubsectionstar}{\@oldsubsubsection*}

%
% PARAGRAPH
%
\let\@oldparagraph\paragraph
\renewcommand{\paragraph}[1]{%
\addtocontents{toc}{\protect\contentsline{paragraph}{#1}{\thepage}}%
\@oldparagraph*{\paragraphfont #1}}
\newcommand{\paragraphstar}{\@oldparagraph*}

%
% SUBPARAGRAPH
%
\let\@oldsubparagraph\subparagraph
\renewcommand{\subparagraph}[1]{%
\@oldsubparagraph*{\subparagraphfont #1}}
\newcommand{\subparagraphstar}{\@oldsubparagraph*}

\newcommand{\prosydbodystart}{\setcounter{prosydcnt}{0}}
\newcommand{\prosydbodyend}{\ifthenelse{\(\value{prosydcnt} = 1\)}{\end{prosydquote}\end{flushright}\setcounter{prosydcnt}{0}}{}}


% %%%%%%%%%%%%%%%%%%%%
% Table of contents stuff
% %%%%%%%%%%%%%%%%%%%%
\renewcommand*\l@section[2]{%
  \ifnum \c@tocdepth >\z@
    \addpenalty\@secpenalty
%    \addvspace{1.0em \@plus\p@}%
    \addvspace{1pt}%
%    \setlength\@tempdima{1.5em}%
    \setlength\@tempdima{1.2em}%
    \begingroup
      \parindent \z@ \rightskip \@pnumwidth
      \parfillskip -\@pnumwidth
      \leavevmode% \bfseries
      \advance\leftskip\@tempdima
      \hskip -\leftskip #1\nobreak \leaders\hbox{$\m@th
        \mkern \@dotsep mu\hbox{\normalfont .}\mkern \@dotsep
        mu$} \hfil \nobreak\hb@xt@\@pnumwidth{\hss #2}\par
    \endgroup
  \fi}
\setcounter{tocdepth}{3}
\renewcommand\@dotsep{0.3}
% Redefinition of the \tableofcontents command
\renewcommand\tableofcontents{%
\cleardoublepage
%\@oldsection*{\sectionfont \huge \contentsname}%
\ifthenelse {\(\value{prosydcnt} = 1\)} {\end{prosydquote}\vskip -3.5\@oldtopsep\end{flushright}\setcounter{prosydcnt}{0}} {}
\addtocontents{toc}{\protect\contentsline{section}{{\contentsname}}{\thepage}}%
\@oldsection*{\sectionfont \Huge \contentsname}
\setcounter{prosydcnt}{1}
\begin{flushright}\begin{prosydquote}
%\sectionstar{\contentsname}%
\vskip 5mm
\sectionmark{\contentsname}%
\begin{flushright}
%\begin{prosydquote}
{\small
\advance \parskip by -1pt
\advance \baselineskip by -5pt
\@starttoc{toc}%
\advance \parskip by 1pt
\advance \baselineskip by 5pt
}
%\end{prosydquote}
\end{flushright}
}

% %%%%%%%%%%%%%%%%%%%%
% List of Figures stuff
% %%%%%%%%%%%%%%%%%%%%
%
\renewcommand*\l@figure{\noindent\@dottedtocline{1}{1.5em}{2.3em}}
\long\def\@caption#1[#2]#3{%
  \par
  \addcontentsline{\csname ext@#1\endcsname}{#1}%
    { \csname fnum@#1\endcsname \ \ - \ \ {\ignorespaces #2}}%
  \begingroup
    \@parboxrestore
    \if@minipage
      \@setminipage
    \fi
    \normalsize
    \@makecaption{\csname fnum@#1\endcsname}{\ignorespaces #3}\par
  \endgroup}
\renewcommand\listoffigures{%
%\clearpage\@oldsection*{\sectionfont Table of Figures}%
\clearpage\sectionstar{Table of Figures}%
%\addtocontents{toc}{\protect\contentsline{section}{{Table of Figures}}{\thepage}}%
\vskip 5mm
\sectionmark{Table of Figures}%
{\small
\advance \baselineskip by -2pt
\advance \parskip by -3pt
\@starttoc{lof}%
\advance \parskip by 3pt
\advance \baselineskip by 2pt
}
}

% %%%%%%%%%%%%%%%%%%%%
% List of Tables stuff
% %%%%%%%%%%%%%%%%%%%%
%
\renewcommand*\l@table{\noindent\@dottedtocline{1}{1.5em}{2.3em}}
\renewcommand\listoftables{%
\clearpage
%\@oldsection*{\sectionfont List of Tables}%
\sectionstar{List of Tables}%
%\addtocontents{toc}{\protect\contentsline{section}{{\sectionfont List of Tables}}{\thepage}}%
\vskip 5mm
\sectionmark{List of Tables}%
{\small
\renewcommand{\baselinestretch}{0.3}
\advance \baselineskip by -2pt
\advance \parskip by -3pt
\@starttoc{lot}%
\advance \parskip by 3pt
\advance \baselineskip by 2pt
}
}

% %%%%%%%%%%%%%%%%%%%%
% Macro to indicate start of the deliverable body
% %%%%%%%%%%%%%%%%%%%%
\newcommand{\prosydbody}{
\ 
\ifodd \value{page}
\cleardoublepage
\else
\clearpage
\fi
\pagestyle{proshead}
\setcounter{page}{1}
\pagenumbering{arabic}
}


% %%%%%%%%%%%%%%%%%%%%
% Re definition of the figure and table environment to allow centering
% on the text instead of the page
% %%%%%%%%%%%%%%%%%%%%
%
% Figures
\renewenvironment{figure}[1][h]{%
\@float{figure}[#1]
\hspace{0.8cm}
\begin{minipage}{\textwidth}
\begin{center}
}
{
\end{center}
\end{minipage}
\end@float}
%
% Tables
\renewenvironment{table}[1][h]{%
\@float{table}[#1]
\hspace{0.8cm}
\begin{minipage}{\textwidth}
\begin{center}
}
{
\end{center}
\end{minipage}
\end@float}


% %%%%%%%%%%%%%%%%%%%%
% Redefinition of the bibliography command
% %%%%%%%%%%%%%%%%%%%%
%
% \prosydbibliography{bib-filenames}
\newcommand{\prosydbibliography}[1]{
\prosydbodyend
\bibliographystyle{\prosydbibstyle}
\bibliography{#1}
\prosydbodystart
}
\renewenvironment{thebibliography}[1]
{ \small \baselineskip 13pt
 \cleardoublepage
 \newpage
  \refstepcounter{section}
  \@oldsection*{\sectionfont \Huge \thesection\ \ \refname}
  \protected@edef\@currentlabel{\csname p@section\endcsname\csname thesection\endcsname}%
  \vskip 1cm
  \sectionmark{\refname}%
  \addtocontents{toc}{\protect\contentsline{section}{{\numberline{\thesection} \refname}}{\thepage}}
  \list{\@biblabel{\@arabic\c@enumiv}}%
       {\settowidth\labelwidth{\@biblabel{#1}}%
        \leftmargin\labelwidth
        %\advance\leftmargin 1cm
        \setlength{\rightmargin}{0cm}
        \advance\leftmargin\labelsep
        \@openbib@code
        \usecounter{enumiv}%
        \let\p@enumiv\@empty
        \renewcommand\theenumiv{\@arabic\c@enumiv}}%
  \sloppy
  \clubpenalty4000
  \@clubpenalty \clubpenalty
  \widowpenalty4000%
  \sfcode`\.\@m}
 {\def\@noitemerr
   {\@latex@warning{Empty `thebibliography' environment}}%
  \endlist
\newpage
}


% %%%%%%%%%%%%%%%%%%%%
% Definition of the PROSYD page header/footer
% %%%%%%%%%%%%%%%%%%%%
%
\newcommand{\@prosydshorttitle}{\@prosydtitle}
\newcommand{\prosydshorttitle}[1]{\renewcommand{\@prosydshorttitle}{#1}}
\newcommand{\ps@proshead}{%
\def\sectionmark##1{\markboth{{\bfseries ##1}}{{\bfseries ##1}}}%
%\renewcommand{\@oddhead}{\centerline{\mbox{\bfseries \footnotesize \@prosydshorttitle}}}%
\renewcommand{\@oddhead}{}%
\let\@mkboth\markboth%
\renewcommand{\@evenhead}{}%
\renewcommand{\@evenfoot}{\begin{minipage}{\@oldtextwidth}%
\begin{spacing}{0.8}%
%\singlespace%
\begin{tabular*}{\@oldtextwidth}{p{0.5\@oldtextwidth}@{\extracolsep\fill}p{0.05\@oldtextwidth}@{\extracolsep\fill}p{0.45\@oldtextwidth}p{0.1\@oldtextwidth}}
\begin{flushleft}{\sffamily \footnotesize \thepage \textbf{\ $\bullet$ \leftmark}}\end{flushleft} &  & \begin{flushright}\sffamily\footnotesize \textbf{\@prosydshorttitle} \end{flushright}\\%
\end{tabular*}%
\end{spacing}%
\end{minipage}%
}

\renewcommand{\@oddfoot}{\begin{minipage}{\@oldtextwidth}%
\begin{spacing}{0.8}%
%\singlespace%
\begin{tabular*}{\@oldtextwidth}{p{0.45\@oldtextwidth}p{0.00\@oldtextwidth}@{\extracolsep\fill}p{0.5\@oldtextwidth}@{\extracolsep\fill}}
 \begin{flushleft}\sffamily\footnotesize \textbf{\@prosydshorttitle} \end{flushleft} & & \begin{flushright}\sffamily\footnotesize \textbf{\leftmark\  $\bullet$} \thepage \end{flushright}\\%
\end{tabular*}%
\end{spacing}
\end{minipage}%
% {\bfseries \thepage\ $\bullet$\ \leftmark\hfil PROSYD Research Report}}%
}
}

% %%%%%%%%%%%%%%%%%%%%
% Definition of macro for introduce authors
% %%%%%%%%%%%%%%%%%%%%
\newcommand{\@prosydauthor}{Authors}
\newcommand{\prosydauthor}[1]{\renewcommand{\@prosydauthor}{#1}}
\newcommand{\prosydprocessauthors}{
 \noindent\begin{tabular}{l}
 \@for\@co:=\@prosydauthor\do{
   \ignorespaces\@co \\
 }
\end{tabular}
}

% %%%%%%%%%%%%%%%%%%%%
% Definition of the PROSYD first page
% %%%%%%%%%%%%%%%%%%%%
\newcommand{\@prosydtitle}{Title}
\newcommand{\prosydtitle}[1]{\renewcommand{\@prosydtitle}{#1}}
\newcommand{\@prosyddeliverable}{(Deliverable \#.\#/\#)}
\newcommand{\prosyddeliverable}[1]{\renewcommand{\@prosyddeliverable}{#1}}
\newcommand{\@prosyddeliverableleader}{XXXXX}
\newcommand{\prosyddeliverableleader}[1]{\renewcommand{\@prosyddeliverableleader}{#1}}
\newcommand{\@prosydduedate}{Month Day, Year}
\newcommand{\prosydduedate}[1]{\renewcommand{\@prosydduedate}{#1}}
\newcommand{\@prosydsubmissiondate}{Month Day, Year}
\newcommand{\prosydsubmissiondate}[1]{\renewcommand{\@prosydsubmissiondate}{#1}}

\newcommand{\prosydtitlepage}{
\advance \oddsidemargin by -0.05\paperwidth
\thispagestyle{empty}
\pagenumbering{roman}%
\begin{center}
  \begin{tabular}{p{0.45\textwidth}@{\hspace*{1.5cm}}p{3cm}}
    \includegraphics[width=7cm]{figs/prosyd} &
    \includegraphics[width=3cm]{figs/fp6}
  \end{tabular}
\end{center}

\vskip 10mm

\begin{center}
\bfseries \itshape FP6-IST-507219
\end{center}

\vskip 10mm

%\vspace{2cm}
{\bfseries
\begin{center}
  \begin{tabular}{c}
  {\Large \sffamily PROSYD:}\\[5mm]
  \itshape Property-Based System Design
  \end{tabular}
\end{center}
}

\vskip 5mm

Instrument: Specific Targeted Research Project

\vskip 5mm

Thematic Priority: Information Society Technologies

\vskip 10mm

\begin{minipage}[t]{0.9\@oldtextwidth}
  \begin{center}\large\bfseries
    \@prosydtitle\\
    (Deliverable \@prosyddeliverablenum{})
  \end{center}
\end{minipage}

\ifthenelse{\boolean{@pdraft}}{
\vskip 5mm
\centerline{\textbf{\Large -- DRAFT --}}
\let\@oldprosydshorttitle\@prosydshorttitle
\renewcommand{\@prosydshorttitle}{DRAFT -- \@oldprosydshorttitle{} -- DRAFT}
\vskip 5mm
}
{
\vskip 10mm
}

\begin{center}
\begin{tabular}{c}
Due date of deliverable: \@prosydduedate{}\\
Actual submission date: \@prosydsubmissiondate{}
\end{tabular}
\end{center}

\vskip 10mm

\begin{tabular}{p{0.45\textwidth}@{\hspace*{1.5cm}}p{4cm}}
  Start date of project: January 1, 2004 & Duration: Three years
\end{tabular}

\vskip 10mm

\begin{tabular}{l}
  Organisation name of lead contractor for this deliverable: \@prosyddeliverableleader{}
\end{tabular}

\vskip 10mm

\begin{tabular}{p{0.45\textwidth}@{\hspace*{1.5cm}}p{4cm}}
  \ & Revision \@prosydrevisionnum
\end{tabular}

\vskip 10mm
\ 
\vfill
{\footnotesize\renewcommand{\arraystretch}{1.1}
\noindent\begin{tabular}{|l|l|c|}\hline
\multicolumn{3}{|c|}{\bfseries Project co-funded by the European Commission within the Sixth Framework Programme (2002-2006)}\\\hline
\multicolumn{3}{|c|}{\bfseries Dissemination Level}\\\hline
\textbf{PU} & Public & \@prosydPU{}\\\hline
\textbf{PP} & Restricted to other programme participants (including the Commission Services) & \@prosydPP{}\\\hline
\textbf{RE} & Restricted to a group specified by the consortium (including the Commission Services) & \@prosydRE{}\\\hline
\textbf{CO} & Confidential, only for members of the consortium (including the Commission Services) & \@prosydCO{}\\\hline
\end{tabular}
}
\newpage
\advance \oddsidemargin by 0.05\paperwidth
\pagestyle{proshead}
\prosydnotices
\onehalfspace
}

% %%%%%%%%%%%%%%%%%%%%
% Dissemination level
% %%%%%%%%%%%%%%%%%%%%
\newcommand{\@prosyddisseminationlevel}{PU,PP,RE,CO}
\newcommand{\@prosydPU}{$\Box$}
\newcommand{\@prosydPP}{$\Box$}
\newcommand{\@prosydRE}{$\Box$}
\newcommand{\@prosydCO}{$\Box$}
\def\@ciempty{}
\newcommand{\prosyddisseminationlevel}[1]{
 \def\@coco{#1}
 \@for\@co:=\@prosyddisseminationlevel\do{
    \ifx\@ciempty\@co
      \typeout{END OF LIST}
    \else
      \ifx\@co\@coco
        \expandafter\def\csname @prosyd\@co\endcsname {$\boxtimes$}
      \fi
    \fi
 }
}

% %%%%%%%%%%%%%%%%%%%%
% Definition of the notices/contact page
% %%%%%%%%%%%%%%%%%%%%
%
% Contact
%
\newcommand{\@prosydcontact}{XXXXXXXXXXXXXXX}
\newcommand{\prosydcontact}[1]{\renewcommand{\@prosydcontact}{#1}}
%
% Deliverable number
%
\newcommand{\@prosyddeliverablenum}{XX.YY/ZZ}
\newcommand{\prosyddeliverablenum}[1]{\renewcommand{\@prosyddeliverablenum}{#1}}
%
% Year of the deliverable
%
\newcommand{\@prosydyear}{\the\year}
\newcommand{\prosydyear}[1]{\renewcommand{\@prosydyear}{#1}}
%
% Notices message
%
\newcommand{\prosydnotices}{{
\newcommand{\nspace}{\\[0.01\textheight]}
\parskip=0.01\textheight
\parindent=0pt

\textbf{Notices}

For information, contact \@prosydcontact{}.

This document is intended to fulfil the contractual obligations of the
PROSYD project concerning deliverable \@prosyddeliverablenum{} described in
contract number 507219.

The information in this document is provided "as is", and no guarantee
or warranty is given that the information is fit for any particular
purpose. The user thereof uses the information at its sole risk and
liability.

\copyright{} Copyright PROSYD \@prosydyear{}. All rights reserved.
}
\newpage
}


% %%%%%%%%%%%%%%%%%%%%
% Definition of the revisions environment
% %%%%%%%%%%%%%%%%%%%%
%
% Revision number of the document
%
\newcommand{\@prosydrevisionnum}{XX.YY}
\newcommand{\prosydrevisionnum}[1]{\renewcommand{\@prosydrevisionnum}{#1}}
% write the
\newcommand{\@prosydrevnumwriteaux}[1]{%
\def\@ccciempty{}%
\def\@cccciempty{#1}%
\ifx\@ccciempty\@cccciempty
 \@ccciempty
\else
% notice the use of \gdef to make the defined symbol global. Otherwise
% it is local and the default value is used.
 \protected@write\@auxout{}{\string\gdef\string\@prosydrevisionnum{#1}}
\fi
}
%
% Revision macro: \prosydrevision{version}{date}{description}{by}{affected}
%
\newcommand{\prosydrevision}[5]{#1 \@prosydrevnumwriteaux{#1} & #2 & #3 & #4 & #5 \\\hline}
%
% Revisions environment
%
\newenvironment{prosydrevisions}{%
\@oldsection*{Table of Revisions}
\addtocontents{toc}{\protect\contentsline{section}{{Table of Revisions}}{\thepage}}
%
\begin{footnotesize}
\renewcommand{\arraystretch}{1.2}
\begin{tabular}{|>{\columncolor[gray]{.94}}l|l|p{0.4\@oldtextwidth}|p{0.15\@oldtextwidth}|p{0.17\@oldtextwidth}|}\hline%
\rowcolor[gray]{.94}
\textbf{Version} & \textbf{Date} & \textbf{Description and reason} & \textbf{By} & \textbf{Affected sections} \\ \hline
}{%
\end{tabular}%
\end{footnotesize}
}

% Macros for using the better the page space 
\topmargin 0pt
\advance \topmargin by -\headheight
\advance \topmargin by -\headsep
%\textheight 8.9in
\setlength\@tempdima{\paperheight}
\addtolength\@tempdima{-2in}
%\addtolength\@tempdima{-1.5in}
\divide\@tempdima\baselineskip
\@tempcnta=\@tempdima
\setlength{\textheight}{\@tempcnta\baselineskip}
%\oddsidemargin 0pt
\oddsidemargin 45pt
%\evensidemargin \oddsidemargin
\evensidemargin \oddsidemargin
\advance \evensidemargin by -30pt
\marginparwidth 0.5in
\advance \textwidth by 17pt
%\textwidth 6.5in

% For users of A4 paper: The above values are suited for american
% 8.5x11in paper. If your output driver performs a conversion for A4
% paper, keep those values. If your output driver conforms to the TeX
% standard (1in/1in), then you should add the following commands to
% center the text on A4 paper:
% \advance\hoffset by -3mm  % A4 is narrower.
% \advance\voffset by  8mm  % A4 is taller.

\newlength{\@oldtextwidth}
\newlength{\@oldtextheight}
\newlength{\@oldeven}
\newlength{\@oldodd}
\newlength{\@oldparskip}
\newlength{\@oldtopsep}
\@oldtextwidth\textwidth
\@oldtextheight\textheight
\@oldeven\evensidemargin
\@oldodd\oddsidemargin
\@oldparskip\parskip
%\advance \textheight by 9mm
%\advance \footskip by 6mm
%\@oldtopsep\topsep

% Macros executed at the beginning/end of the document
%\AtBeginDocument{\pagestyle{empty}\prosydbodystart\baselineskip 1pt}
\AtBeginDocument{\pagestyle{empty}\prosydbodystart}
\AtEndDocument{\prosydbodyend}

\endinput
%%
%% End of file `prosyd.cls`.
