##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.controller import Controller
import utils.globals
import utils.dialogs

import gtk
import pango

# ----------------------------------------------------------------------
class CategoryCtrl (Controller):
    """the controller for the dialog that shows trace information.  If
    provided, cat_name must refer an existing category (that means
    that this dialog is to edit a category)"""

    def __init__(self, traces_model, cat_name=None, parent_ctrl=None):
        """Controller for the category editing dialog"""
        Controller.__init__(self, traces_model)

        self.cat_name = cat_name
        self.parent = parent_ctrl
        return

    def registerView(self, view):
        Controller.registerView(self, view)
        self.setup_view(view)
        return

    def setup_view(self, view):
        if not self.cat_name:
            view['button_ok'].set_sensitive(False)
            view.set_subtitle("Add a new Category")
            return

        view['entry_name'].set_text(self.cat_name)
        
        # searches the cat description if cat is already existing:
        cats = self.model.get_categories_model()
        iter = cats.get_iter_first()
        while iter:
            if cats.get_value(iter, 0) == self.cat_name: break
            iter = cats.iter_next(iter)
            pass

        assert iter
        buf = view['textview_notes'].get_buffer()
        buf.set_text(cats.get_value(iter, 1))

        editable = cats.get_value(iter, 3)
        view['button_ok'].set_sensitive(editable)
        view['entry_name'].set_editable(editable)
        view['textview_notes'].set_editable(editable)

        if editable: view.set_subtitle("Editing an existing Category")
        else: view.set_subtitle("View an existing Category")
        return
          

    def update_model(self):
        """@return True if success, False otherwise"""
        
        name = self.view['entry_name'].get_text()
        buf = self.view['textview_notes'].get_buffer()
        notes = buf.get_text(buf.get_start_iter(), buf.get_end_iter())        

        if self.cat_name:
            self.model.change_category(self.cat_name, name, notes)
            if self.cat_name != name: self.parent.category_renamed(self.cat_name, name)
        else:
            # creating a new category (if possible)
            if self.model.is_there_category(name):
                # invalid name!
                utils.dialogs.open_invalid_name_dialog()
                self.view['entry_name'].grab_focus()
                self.view['entry_name'].set_position(-1)
                return False
            else: self.model.add_category(name, notes, True)
            pass
        
        return True
    

    # ------------------------------------------------------------ #
    #                        GUI's signals                         #
    # ------------------------------------------------------------ #

    def on_entry_name_changed(self, entry):
        txt = entry.get_text()
        self.view.set_title(txt)
        self.view['button_ok'].set_sensitive(len(txt.strip())>0)
        return

    def on_button_ok_clicked(self, button):
        res = self.update_model()
        if res:
            # ok, closes the dialog
            self.model.unregisterObserver(self)
            if self.parent:
                self.parent.category_editing_terminated(self.cat_name)
                self.view.destroy()
            else: gtk.main_quit()                
            pass
        return

    def on_button_cancel_clicked(self, button):
        self.model.unregisterObserver(self)
        if self.parent:
            self.parent.category_editing_terminated(self.cat_name)
            self.view.destroy()
        else: gtk.main_quit()
        return

    def on_dialog_top_level_delete_event(self, window, event):
        self.model.unregisterObserver(self)
        if self.parent:
            self.parent.category_editing_terminated(self.cat_name)
            self.view.destroy()
        else: gtk.main_quit()
        return True

    pass # end of class
