##  ===========================================================================
##  Author: Ingo Pill <ipill@ist.tugraz.at>
##
##  Copyright (c) 2005 by Graz University of Technology
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import gtk.gdk
import gtk
import time

from gtkmvc.controller import Controller

from controllers.trace import TraceCtrlPS
from views.prop_simulation import PropertySimulationView
from views.prop_simulation import PropertySimulationAnalysisView

class PropertySimulationCtrl (Controller):

    """This is the controller of the frame about Property Simulation.
    """
    
    def __init__(self, model):
        """@param model It is assumed the model contains the
        properties and the assertions sub-models"""
        Controller.__init__(self, model)

        self.apCtrl = TraceCtrlPS(model.trcModelAp,"Signals",False, self)  # not editable on initalized state
        self.evlCtrl = TraceCtrlPS(model.trcModelEvl,"Evaluation",False, self)
	self.model = model
	self.model.addVisStubEndCall(self.enable_buttons)
	#self.model.addVisStubEndCall(self.setStatusMessage)
	self.model.addVisStubEndCall(self.updateView)
	# setup subview
	self.psaCtrl = PropertySimulationAnalysisCtrl(self.model)
        return
    
    def on_button_make_true_clicked(self, button):
        self.model.deriveProperty()
	if self.model.isPropertySet()==False: return
	self.model.status='Computing W'
        self.model.psVisStub.errorMessage=''
	self.setStatusMessage()
        self.model.generateWitness()
	self.disable_buttons()
        return
      
    def on_button_make_false_clicked(self, button): 
        self.model.deriveProperty()
	if self.model.isPropertySet()==False: return
	self.model.status='Computing CE'
        self.model.psVisStub.errorMessage=''
	self.setStatusMessage()
	self.model.generateCounterExample()
	self.disable_buttons()
        return

    def on_button_analysis_clicked(self, button):
        self.psaCtrl.showPSAView()
        return
    
    def disable_buttons(self):
	self.view['button_make_false'].set_sensitive(False)
	self.view['button_make_true'].set_sensitive(False)
        return
		
    def enable_buttons(self):
	self.view['button_make_false'].set_sensitive(True)
	self.view['button_make_true'].set_sensitive(True)
        return
	
    def setStatusMessage(self):
        self.view['Status_Message'].set_label('<big>'+self.model.status+'</big>')
	return
    
    def insertTimeStep(self, menu_item, pathinfo):
		
	if pathinfo[1].get_title()[:4]!='Step': 
	  return
	else:
	  step=int((pathinfo[1].get_title()[4:]),10)
	
	self.model.insertTimeStep(step)		
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
        return

    def removeTimeStep(self, menu_item, pathinfo):
					
	if pathinfo[1].get_title()[:4]!='Step': 
	  return
	else:
	  step=int((pathinfo[1].get_title()[4:]),10)

	ret=self.model.removeTimeStep(step)
	if ret==1:
	  self.apCtrl.update_view()
	  self.setStatusMessage()
          self.model.greyoutEVLView()
	  self.evlCtrl.update_view()
	  self.view.evl['treeview'].expand_all()	
	  return 1
	else:
	  self.view.showMessagePopup(gtk.MESSAGE_ERROR,'Error removing timestep; e.g. cannot remove the one and only infinite step.')
	  return 0


	
    def setSignalValueToTrue(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:4]!='Step': 
	  return
	else:
	  step=int((pathinfo[1].get_title()[4:]),10)
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name()
	self.model.setSignalValueToTrue(variable, step)	 
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
	return
    
    def setSignalValueToFalse(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:4]!='Step': 
	  return
	else:
	  step=int((pathinfo[1].get_title()[4:]),10)
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name() 
	self.model.setSignalValueToFalse(variable, step)
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
        return
	
    def setSignalValueToDNC(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:4]!='Step': 
	  return
	else:
	  step=int((pathinfo[1].get_title()[4:]),10)
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name() 
	self.model.setSignalValueToDNC(variable, step)
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
        return

    def setSignalToTrue(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:6]!='Signal': 
	  return
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name()
	self.model.setSignalToTrue(variable)	 
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
	return

    def setSignalToFalse(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:6]!='Signal': 
	  return
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name()
	self.model.setSignalToFalse(variable)	 
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
	return

    def setSignalToDNC(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:6]!='Signal': 
	  return
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name()
	self.model.setSignalToDNC(variable)	 
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
	return

    def fixSignalValues(self, menu_item, pathinfo):
        
	if pathinfo[1].get_title()[:6]!='Signal': 
	  return
	
	variable=self.apCtrl.model.get_value(self.apCtrl.model.get_iter(pathinfo[0]),0).get_name()
	self.model.fixSignalValues(variable)	 
	self.apCtrl.update_view()
	self.setStatusMessage()
        self.model.greyoutEVLView()
	self.evlCtrl.update_view()
	self.view.evl['treeview'].expand_all()
	return

	   
    
    def updateView(self):
        # update view 
	self.setStatusMessage()
	self.apCtrl.update_view()
	self.evlCtrl.update_view()
	self.apCtrl.set_editable(True)     # Now we can fix values; allow for editing
	self.evlCtrl.set_editable(False)   # Just for insanity; evaluation is set again as non-editable
	
	#if status is 'Vis Error' and there's an error message show popup.
	if ((self.model.status=='Vis Error') & (self.model.psVisStub.errorMessage!='')):
	  self.view.showMessagePopup(gtk.MESSAGE_ERROR,self.model.psVisStub.errorMessage)
	if ((self.model.trcDataEvl.dataRead!=True) and ((self.model.status=='Counterexample') or (self.model.status=='Witness'))):
	  self.view.showMessagePopup(gtk.MESSAGE_WARNING, 'The formula contains an operand (e.g. a \'*\') which is not supported by waveform evaluation. A correct trace could be derived however.')
	
	# reset column size of first column for synchronized view
	# doesn't work - reported size is always zero ... ?
	aptv=self.view.ap['treeview']
	evltv=self.view.evl['treeview']
	aptv.expand_all()
	evltv.expand_all()
        
	#aptv.hide_all()
	#aptv.show_all()
	#aptv.show_now()
        #evltv.hide_all()
	#evltv.show_all()
	#evltv.show_now()

	#apwidth=(aptv.get_column(0)).get_property("width")
	#evlwidth=(evltv.get_column(0)).get_property("width")
	
	#print 'width', apwidth,evlwidth
	
	#(aptv.get_column(0)).set_property("min-width", max(apwidth,evlwidth))
        #(evltv.get_column(0)).set_property("min-width", max(apwidth,evlwidth))
	#aptv.collapse_all()
	#evltv.collapse_all()
	
	
        return
	    
    def registerView(self, view):
        Controller.registerView(self, view)
	self.view.create_PSA_subview(self.psaCtrl)
      	return

    pass # end of class


class PropertySimulationAnalysisCtrl (Controller):

    def __init__(self, model):
        """ PSACtrl"""
        Controller.__init__(self, model)

	self.model = model
	return

    def registerView(self, view):
        Controller.registerView(self, view)
	return
	
    def on_PSAWindow_close_event(self, param1=None, param2=None):
        # called when either the ok button was clicked or the delete event was released
	# -> just hide but dont destroy (return value True does this trick )
        self.view.hide_view()
        return True
	
    def PSAViewNameFunc(self, col, cell, model, iter):
        name = model.get_value(iter, 0)
        cell.set_property("text", name)
	return
	
    def PSAViewDataFunc(self, col, cell, model, iter, (section, part)):
        data = model.get_value(iter, (1+(section*3)+part))
	cell.set_property("text", str(data)) 

	# for sections 0,1,2 (coverage) use different background colors
	# for section 3 set foreground color dependend on data
        if section==0: # finite stem -> define same color as for trace view
          cell.set_property("cell-background", "#C0C0C0")
	elif section==1: # infinite stem -> define same color as for trace view
	  cell.set_property("cell-background", "#909090")
	elif section==2: # sum -> define new aequidistant color
	  cell.set_property("cell-background", "#606060")	    
        # use different foreground colors for section 3 (dark green for DNC and red for required
	elif section==3:
	  if str(data)=='1':
	    cell.set_property("foreground", "dark red")
	  else:
  	    cell.set_property("foreground", "dark green")	  
        return
	
    def updatePSAView(self):
        tv = self.view['dataView']
	tv.set_model(self.model.psaTS)
        # remove contents
        for i in tv.get_columns():
	  tv.remove_column(i)
	
	#1. Signal Name
	rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn('Signal/Formula', rend)
        col.set_cell_data_func(rend, self.PSAViewNameFunc)
	col.set_property("expand", True)
	# for PS this column is not resizable, but the one which expands
        tv.append_column(col)
	
	# 2. Statistics
	for i in [0,1,2]:
	  for j in [0,1,2]:
	    rend = gtk.CellRendererText()
	    if j==2:
	      col = gtk.TreeViewColumn('C', rend)
	    else:
	      col = gtk.TreeViewColumn(str(j), rend)
	    col.set_cell_data_func(rend, self.PSAViewDataFunc, (i, j))
	    tv.append_column(col)
	    
	# 3. Request features for next trace
	for i in range(4):
	  if i==0: label='F(=1)'
	  elif i==1: label='G(=1)'
	  elif i==2: label='F(=0)'
	  elif i==3: label='G(=0)'
	  rend = gtk.CellRendererText()
	  col=gtk.TreeViewColumn(label, rend)
	  col.set_cell_data_func(rend, self.PSAViewDataFunc, (3, i))
	  tv.append_column(col)
	tv.expand_all()  
        return

    def showPSAView(self):
        self.updatePSAView()
        self.view.show_view()
	self.view['dataView'].expand_all()
        return
	
    def hidePSAView(self):
        self.view.hide_view()
        return

    def activatePSATracefeature(self, menu_item, pathinfo):
        feature=-1
	if pathinfo[1]==self.view['dataView'].get_column(10): feature=0
	if pathinfo[1]==self.view['dataView'].get_column(11): feature=1
	if pathinfo[1]==self.view['dataView'].get_column(12): feature=2
	if pathinfo[1]==self.view['dataView'].get_column(13): feature=3
	assert(feature!=-1)
        sig_name=self.view['dataView'].get_model().get_value(self.view['dataView'].get_model().get_iter(pathinfo[0]),0)
        sig_num=self.model.trcDataEvl.getSignalNumber(sig_name)
	self.model.activatePSATraceFeature(sig_num, feature)
	self.model.updatePSATS()
	self.updatePSAView()
        return True
	
    def deactivatePSATracefeature(self, menu_item, pathinfo):
        feature=-1
	if pathinfo[1]==self.view['dataView'].get_column(10): feature=0
	if pathinfo[1]==self.view['dataView'].get_column(11): feature=1
	if pathinfo[1]==self.view['dataView'].get_column(12): feature=2
	if pathinfo[1]==self.view['dataView'].get_column(13): feature=3
	assert(feature!=-1)
        sig_name=self.view['dataView'].get_model().get_value(self.view['dataView'].get_model().get_iter(pathinfo[0]),0)
        sig_num=self.model.trcDataEvl.getSignalNumber(sig_name)
	self.model.deactivatePSATraceFeature(sig_num, feature)
	self.model.updatePSATS()
	self.updatePSAView()
        return True
    
    def PSA_edit_menu(self, pathinfo):
        menu = gtk.Menu()

        item = gtk.MenuItem("Feature required for next trace")
        item.connect("activate", self.activatePSATracefeature, pathinfo )
        item.show()
        menu.append(item)

        item = gtk.MenuItem("Feature NOT required for next trace")
        item.connect("activate", self.deactivatePSATracefeature, pathinfo)
        item.show()
        menu.append(item)

        return menu

    def on_PSADataView_button_press_event(self, treeview, event):
                
	if (event.button == 3):
	    pathinfo = treeview.get_path_at_pos(int(event.x), int(event.y))
	    
	    # Todo - more columns are not editable with this menu :)	    
	    if (pathinfo is None): return False
	    # only columns 10-13 are editable 
	    if not ((pathinfo[1]==treeview.get_column(10)) |
	    (pathinfo[1]==treeview.get_column(11)) |
	    (pathinfo[1]==treeview.get_column(12)) |
	    (pathinfo[1]==treeview.get_column(13))): return False	    
            menu = self.PSA_edit_menu(pathinfo)
            menu.popup(None, None, None, event.button, event.time)
	    
            return True

        return False        

    pass # end of class
