##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2006 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.controller import Controller

from controllers.outcomes import OutcomeCtrl
from controllers.check_panel import CheckingPanelCtrl
from controllers.synthesis_panel import SynthesisPanelCtrl
import utils.globals
import gtk
import gobject


## ----------------------------------------------------------------------
class RealizabilityCtrl (Controller):
    
    """The main controller for realizability. This controller
    contains a sub-controllers for the checking control."""

    def __init__(self, model):
        """@param model It is assumed the model contains the
        check panel sub-model"""

        Controller.__init__(self, model)
        self.outcomes = OutcomeCtrl(model)
        self.check_panel = CheckingPanelCtrl(model.check_panel, self)
        self.synthesis_panel = SynthesisPanelCtrl(model.synthesis_panel, self)

        self.status2stock = {"yes" : ('stock-id', gtk.STOCK_YES),
                             "no" : ('stock-id', gtk.STOCK_NO),
                             "error"  : ('stock-id', 'rat-stock-aborted'),
                             }
        return

    def registerView(self, view):
        Controller.registerView(self, view)
        view.create_sub_views(self.outcomes, self.check_panel, self.synthesis_panel)

        tv = view['tv_result']
        tv.set_buffer(self.model.res_buffer)    
        self.setup_treeview()
        return

    def __aux_show_realizable(self, tv, cell, model, iter):
        st = model.get_value(iter, 1)
        if self.status2stock.has_key(st): cell.set_property(*self.status2stock[st])
        else: cell.set_property('stock-id', None)
        return

    def __aux_show_reqs(self, tv, cell, model, iter):
        reqs = model.get_value(iter, 2)
        if reqs: cell.set_property('text', ", ".join(reqs))
        else: cell.set_property('text', "")
        return


    def setup_treeview(self):
        tv = self.view['tv_results']
        tv.set_model(self.model)
        self.model.connect("row-inserted", self.on_row_inserted)
                           
        sel = tv.get_selection()
        sel.connect("changed", self.on_change_selection)
        
        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn("Num", rend, text=0)
        tv.insert_column(col, 0)

        col = gtk.TreeViewColumn("Realizable")

        # pixbuf
        rend = gtk.CellRendererPixbuf()
        col.pack_start(rend)
        col.set_cell_data_func(rend, self.__aux_show_realizable)

        # text
        rend = gtk.CellRendererText()
        col.pack_end(rend)
        col.set_attributes(rend, text=1)
        
        tv.insert_column(col, 1)

        rend = gtk.CellRendererText()
        col = gtk.TreeViewColumn("Requirements", rend)
        col.set_cell_data_func(rend, self.__aux_show_reqs)
        tv.insert_column(col, 2)

        return

    def check(self, check_reqs_cons):
        self.outcomes.auto_expand_after_time(2500)
        self.model.check_realizability(check_reqs_cons)
        return

    def synthesize(self, check_reqs_cons):
        self.outcomes.auto_expand_after_time(2500)
        self.model.do_synthesis(check_reqs_cons)
        return
    

    # ------------------------------------------------------------ #
    #                            Signals                           #
    # ------------------------------------------------------------ #    
    def on_change_selection(self, sel):
        iter = sel.get_selected()[1]
        if iter: self.model.show_realizable(iter)
        return
    
    def on_checking_timeout(self):
        if self.model.busy_level > 0:
            self.outcomes.view.show_outcomes(True)
            pass
        return False

    def on_row_inserted(self, treemodel, path, iter):
        """To track last added entry"""
        self.view['tv_results'].scroll_to_cell(path)
        return

    # ---------------------------------------- #
    #   Notifications for observed properties  #
    # ---------------------------------------- #
    def property_busy_level_change_notification(self, model, old, new):
        self.model.check_panel.busy_level = new
        self.model.synthesis_panel.busy_level = new
        return
    
    pass # end of class
