##  ===========================================================================
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.controller import Controller

import models.project
import utils.dialogs

import gtk

## ----------------------------------------------------------------------
class SynthesisPanelCtrl (Controller):
    
    """Controller for the synthesis panel view embedded within the
    realizability view"""

    def __init__(self, model, parent):
        Controller.__init__(self, model)
        self.parent = parent

        return

    def registerView(self, view):
        Controller.registerView(self, view)
        self.setup_view(view)
        return

    def setup_view(self, view):

        cb = view['cb_mode']

        view['cb_mode'].set_active(0)
        
        return

    # ------------------------------------------------------------ #
    #                            Signals                           #
    # ------------------------------------------------------------ #
    

    def on_cb_r1_toggled(self, cb):
        self.model.r1 = cb.get_active()
        return

    def on_cb_r2_toggled(self, cb):
        self.model.r2 = cb.get_active()
        return

    def on_cb_kill_toggled(self, cb):
        self.model.kill = cb.get_active()
        return

    def on_rb_one_hot_toggled(self, rb):
        self.model.one_hot = rb.get_active()
        return

    def on_rb_binary_toggled(self, rb):
        self.model.one_hot = ~rb.get_active()
        return

    def on_cb_mode_changed(self, cb):
        m = cb.get_model()
        mode = m.get_value(cb.get_active_iter(), 0)
        self.model.mode = mode

        # Deactivate reorder2 and kill if mode is one of the IrrSOP modes.
        if mode == 'BLIF-IRRSOP' or mode == 'VERILOG-IRRSOP':
            self.view['cb_r2'].set_active(False)
            self.view['cb_kill'].set_active(False)

        # Activate reorder2 and kill if mode is (normal) BLIF mode
        elif mode == 'BLIF':
            self.view['cb_r2'].set_active(True)
            self.view['cb_kill'].set_active(True)
        
        return

    def on_button_synthesize_clicked(self, button):
        #print "Clicked Synthesize"
        self.model.file_name = utils.dialogs.save_synthesis_results_dialog(self.model.last_visited_synthesis_result_path)
        if self.model.file_name:
            self.model.last_visited_synthesis_result_path = self.model.file_name
            #pass
        #print "the file name is:", self.model.file_name
        check_req = self.parent.check_panel.view['cb_check_reqs_consistency'].get_active()
        self.parent.synthesize(check_req)
        return
    

        

    # ------------------------------------------------------------ #
    #                       Notifications                          #
    # ------------------------------------------------------------ #


    # ---------------------------------------- #
    #   Notifications for observed properties  #
    # ---------------------------------------- #

    def property_busy_level_change_notification(self, model, old, new):
        if old != new: self.view['button_synthesize'].set_sensitive(new == 0)
        return            

    pass # end of class
