##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.controller import Controller

from controllers.trace import TraceCtrl
import utils.globals

import gtk


# ----------------------------------------------------------------------
class TraceCompactCtrl (Controller):
    """This is the controller for a single trace."""

    def __init__(self, trace_model, traces_model, parent=None):
        """This needs traces model to access categories information"""
        Controller.__init__(self, trace_model)
        self.traces = traces_model
        self.trace = TraceCtrl(trace_model)
        
        self.parent = parent  # parent controller

        self.changing_name = False # part of the internal status
        return

    def registerView(self, view):
        Controller.registerView(self, view)

        self.setup_view(view)
        return        

    def setup_view(self, view):        
        # name
        text = "<span size='larger'>%s</span>"
        view['label_name'].set_markup(text % self.model.name)

        # dependencies label:
        deps = self.model.get_dependencies()
        # no more than 10 chars:
        if gtk.pygtk_version < (2,6,0) and len(deps) > 10: deps = deps[:10]+"..."
        text = "<small>%s</small>" % ", ".join(deps)
        view['label_deps'].set_markup(text)
        
        # categories
        view['label_cat'].set_text(utils.globals.CAT_NEW_NAME)

        return


    def create_contextual_menu(self):
        menu = gtk.Menu()
        menu.set_title("Move to category")

        invalid_cats = (utils.globals.CAT_NEW_NAME, utils.globals.CAT_OOD_NAME)
        cats = self.traces.get_categories_model()
        iter = cats.get_iter_first()
        while iter:
            cat_name = cats.get_value(iter, 0)

            if cat_name not in invalid_cats:
                item = gtk.MenuItem(cat_name)
                item.connect("activate", self.on_move_to_cat_activate, cat_name)
                item.show()
                menu.append(item)
                pass
            
            iter = cats.iter_next(iter)
            pass

        return menu

    # ----------------------------------------------------------------------
    #  Widgets Signals
    # ----------------------------------------------------------------------

    def on_button_trace_del_clicked(self, button):
        # moves to the trash
        self.traces.remove_trace(self.model, utils.globals.CAT_NEW_NAME)
        self.traces.add_trace(self.model, utils.globals.CAT_TRA_NAME)
        return

    def on_button_move_to_clicked(self, button):
        # prepares and shows a menu with the possible categories:
        menu = self.create_contextual_menu()
        menu.popup(None, None, None, 1, 0)
        return

    def on_move_to_cat_activate(self, item, cat_name):
        """The user selected a category from the menu 'Move to category'"""
        self.traces.remove_trace(self.model, utils.globals.CAT_NEW_NAME)
        self.traces.add_trace(self.model, cat_name)        
        return



    # ---------------------------------------- #
    #   Notifications for observed properties  #
    # ---------------------------------------- #
    def property_name_change_notification(self, model, old, new):
        if model == self.model and not self.changing_name:
            # changed the trace'name, not by self
            text = "<span size='larger'>%s</span>"
            self.view['label_name'].set_markup(text % new)
            pass
        return

    def property_description_changed_change_notification(self, model, old, new):
        # dependencies label:
        text = "<small>%s</small>" % ", ".join(self.model.get_dependencies())
        self.view['label_deps'].set_markup(text)
        return
    
    pass # end of class
