#  Author: Roberto Cavada <cavada@fbk.eu>
#
#  Copyright (c) 2005 by FBK 
#
#  pygtkmvc is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  pygtkmvc is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#  For more information on pygtkmvc see <http://pygtkmvc.sourceforge.net>
#  or email to the RAT Team <rat@fbk.eu>.
#  Please report bugs to <rat@fbk.eu>..

import inspect

class ObservableWrapper:
    """This class wraps an object instance to make that instance observable.

    This allows user's class instances and python's containers (lists
    and maps) to be used in the observer pattern as well.
    This feature is new in version 0.9.3."""
    
    def __init__(self, name, inst, setters, model=None):
        """Takes the object to monitor, and a list of method names
        that can change the instance itself. Later when those method
        are invoked, the observers will be notified.

        @param name is the observable property this object is associted with
        @param inst is the object instance to monitor
        @param setters is a list of inst's methods that may change the object
        @param model is the model containing the observable property
        """

        self.name = name
        self.obs_obj = inst
        self.obs_model = model
        self.__setters = setters
        return

    def __getattr__(self, name):
        res = getattr(self.obj, name)
        if name in self.__setters and self.model:
            # notify the modification            
            self.model.notify_property_change(self.name, None, self)
            pass
        return res

    pass # end of class



class ObservableContainer (ObservableWrapper):
    """A wrapper for python lists and other containers"""
    def __init__(self, name, inst, model=None):
        setters = ("__setitem__", "__delitem__", "append", "extend",
                   "insert", "pop", "remove", "reverse", "sort")
        ObservableWrapper.__init__(self, name, inst, setters, model)
        return
    pass # end of class
                                   
