##  ===========================================================================
##  Author: Ingo Pill
##
##  Copyright 2005 by Ingo Pill <ipill@ist.tugraz.at>
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information on this software see <rat.fbk.eu>
##  or email to Ingo Pill <ipill@ist.tugraz.at>.
##  Please report bugs to Ingo Pill <ipill@ist.tugraz.at>
##
##  ===========================================================================

import gtk
import os
import utils
import gobject

from gtkmvc.model import Model
from models.trace import TraceModel
from tools.stubs.vis import PSVisStub
from tools.utils.vis import TraceData
from tools.utils.vis import TraceTranslation
from utils.widgets import ParenMatchTextBuffer

import utils.globals

class PropertySimulationModel (Model):
    """This is the model of the frame about Property Simulation.
    """

    # -------------------------------------- #
    #      Set of observable properties      #
    # -------------------------------------- #    
    __properties__ = {
        "changed" : False, # the status changed
        }

    def __init__(self):
        Model.__init__(self)
	self.projectName = 'project1'
        self.property = '{{a};{b};{b}:{c}}(G(a && b))'
	self.fixedValuesSere = None
	self.trcDataAp = TraceData()
	self.trcDataEvl = TraceData()
        self.trcModelAp = TraceModel()
        self.trcModelEvl = TraceModel()
	self.visOutputBuffer = gtk.TextBuffer()
        self.psVisStub = None #HIDE: PSVisStub(utils.globals.VIS_PATH, self.visOutputBuffer)
	self.registerObserver(self)
	self.status = 'Initialized'
	self.visStubEndCalls=[]
	self.translator = TraceTranslation()
	self.psaFeatureRequests = None
	self.psaTS = gtk.TreeStore(str,int,int,int,int,int,int,int,int,int,int,int,int,int)
	self.propertyTextBuffer = ParenMatchTextBuffer()
	return

    def clear(self):
        """Cleans up the model"""
        self.changed = True
        return   

    def to_xml(self, root_doc, parent_node):
        ps = root_doc.createElement("property_simulation")
        parent_node.appendChild(ps)
        return

    def from_xml(self, root_doc, node):
        return
    
    def stop_stub(self): 
        self.psVisStub.stop_session()
	return
	
    def addVisStubEndCall(self, call):
        self.visStubEndCalls.append(call)
	return

    def getProperty(self):
        """@return the raw property text"""
        start, end = self.propertyTextBuffer.get_bounds()
        return self.propertyTextBuffer.get_text(start, end, False)

    def setProperty(self, prop_text):
        """Sets the property text content, as if the user did it.
        Before checking the property, method deriveProperty must be
        called then to update the internal status.
        This method will overwrite any previously existing text."""

        self.propertyTextBuffer.set_text(prop_text)
        return        
	
    def deriveProperty(self):
        temp = self.getProperty()
	self.property=temp.replace('\n', ' ').lstrip()
	return
    
    def isPropertySet(self):
        if len(self.property)==0:
          return False
	else:
	  return True
    
    def generateWitness(self):
    
        assumptions=[]
	if self.fixedValuesSere!=None: assumptions.append(self.fixedValuesSere)
	#self.fixedValuesSere=None
	if self.psaFeatureRequests!=None:
	  for i in range(2,len(self.psaFeatureRequests)):
	    if self.trcDataEvl.signalIsSere[i]==0:
              if self.psaFeatureRequests[i][0]==1:
		assumptions.append('F('+self.trcDataEvl.signalNames[i]+')')
              if self.psaFeatureRequests[i][1]==1:
		assumptions.append('G('+self.trcDataEvl.signalNames[i]+')')
              if self.psaFeatureRequests[i][2]==1:
		assumptions.append('F(!('+self.trcDataEvl.signalNames[i]+'))')
              if self.psaFeatureRequests[i][3]==1:
		assumptions.append('G(!('+self.trcDataEvl.signalNames[i]+'))')   
	    else:
              if self.psaFeatureRequests[i][0]==1:
		assumptions.append('F({True}|+>{'+self.trcDataEvl.signalNames[i]+'})')
              if self.psaFeatureRequests[i][1]==1:
		assumptions.append('G({True}|+>{'+self.trcDataEvl.signalNames[i]+'})')
              if self.psaFeatureRequests[i][2]==1:
		assumptions.append('F(!({True}|+>{'+self.trcDataEvl.signalNames[i]+'}))')
              if self.psaFeatureRequests[i][3]==1:
		assumptions.append('G(!({True}|+>{'+self.trcDataEvl.signalNames[i]+'}))')   
        self.psVisStub.setProperty(self.property)
	self.psVisStub.setProjectName(self.projectName)
	self.psVisStub.setAssumptions(assumptions)
	self.psVisStub.set_end_session_callback(self.__vis_stub_end)
	
	self.status='Computing W'
	# launches the stub's session and exits 
        #self.busy_level += 1
	self.psVisStub.begin_session()
	self.psVisStub.run()
        return
	
    def generateCounterExample(self):
        assumptions=[]
	if self.fixedValuesSere!=None: assumptions.append(self.fixedValuesSere)
	#self.fixedValuesSere=None
	if self.psaFeatureRequests!=None:
	  for i in range(2,len(self.psaFeatureRequests)):
	    if self.trcDataEvl.signalIsSere[i]==0:
              if self.psaFeatureRequests[i][0]==1:
		assumptions.append('F('+self.trcDataEvl.signalNames[i]+')')
              if self.psaFeatureRequests[i][1]==1:
		assumptions.append('G('+self.trcDataEvl.signalNames[i]+')')
              if self.psaFeatureRequests[i][2]==1:
		assumptions.append('F(!('+self.trcDataEvl.signalNames[i]+'))')
              if self.psaFeatureRequests[i][3]==1:
		assumptions.append('G(!('+self.trcDataEvl.signalNames[i]+'))')   
	    else:
              if self.psaFeatureRequests[i][0]==1:
		assumptions.append('F({True}|+>{'+self.trcDataEvl.signalNames[i]+'})')
              if self.psaFeatureRequests[i][1]==1:
		assumptions.append('G({True}|+>{'+self.trcDataEvl.signalNames[i]+'})')
              if self.psaFeatureRequests[i][2]==1:
		assumptions.append('F(!({True}|+>{'+self.trcDataEvl.signalNames[i]+'}))')
              if self.psaFeatureRequests[i][3]==1:
		assumptions.append('G(!({True}|+>{'+self.trcDataEvl.signalNames[i]+'}))')         
        self.psVisStub.setProperty('!('+self.property+')')
	self.psVisStub.setProjectName(self.projectName)
	self.psVisStub.setAssumptions(assumptions)
	self.psVisStub.set_end_session_callback(self.__vis_stub_end)
	
	self.status='Computing CE'
	# launches the stub's session and exits 
        #self.busy_level += 1
        self.psVisStub.begin_session()
	self.psVisStub.run()	
        return

    def insertTimeStep(self, step):
		
	self.trcDataAp.insertTimeStep(step)
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.status='Outdated'
        return
    
    def removeTimeStep(self, step):
	ret=self.trcDataAp.removeTimeStep(step)
	if ret==1:
	  self.translator.translate(self.trcDataAp, self.trcModelAp)
	  self.status='Outdated'
	  return 1
	else:
          return 0
	
    def setSignalValueToTrue(self, signal, step):
        
	self.trcDataAp.fixSignalValue(signal, step, 'True')
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	self.status='Outdated'
	return
    
    def setSignalValueToFalse(self, signal, step):
        
	self.trcDataAp.fixSignalValue(signal, step, 'False')
	self.translator.translate(self.trcDataAp, self.trcModelAp)	  
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	self.status='Outdated'
        return

    def setSignalValueToDNC(self, signal, step):
        
	self.trcDataAp.setSignalValueDNC(signal, step)
	self.translator.translate(self.trcDataAp, self.trcModelAp)	  
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	self.status='Outdated'
        return

    def setSignalToTrue(self, signal):
        
	self.trcDataAp.fixSignalToValue(signal, 'True')
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	self.status='Outdated'
	return

    def setSignalToFalse(self, signal):
        
	self.trcDataAp.fixSignalToValue(signal, 'False')
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	self.status='Outdated'
	return

    def setSignalToDNC(self, signal):
        
	self.trcDataAp.setSignalDNC(signal)
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	return

    def fixSignalValues(self, signal):

	self.trcDataAp.fixSignalValues(signal)
	self.translator.translate(self.trcDataAp, self.trcModelAp)
	self.trcDataAp.deriveTraceRestrictions()
	self.fixedValuesSere=self.trcDataAp.getFixedValuesSere()
	return

    
    def greyoutEVLView(self):
        self.translator.setColorMode(2)
        self.translator.translate(self.trcDataEvl, self.trcModelEvl)
	self.translator.setColorMode()  # no param means reset
        return
    
    def activatePSATraceFeature(self, signal, feature):
        self.psaFeatureRequests[signal][feature]=1
        return
	
    def deactivatePSATraceFeature(self, signal, feature):
        self.psaFeatureRequests[signal][feature]=0
        return

    def updatePSATS_recur(self, parent, parent_iter):
	trcHierarchy = self.trcDataEvl.getHierarchy()
	trcChildren  = []

	for i in trcHierarchy:
	  if i[0]==parent:
            trcChildren=i[1]
	    
	for i in trcChildren:
 	  data=[]
	  data.append(self.trcDataEvl.signalNames[i])
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][0][j])	
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][1][j])	
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][2][j])
	  for j in range(4):
	    data.append(self.psaFeatureRequests[i][j])
	  sig_iter=self.psaTS.insert_before(parent_iter, None, data)
	  self.updatePSATS_recur(i, sig_iter)
        
        return

    def updatePSATS(self):

        self.psaTS.clear()
        trcHierarchy = self.trcDataEvl.getHierarchy()
        trcRootList  = []

        for i in trcHierarchy:
          if i[0] == -1:
            trcRootList=i[1]              # These are the signals on top level

        for i in trcRootList:      # i = Signal index
 	  data=[]
	  data.append(self.trcDataEvl.signalNames[i])
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][0][j])	
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][1][j])	
	  for j in range(3):
	    data.append(self.trcDataEvl.formulaStats[i][2][j])
	  for j in range(4):
	    data.append(self.psaFeatureRequests[i][j])
	  #print data
	  sig_iter=self.psaTS.insert_before(None, None, data)
	  self.updatePSATS_recur(i, sig_iter)
        return    
    
    def __vis_stub_end(self, ret_value, result, dummy):
        evl=True
	if self.status=='Computing W':
	  self.status='Witness'
	elif self.status=='Computing CE':
	  self.status='Counterexample'
	#print " vis stub end"

	if self.psVisStub.resultStatus=='Trace':
          #zeros the fixed values for signals 
	  self.fixedValuesSere=None
          self.trcDataAp=result[0]
	  self.trcDataEvl=result[1]
	  #print self.trcDataEvl.signalNames
	  #print self.trcDataEvl.signalIsSere
	  
	  if self.trcDataEvl.dataRead!=True:
	    self.trcDataEvl=TraceData()
	    evl=False
	    if self.status=='Counterexample':
	      self.status=='No EVL:Counterexample'
	    elif self.status=='Witness':
	      self.status=='NO EVL:Witness'
	      
	  self.trcDataAp.initializeDNC(1)
	  self.trcDataEvl.initializeDNC(1)
	  self.trcDataEvl.computeStats()
	  self.psaFeatureRequests=[]
	  for i in self.trcDataEvl.signalNames:
	    self.psaFeatureRequests.append([0,0,0,0])
	  self.updatePSATS()
	  
	  self.translator.translate(self.trcDataAp, self.trcModelAp)
	  if evl:
	    self.translator.translate(self.trcDataEvl, self.trcModelEvl)
	  else:
	    self.trcModelEvl.clear()
	else:
	  self.status=self.psVisStub.resultStatus	  		  
	for i in self.visStubEndCalls:
          i()
	return

    pass # end of class


