##  ===========================================================================
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.model import Model

import utils.globals

import gtk
import gobject

class SynthesisPanelModel (Model):

    """This is the model of the synthesis panel. One instance at the
    moment is embedded within the realizability window."""


    # -------------------------------------- #
    #      Set of observable properties      #
    # -------------------------------------- #    
    __properties__ = {
        'busy_level' : 0  # how many tools are running
        }

    def __init__(self, parent):
        Model.__init__(self)
        self.parent = parent
        self.last_visited_synthesis_result_path = None

        self.mode = 'BLIF'
        self.r1 = True
        self.r2 = True
        self.kill = True
        self.one_hot = False
        self.file_name = None
        
        return


    def get_options(self):        
        """Returns a list of options."""

        opts = []

        
        if self.parent.check_panel.use_dyn_reord:
            opts.append("--dyn")
            opts.append("--reorder_method=%s" % self.parent.check_panel.reord_meth)

        if self.r1: opts.append("--r1")
        if self.r2: opts.append("--r2")
        if self.kill: opts.append("--kill")
        if self.one_hot: opts.append("--one_hot")

        opts.append("--partition=%s" % self.parent.check_panel.part_meth)

        opts.append("--mode=%s" % self.mode)
            
        return opts




    pass # end of class

