##  ===========================================================================
##  Author: Ingo Pill <ipill@ist.tugraz.at>
##
##  Copyright (c) 2005 by Graz University of Technology
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from models.trace import TraceModel
from models.trace import TraceStripeModel
import gtk

# This module contains classes and functions used during the usage of VIS
# in property simulation.

class VisResults:
    """ Read a .str and .evl file to gain a trace and compute the temporal evaluation
    of a formula and its subformulae."""
    
    def __init__(self):
        return

    def read(self, projectName):
        """This function parses two files to fill 2 trace data containers, 
	containing the trace and the evaluation information."""
	self.trc=TraceData()
	self.evl=TraceData()
	self.trc.setProject(projectName, 0)
	self.evl.setProject(projectName, 0)
	#print "read STR\n"
	self.trc.readVisSTRFile()
	#print "read EVL\n"
	self.evl.readVisEVLFile(self.trc)
	#print "read all\n"
	return(self.trc,self.evl)
    pass # end of class


class TraceTranslation:
   """ Translates a TraceData Structure into a TraceModel for display. """
   
   def __init__(self, colorMode=None):
       self.colorMode=colorMode
       return
   
   def setColorMode(self, colorMode=None): # no param means reset
       self.colorMode=colorMode
       return
       
   def translate_rec(self, signal, iterator, trcData, trcModel):

       trcHierarchy = trcData.getHierarchy()
       trcChildren  = []

       for i in trcHierarchy:
	 if i[0]==signal:
           trcChildren=i[1]
       
       #print "recursive"
       #print trcChildren
       
       for i in trcChildren:
         
	 # initialize stripe
	 stripe = TraceStripeModel(trcData.getSignalNames()[i])   
         # TODO Error Management - Check if binary signal 
         stripe.set_boolean()  # Setting signal type to boolean
         
	 # get a row for the signal
         sig_iter = trcModel.append(iterator)
         
         # fill stripe with values for the signal
         for j in range(trcData.getFinitePartLength()):
	   if self.colorMode==None:
             stripe.add_value((trcData.getFinitePart()[i][j],trcData.getFiniteDNC()[i][j]))
	   else:
             stripe.add_value((trcData.getFinitePart()[i][j],self.colorMode))
	     

         for j in range(trcData.getInfinitePartLength()):
	   if self.colorMode==None:
             stripe.add_value((trcData.getInfinitePart()[i][j],trcData.getInfiniteDNC()[i][j]))
	   else:
             stripe.add_value((trcData.getInfinitePart()[i][j],self.colorMode))
         
	 #print "Created Stripe "+trcData.getSignalNames()[i]
	 #print stripe.values
         # store stripe 
         trcModel.set_value(sig_iter, 0, stripe)    	       
         # search for children and do the same for them
         self.translate_rec(i, sig_iter, trcData, trcModel)

       return 

   def translate(self, trcData,trcModel):
       """ This function takes a traceData structure as argument, translates it to 
       TraceModel structure format to fill the given trcModel. """
       
       trcHierarchy = trcData.getHierarchy()
       trcRootList  = []
    
       # CLEAN trcModel
       trcModel.clear()  # clear treestore
    
#       gtk.rc_parse("styles.rc")
       
       trcModel.loops=[]
       trcModel.dependencies=[]
       
       # Set new info
       
       trcModel.loops.append(trcData.getFinitePartLength())
       trcModel.max_loop=(trcData.getFinitePartLength()+trcData.getInfinitePartLength())
       
       # create stripes and fill structure
    
       for i in trcHierarchy:
         if i[0] == -1:
           trcRootList=i[1]              # These are the signals on top level
 
    
       for i in trcRootList:      # i = Signal index
         
	 # initialize stripe
	 stripe = TraceStripeModel(trcData.getSignalNames()[i])   
         # TODO Error Management - Check if binary signal 
         stripe.set_boolean()  # Setting signal type to boolean
         
	 # get a row for the signal
         sig_iter=trcModel.append(None)
	 #sig_iter = trcModel.insert_before(None, None)
      
         # fill stripe with values for the signal
         for j in range(trcData.getFinitePartLength()):
	   if self.colorMode==None:
             stripe.add_value((trcData.getFinitePart()[i][j],trcData.getFiniteDNC()[i][j]))
           else:
             stripe.add_value((trcData.getFinitePart()[i][j],self.colorMode))
	      
         for j in range(trcData.getInfinitePartLength()):
	   if self.colorMode==None:
	     stripe.add_value((trcData.getInfinitePart()[i][j],trcData.getInfiniteDNC()[i][j]))
	   else:
	     stripe.add_value((trcData.getInfinitePart()[i][j],self.colorMode))
         
	 #print "Created Stripe "+trcData.getSignalNames()[i]
	 #print stripe.values
         # store stripe 
	 trcModel.set_value(sig_iter, 0, stripe)    
         # search for children and do the same for them
         self.translate_rec(i, sig_iter, trcData, trcModel)

       return
   
   pass # end of class

class TraceData:
  """This class instances containers for the trace data as read from VIS and 
  used to compute the evaluation (Hintikka)."""  
  
  projectName = ''      # Project name related to the trace
  projectInstance =0    # As several data sets may be related to a project, this minor
                        #  version number may be used to differentiate
  signalNames = []	# Storing signal names -> position of the name in the
                        # list equals signal number in the traceVal arrays
  signalIsSere=[]       # determines if a signal is a SERE or NOT
  signalValues = []     # For each signal there is a list with symbolic signal values
                        #   the value number (used for data storag) equals list position
  traceValFin = []      # List containing value lists for each signal, for the finite part
  traceValInf = []      # List containing value lists for each signal, for the infinite part
  traceDNCFin = []
  traceDNCInf = []
  sereLength = []       # List with length of the formula for SERE creation -> will be ignored for non-SERE formulae
  dataRead=False        # Was data read in ?
  traceHierarchy = []   # Tuples of signal number and a list of childs represent the display
                        #  hierarchy for display in a tree widget -> root = (-1)
  formulaStats = []     # Contains coverage evaluation data
  traceFixedSere = None # Holds a formula representing a sere built from DNC info
  
  def __init__(self):   # Default constructor - no real function yet
    
    self.projectName = ''
    self.projectInstance = 0
    self.signalNames = []
    self.signalIsSere = []
    self.traceValFin = []
    self.traceValInf = []
    self.serelength = []
    self.dataRead = False
    self.traceHierarchy = []
    self.formulaStats = []
    return
  # set attributes
  
  def setProject(self, projectName, projectInstance):
    self.projectName = projectName
    self.projectInstance = projectInstance
    return

  # Initialize DNC structures with given value (should be True/False)
  # AP Traces should be initialized with True, Evaluation Traces with False
  def initializeDNC(self, value):
    self.traceDNCFin=[]
    self.traceDNCInf=[]
    for i in range(len(self.traceValFin)):
      buffer=[]
      for j in range(len(self.traceValFin[i])):
        buffer.append(value)
      self.traceDNCFin.append(buffer)

    for i in range(len(self.traceValInf)):
      buffer=[]
      for j in range(len(self.traceValInf[i])):
        buffer.append(value)
      self.traceDNCInf.append(buffer)
      
    return
  
  # Set a value DNC 
  def setSignalValueDNC(self, variable, timestep):
    
    signal=self.getSignalNumber(variable)
    assert(signal!=-1)
    
    if timestep>self.getFinitePartLength():
      timestep=timestep-(self.getFinitePartLength()+1)
      self.traceDNCInf[signal][timestep]=1
    else:
      timestep=timestep-1
      self.traceDNCFin[signal][timestep]=1
    return
    
  # Set Signal Value
  def setSignalValue(self, variable, timestep, value):
    
    signal=self.getSignalNumber(variable)
    assert(signal!=-1)
    value=self.getValueNumber(signal, value)
    assert(value!=-1)
    
    if timestep>self.getFinitePartLength():
      timestep=timestep-(self.getFinitePartLength()+1)
      self.traceValInf[signal][timestep]=str(value)
    else:
      timestep=timestep-1
      self.traceValFin[signal][timestep]=str(value)
    return
  
  # Fix signal value -> the same as set signal Value, but DNC is set to false as well
  def fixSignalValue(self, variable, timestep, value):
    
    signal=self.getSignalNumber(variable)
    assert(signal!=-1)
    value=self.getValueNumber(signal, value)
    assert(value!=-1)
    
    if timestep>self.getFinitePartLength():
      timestep=timestep-(self.getFinitePartLength()+1)
      self.traceValInf[signal][timestep]=str(value)
      self.traceDNCInf[signal][timestep]=0
    else:
      timestep=timestep-1
      self.traceValFin[signal][timestep]=str(value)
      self.traceDNCFin[signal][timestep]=0
    return

  # Fix all timesteps of a signal to a certain value
  def fixSignalToValue(self, variable, value):
    
    for i in range(1, self.getFinitePartLength()+self.getInfinitePartLength()+1):
      self.fixSignalValue(variable, i, value)
    return
      
  # Set all timesteps of a signal as DNC values
  def setSignalDNC(self, variable):
    
    signal=self.getSignalNumber(variable)
    assert(signal!=-1)
    
    for i in range(0, self.getFinitePartLength()):
      self.traceDNCFin[signal][i]=1
    for i in range(0, self.getInfinitePartLength()):
      self.traceDNCInf[signal][i]=1
    
    return

  # Fix all timesteps of a signal as they are    
  def fixSignalValues(self, variable):

    signal=self.getSignalNumber(variable)
    assert(signal!=-1)
    
    for i in range(0, len(self.traceDNCFin[signal])):
      self.traceDNCFin[signal][i]=0
    
    for i in range(0, len(self.traceDNCInf[signal])):
      self.traceDNCInf[signal][i]=0    

    return

    
  
  # Insert a timestep (step number beginning with 1)  
  def insertTimeStep(self, step):
    
    if step>self.getFinitePartLength():   # on the stepnumber decide whether in the inf or fin part
      step=step-self.getFinitePartLength()
      newTraceValInf=[]
      for i in self.traceValInf:
        newTraceValInf.append(i[:(step-1)]+['0']+i[(step-1):])
      self.traceValInf=newTraceValInf
      newDNCInf=[]
      for i in self.traceDNCInf:
        newDNCInf.append(i[:(step-1)]+[1]+i[(step-1):])
      self.traceDNCInf=newDNCInf      
    else:
      newTraceValFin=[]
      for i in self.traceValFin:
        newTraceValFin.append(i[:(step-1)]+['0']+i[(step-1):])
      self.traceValFin=newTraceValFin
      newDNCFin=[]
      for i in self.traceDNCFin:
        newDNCFin.append(i[:(step-1)]+[1]+i[(step-1):])
      self.traceDNCFin=newDNCFin      
    return

  # Remove a timestep (step number beginning with 1)
  def removeTimeStep(self, step):
      
    if step>self.getFinitePartLength():   # on the stepnumber decide whether in the inf or fin part
      if self.getInfinitePartLength()==1:
	return 0
      step=step-self.getFinitePartLength()
      newTraceValInf=[]
      for i in self.traceValInf:
        newTraceValInf.append(i[:(step-1)]+i[(step):])
      self.traceValInf=newTraceValInf
      newDNCInf=[]
      for i in self.traceDNCInf:
        newDNCInf.append(i[:(step-1)]+i[(step):])
      self.traceDNCInf=newDNCInf      
    else:
      newTraceValFin=[]
      for i in self.traceValFin:
        newTraceValFin.append(i[:(step-1)]+i[(step):])
      self.traceValFin=newTraceValFin
      newDNCFin=[]
      for i in self.traceDNCFin:
        newDNCFin.append(i[:(step-1)]+i[(step):])
      self.traceDNCFin=newDNCFin      
    return 1



  # some object queries
    
  def hasData(self):
    return self.dataRead
    
  def getProjectName(self):
    return self.projectName
    
  def getProjectInstance(self):
    return self.projectInstance

  def getFinitePart(self):
    return self.traceValFin
  
  def getFiniteDNC(self):
    return self.traceDNCFin
      
  def getSignalValues(self, signumber):
    return self.signalValues[signumber]
    
  def getInfinitePart(self):
    return self.traceValInf

  def getInfiniteDNC(self):
    return self.traceDNCInf
  
  def getFinitePartLength(self):
    return len(self.traceValFin[0])

  def getInfinitePartLength(self):
    return len(self.traceValInf[0])

  def getSignalNumber(self, signalName):
    num=-1
    for i in range(len(self.signalNames)):
      if self.signalNames[i]==signalName:
        num=i
	break
    return num

  def getValueNumber(self, signalNumber, valueName):
    num=-1
    for i in range(len(self.signalValues[signalNumber])):
      if self.signalValues[signalNumber][i]==valueName:
        num=i
	break
    return num
  
  def getSignalNames(self):
    return self.signalNames
    
  def getHierarchy(self):
    return self.traceHierarchy
  
  def getFixedValuesSere(self):
    return self.traceFixedSere  
    
  # for debugging - print contained data
  
  def printSignalNames(self):
    
    for i in self.signalNames:
      print 'Signal'
      print i
      print '\n'
    
    return
  
  def printTraceData(self):  
    
    for i in range(len(self.signalNames)):
      
      print '\nSignal'
      print self.signalNames[i]
      print '\nFinite part\n'
      
      for j in self.traceValFin[i]:
        print j
	#print '\n'
	
      print '\nInfinite part\n'
      
      for j in self.traceValInf[i]:
        print j
	#print '\n'
	
    return
  
  def printStats(self):
    
    for i in range(0,len(self.signalNames)):
    
      print 'Signal '+self.signalNames[i]+' has Stats:'
      print self.formulaStats[i]
    
    return
  
  def deriveTraceRestrictions(self):
    """ Builds a sere from the trace value and DNC info representing fixed values
        in the trace."""
    self.traceFixedSere=None	           # reset
    if len(self.traceDNCFin)==0: return     # if there's no DNC info return
    if len(self.traceDNCFin[0])!=self.getFinitePartLength(): return  
    found=False

    if (self.getFinitePartLength()>0):
      sere='{'      # Finite Part
      for i in range(self.getFinitePartLength()):
	var=[]
	for j in range(len(self.traceDNCFin)):
          if self.traceDNCFin[j][i]==0:
	    found=True
	    if self.traceValFin[j][i]=='1':
	      var.append(self.signalNames[j])
	    else:
	      var.append('!'+self.signalNames[j])
	if len(var)>0:
          varstring='{'
          for j in var:
            varstring=varstring+'{'+j+'}'+':'
          varstring=varstring[:-1]
          varstring=varstring+'}'
	else:
          varstring='True'
	sere=sere+varstring+';'
      sere=sere[:-1]
      sere=sere+'}'    # Finite Part is finished
    
      sere=sere+';{{'   # Infinite Part
    else:
      sere='{{'
    
    
    for i in range(self.getInfinitePartLength()):
      var=[]
      for j in range(len(self.traceDNCInf)):
        if self.traceDNCInf[j][i]==0:
	  found=True
	  if self.traceValInf[j][i]=='1':
	    var.append(self.signalNames[j])
	  else:
	    var.append('!'+self.signalNames[j])
      if len(var)>0:
        varstring='{'
        for j in var:
          varstring=varstring+'{'+ j + '}'+':'
        varstring=varstring[:-1]
        varstring=varstring+'}'
      else:
        varstring='True'
      sere=sere+varstring+';'
    sere=sere[:-1]
    sere=sere+'}[*]}'    # Infinite Part is finished
    
    sere='{True}|+>{'+sere+'}'
    #print sere
    
    
    if found==True:   # if we found some restrictions then set formula
      self.traceFixedSere=sere
    else:             # otherwise don't use formula as it could only blast the automaton
      self.traceFixedSere=None
    return
     
  
  # Fill Structure with info from an .str file provided by VIS  
  def readVisSTRFile(self):
    """Reads data from a .str file provided by VIS sugar_sp function"""
    
    # Open file and remove newlines    
    visFile = open(self.projectName+'.str','r')
    visLines = visFile.readlines()
    
    for i in range(len(visLines)):
      
      visLines[i] = visLines[i][:-1]      # remove newline at the end of the line
      
      if visLines[i].isspace():           # replace empty lines by '#E '
        visLines[i]='#E '
      if len(visLines[i])==0:
        visLines[i]='#E '
    
    
    # Parse the file for the trace data
    
    # Not really robust; assumes perfect input file :-)    
    # As of yet this is pseudo code ;)
    
    # uses three counters/pointers segment, stepNumber, signalNumber
    # as of yet stepNumber is unused, as no consistency check is performed yet
    segment=0
    stepNumber=0
    signalNumber=0
    signals=[]
        
    for line in visLines:
      
      if   line[0:2] == '# ':       # Ignore comment
        1==1
      elif line      == '#E ':      # Ignore Empty Line
        1==1
      elif line[0:3] == '#S ':      # Begin reading signal names
        
	segment=1
	stepNumber=0
	signalNumber=0
        
      elif line[0:3] == '#F ':      # Begin reading finite part of the trace
      
        self.traceHierarchy.append((-1,signals))  # All signals have been read -> write hierarchy
	segment=2
	stepNumber=0
	signalNumber=0
        
      elif line[0:3] == '#I ':      # Begin reading infinite part of the trace
        
	segment=3
	stepNumber=0
	signalNumber=0
        
      elif line[0:3] == '#N ':      # Indicates next transition (next time step)
        
	stepNumber+=1
	signalNumber=0
        
      else:                         # Now it has to be a signal assignment or declaration
        
	if segment==1:              # Declaration -> store info in signal name vs. number table
	
	  self.signalNames.append(line)  # whereas signal number is the position in the name list
	  self.signalValues.append(['False', 'True'])
	  self.traceValFin.append([])    # Initialize value list for the signal's finite part
	  self.traceValInf.append([])    # Initialize value list for the signal's infinite part
	  self.signalIsSere.append(0)
          signals.append(signalNumber)
	  signalNumber+=1
          
	elif segment==2:            # We're in the finite part; store value appropriately
	
	  spline=line.split(':')
	  if self.signalNames.count(spline[0]):      # There might be additional selector variables
	    self.traceValFin[self.signalNames.index(spline[0])].append(spline[1])
	  signalNumber+=1
          
	elif segment==3:            # We're in the infinite part; store value appropriately
	
	  spline=line.split(':')
	  if self.signalNames.count(spline[0]):      # There might be additional selector variables
	    self.traceValInf[self.signalNames.index(spline[0])].append(spline[1])
	  signalNumber+=1
          
    self.dataRead=True
    #print self.traceHierarchy
    visFile.close()


  # Fill Structure with info from an .evl file provided by VIS
  def readVisEVLFile(self, apData):
    """Reads data from a .evl file provided by VIS sugar_sp function"""
    
    # Open file and remove newlines    
    visFile = open(self.projectName+'.evl','r')
    visLines = visFile.readlines()
    
    for i in range(len(visLines)):
      
      visLines[i] = visLines[i][:-1]      # remove newline at the end of the line
      
      if visLines[i].isspace():           # replace empty lines by '#E '
        visLines[i]='#E '
      if len(visLines[i])==0:
        visLines[i]='#E '
    
    
    # Parse the file for the trace data
    
    # Not really robust; assumes perfect input file :-)    
    # As of yet this is pseudo code ;)
    
    # uses counters/pointers segment
    segment=0
    apNumber=-1
    compBegin=-1
    compEnd=-1
   
    apUnset=[]       # subset of atomic propostitions
                     # for which data could not be derived from given apData

    
    # Add the root signal
    self.traceHierarchy.append((-1,[2]))
    
    
    # The file format uses 4 Segments started by distinctive desriptors
    #  1. '#S '-> Signal names
    #  2. '#P '-> Signals which are atomic propositions
    #  3. '#V '-> which Signals to display and their dependencies
    #  4. '#C '-> how to derive the signal data, beginning of section
    #     '#T '->                                end of section 

    for i in range(len(visLines)):
      
      if   visLines[i][0:2] == '# ':       # Ignore comment
        1==1
      elif visLines[i]      == '#E ':      # Ignore Empty Line
        1==1
      elif visLines[i][0:3] == '#S ':      # Begin reading signal names
        
	segment=1

      elif visLines[i][0:3] == '#P ':      # Begin determining atomic propositions
                                           # and set values for TRUE and FALSE Signals
                                           
        segment=2

        for j in range(len(self.signalNames)):
	  if (self.signalNames[j]=='FALSE'):
	    if (len(self.traceValFin[j])==0):
	      for k in range(apData.getFinitePartLength()):
	        self.traceValFin[j].append(0)
	    if (len(self.traceValInf[j])==0):
	      for k in range(apData.getInfinitePartLength()):
	        self.traceValInf[j].append(0)
	  if (self.signalNames[j]=='TRUE'):
	    if (len(self.traceValFin[j])==0):
	      for k in range(apData.getFinitePartLength()):
	        self.traceValFin[j].append(1)
	    if (len(self.traceValInf[j])==0):
	      for k in range(apData.getInfinitePartLength()):
	        self.traceValInf[j].append(1)
    
    	
      elif visLines[i][0:3] == '#V ':      # Begin determining view and set unset APs
        segment=3
	for j in apUnset:

	  # the data was not in ap so we have to guess it and add it to apdata as well
	  valFin=[]
	  valInf=[]
	  for k in range(apData.getFinitePartLength()):
	    valFin.append('0')
	  for k in range(apData.getInfinitePartLength()):
	    valInf.append('0')
	  self.traceValFin[j]=valFin[:]
	  self.traceValInf[j]=valInf[:]
	  apData.signalNames.append(self.signalNames[j])
	  apData.signalValues.append(['False', 'True'])
	  apData.sereLength.append(1)      # Initialize the length with 1
	  apData.formulaStats.append([[0,0,0],[0,0,0],[0,0,0]]) # Initialize StatisticData
	  apData.traceValFin.append(valFin[:])
	  apData.traceValInf.append(valInf[:])
	  apData.traceHierarchy[0][1].append(apData.getSignalNumber(self.signalNames[j]))
	apUnset=[]
	   
	
      elif visLines[i][0:3] == '#C ':      # Begin determining signal computation

        segment=4
	compBegin=i+1

      elif visLines[i][0:3] == '#T ':      # End determining signal computation section
        compEnd=i
	missingSig=1                       # because of range it's not necessary to subtract 1.
	count=1
	while missingSig > 0:              # See if all computation rules have been used
	  count+=1
	  
	  missingSig=0
	  r=range(compBegin, compEnd)
	  r.reverse()
	  
	  for j in r:
	    
	    spline=visLines[j].split('^')
	    #print "Trying"
	    #print spline
	      
	    if ((len(self.traceValFin[int(spline[0],10)])==0) and (len(self.traceValInf[int(spline[0],10)])==0)):
	      
 	      missingCh=0
	      for k in spline[2:]:
	        
		if ((len(self.traceValFin[int(k,10)])==0) and (len(self.traceValInf[int(k,10)])==0)):
		  
		  #print k
		  #print "is missing"
		  
	          missingCh+=1
		  
              if missingCh==0:              # We have all data to compute the signal
	        
		# Compute Signal
		self.computeSignal(spline[0], spline[1], spline[2:])
	        
	      else:                         # We do not have all data and have to wait for another
	                                    #   iteration to derive the signal
		missingSig+=1
	      
	    
	
      else:                         # Store Values appropriately
        
	if segment==1:
	  spline=visLines[i].split('^')
	  self.signalNames.append(spline[1])  # whereas signal number is the position in the name list
	  self.signalIsSere.append(0)
	  self.signalValues.append(['False', 'True'])
          self.traceValFin.append([])    # Initialize value list for the signal's finite part
	  self.traceValInf.append([])    # Initialize value list for the signal's infinite part
	  self.sereLength.append(1)      # Initialize the length with 1
	  self.formulaStats.append([[0,0,0],[0,0,0],[0,0,0]]) # Initialize StatisticData
          
	elif segment==2:
          
	  apNumber=-1
	  if apData.signalNames.count(self.signalNames[int(visLines[i],10)]):        # The data for atomic proposition is in apData
	    apNumber=apData.signalNames.index(self.signalNames[int(visLines[i],10)])
	    self.traceValFin[int(visLines[i],10)]=apData.getFinitePart()[apNumber][:]    # copy the data ....
	    self.traceValInf[int(visLines[i],10)]=apData.getInfinitePart()[apNumber][:]  # copy the data ....	    
	  else:    
	    apUnset.append(int(visLines[i],10))  
          

	
	elif segment==3:				  # Retrieve hierarchy info for the view
          	  
	  spline=visLines[i].split('^')
	  father=int(spline[0],10)
	  children=[]
	  for j in spline[1:]:
	    children.append(int(j,10))
	  self.traceHierarchy.append((father,children))  # add tuple (father,[child_list]) to hierarchy
	  
	elif segment==4:       # Compute missing signal values
          
	  spline=visLines[i].split('^')
   
          if ((len(self.traceValFin[int(spline[0],10)])==0) and (len(self.traceValInf[int(spline[0],10)])==0)):
  	      
 	    missingCh=0
	    for k in spline[2:]:
	             
	      if ((len(self.traceValFin[int(k,10)])==0) and (len(self.traceValInf[int(k,10)])==0)):
	          
		  missingCh+=1
		  
            if missingCh==0:              # We have all data to compute the signal
	      
	      # Compute Signal	
	      if not self.computeSignal(spline[0], spline[1], spline[2:]):
	        #print 'unknown operator encountered'
		visFile.close()
		return False
    
    self.dataRead=True
    #print "Eval Hierarchy"
    #print self.traceHierarchy
    visFile.close()
    return True
  
  def computeStats(self):
    
    for i in range(0,len(self.signalNames)):  
                                              
      finlen=len(self.traceValFin[i])
      inflen=len(self.traceValInf[i])
      
      for j in range(finlen):
        if (j!=0):
          if self.traceValFin[i][j]!=self.traceValFin[i][j-1]:   # register value changes
	    self.formulaStats[i][0][2]+=1
	    self.formulaStats[i][2][2]+=1
	if self.traceValFin[i][j]=='0':    # register amount of false/zero's
	  self.formulaStats[i][0][0]+=1
	  self.formulaStats[i][2][0]+=1
	else:                              # then it's a true(one) -> register accordingly
	  self.formulaStats[i][0][1]+=1
	  self.formulaStats[i][2][1]+=1

      if finlen>0:      
	if self.traceValFin[i][finlen-1]!=self.traceValInf[i][0]:
          self.formulaStats[i][2][2]+=1
      
      for j in range(inflen):
        if self.traceValInf[i][j]!=self.traceValInf[i][(j-1)%inflen]:  # register value changes
	  self.formulaStats[i][1][2]+=1
	  self.formulaStats[i][2][2]+=1
	if self.traceValInf[i][j]=='0':     # register false values
	  self.formulaStats[i][1][0]+=1
	  self.formulaStats[i][2][0]+=1
	else:                               # register true values
	  self.formulaStats[i][1][1]+=1
	  self.formulaStats[i][2][1]+=1
    return
  
  def computeSignal(self, spsignal, operator, spoperands):
    """ Computes the values for an empty signal -> the data structures are assumed to be empty, 
    and currently only unary and binary operators, as well as that input signalsa are of same length """    
    
    operands=[]
    for i in spoperands:
      operands.append(int(i,10))
    
    signal=int(spsignal,10)
    
    #print "Computing "+spsignal+"with operands",spoperands ,"\n"
    
    if ((operator=='G')|(operator=='g')):    # Globally
      
      NCinInf='1'
      
      for i in self.traceValInf[operands[0]]:    # See if contradiction to G() is in Inf part
        if i=='0':                               #  NCinInf is low active -> afterwards fill Inf part with result
	  NCinInf='0'
      
      for i in self.traceValInf[operands[0]]:    # Fill
        self.traceValInf[signal].append(NCinInf)

      finlen=len(self.traceValFin[operands[0]])  # Now check the finite part - prerequisites
      if finlen>0:
	r=self.traceValFin[operands[0]][:]
	r.reverse()


	if NCinInf=='0':                           # If there's a contradiction everything's done
          for i in r:
	    self.traceValFin[signal].append('0')
	else:                                      # else check the the last element of the finite part
          ev='1'
          if r[0]=='0':
            ev='0'
          self.traceValFin[signal].append(ev)        
          for i in r[1:]:                          # and then go to the beginning checking the previous calculated  element
            ev='1'                                 # and the current element of the input signal
	    if self.traceValFin[signal][0]=='0':
	      ev='0'
	    if i=='0':
	      ev='0'
	    self.traceValFin[signal].insert(0,ev)
    
    
    elif ((operator=='N')|(operator=='n')):     # Never => the same as Globally but check signal for values '0'
      
      NCinInf='1'
      
      for i in self.traceValInf[operands[0]]:    # See if contradiction to N() is in Inf part
        if i=='1':                                 #  NCinInf is low active
	  NCinInf='0'
      
      for i in self.traceValInf[operands[0]]:
        self.traceValInf[signal].append(NCinInf)

      finlen=len(self.traceValFin[operands[0]])
      if finlen>0:
	r=self.traceValFin[operands[0]][:]
	r.reverse()


	if NCinInf=='0':
          for i in r:
	    self.traceValFin[signal].append('0')
	else:        
          ev='1'
          if r[0]=='1':
            ev='0'
          self.traceValFin[signal].append(ev)

          for i in r[1:]:
            ev='1'
	    if self.traceValFin[signal][0]=='1':
	      ev='0'
	    if i=='1':
	      ev='0'
	    self.traceValFin[signal].insert(0,ev)


    
    elif ((operator=='F')|(operator=="f")):   # Eventually                   
      
      FinInf='0'
      
      for i in self.traceValInf[operands[0]]:    # See if F is fulfilled somewhere in Inf part
        if i=='1':                               # FinInf is high-active
	  FinInf='1'
	  
      for i in self.traceValInf[operands[0]]:    # Fill the infinite part of the evaluation 
        self.traceValInf[signal].append(FinInf)
	
      finlen=len(self.traceValFin[operands[0]])  # Finite part
      if finlen>0:
	r=self.traceValFin[operands[0]][:]
	r.reverse()

	if FinInf=='1':                            # if F is fulfilled in the infinite part then
          for i in r:                              # everything's done
	    self.traceValFin[signal].append('1')
	else:        
          ev='0'
          if r[0]=='1':
            ev='1'
          self.traceValFin[signal].append(ev)

          for i in r[1:]:
            ev='0'
	    if self.traceValFin[signal][0]=='1':   # check the last calculated element
	      ev='1'
	    if i=='1':                             # check the current one
	      ev='1'
	    self.traceValFin[signal].insert(0,ev)
    
    elif operator=='U':   # p U q
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      r=range(inflen)                            # 1. infinite part
      r.reverse()
      
      for i in r:                                # First assume the formula is not fulfilled
        self.traceValInf[signal].append('0')
      
      for i in r:                                # Search for q backwards
        
	if self.traceValInf[operands[1]][i]=='1': # When q is found check praefix p's
	  self.traceValInf[signal][i]='1'         # q models pUq when not overlapping until
	  for j in range(inflen):
	    
	    if self.traceValInf[operands[0]][(i-(j+1))%inflen]=='1': # Chech backwards if p is true -> if at the beginning go to the end
	      self.traceValInf[signal][(i-(j+1))%inflen]='1'        
	    else:
	      break;   # if no praefix p any more stop search

      if finlen>0:     
	r=range(finlen)                            # 2. finite part
	r.reverse

	for i in r:                                # First assume not fulfilled
          self.traceValFin[signal].append('0')

	if self.traceValInf[signal][0]=='1':       # If there was a q in the inf part we have to continue the search
          for i in r:                              # for praefix p's
	    if self.traceValFin[operands[0]][i]=='1':
	      self.traceValFin[signal][i]='1'
	    else:
	      break;

	for i in r:                                # Search for new q's in the finite part (backwards)

	  if self.traceValFin[operands[1]][i]=='1': # When q is found check praefix p's
            self.traceValFin[signal][i]='1'         # q models pUg when not overlapping until
	    for j in range(i):

	      if self.traceValFin[operands[0]][(i-(j+1))]=='1': # Chech backwards if p is true 
		self.traceValFin[signal][(i-(j+1))]='1'
	      else:
		break;   # if not stop search
      
        
    elif (operator=='V'):  # Overlapping Until
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      r=range(inflen)
      r.reverse()
      
      for i in r:                                # First assume not fulfilled
        self.traceValInf[signal].append('0')
      
      for i in r:                                # Search for q&p backwards
        
	if (self.traceValInf[operands[1]][i]=='1')&(self.traceValInf[operands[0]][i]=='1'): # When q*p is found check praefix p's
	  
	  for j in range(inflen):
	    
	    if self.traceValInf[operands[0]][(i-j)%inflen]=='1': # Chech backwards if p is true -> if at the beginning go to the end
	      self.traceValInf[signal][(i-j)%inflen]='1'
	    else:
	      break;   # if not stop search

      if finlen>0:      
	r=range(finlen)
	r.reverse

	for i in r:                                # First assume not fulfilled
          self.traceValFin[signal].append('0')

	if self.traceValInf[signal][0]=='1':       # If there was a match in the infinite part we have to continue the search for this q
          for i in r:
	    if self.traceValFin[operands[0]][i]=='1':
	      self.traceValFin[signal][i]=='1'
	    else:
	      break;

	for i in r:                                # Search for q backwards

	  if (self.traceValFin[operands[1]][i]=='1')&(self.traceValFin[operands[0]][i]=='1'): # When q*p is found check praefix p's

	    for j in range(i):

	      if self.traceValFin[operands[0]][(i-(j))]=='1': # Chech backwards if p is true 
		self.traceValFin[signal][(i-(j))]='1'
	      else:
		break;   # if not stop search    

    elif (operator=='R'):  # Release -> Consists of overlapping Until with swapped operands and an OR G(operands[1])
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      r=range(inflen)
      r.reverse()
      
      # 1. to overlapping Until
      
      for i in r:                                # First assume not fulfilled
        self.traceValInf[signal].append('0')
      
      for i in r:                                # Search for q&p backwards
        
	if (self.traceValInf[operands[1]][i]=='1')&(self.traceValInf[operands[0]][i]=='1'): # When q*p is found check praefix p's
	  
	  for j in range(inflen):
	    
	    if self.traceValInf[operands[1]][(i-(j))%inflen]=='1': # Chech backwards if p is true -> if at the beginning go to the end
	      self.traceValInf[signal][(i-(j))%inflen]='1'
	    else:
	      break;   # if not stop search
      
      if finlen>0:
	r=range(finlen)
	r.reverse

	for i in r:                                # First assume not fulfilled
          self.traceValFin[signal].append('0')

	if self.traceValInf[signal][0]=='1':       # Continue search from an p&q in the infinite part
          for i in r:
	    if self.traceValFin[operands[1]][i]=='1':
	      self.traceValFin[signal][i]=='1'
	    else:
	      break;

	for i in r:                                # Search for new q&p backwards

	  if (self.traceValFin[operands[1]][i]=='1')&(self.traceValFin[operands[0]][i]=='1'): # When q*p is found check praefix p's

	    for j in range(i):

	      if self.traceValFin[operands[1]][(i-(j))]=='1': # Chech backwards if p is true 
		self.traceValFin[signal][(i-(j))]='1'
	      else:
		break;   # if not stop search    
      
      # 2. Do Globally(operands[1]) and OR the result.
      ContrFound='0'
      for i in self.traceValInf[operands[1]]:
        if i=='0':
	  ContrFound='1'
	  
      if ContrFound=='0':
        for i in range(inflen):
	  self.traceValInf[signal][i]='1'

      if finlen>0:      
	r=range(finlen)
	r.reverse()

	if ContrFound=='0':
       	  for i in r:
	    if self.traceValFin[operands[1]][i]=='1':
	      self.traceValFin[signal][i]=='1'
	    else:
	      break;
     
       
    elif operator=='X': # Next
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      if len(operands)==1:  # single X
        
        if finlen>0:
	  for i in range(finlen):

	    if i==(finlen-1):
	      if self.traceValInf[operands[0]][0]=='1':
		self.traceValFin[signal].append('1')
	      else:
		self.traceValFin[signal].append('0')
	    else:
	      if self.traceValFin[operands[0]][i+1]=='1':
		self.traceValFin[signal].append('1')
	      else:
		self.traceValFin[signal].append('0')

	for i in range(inflen):

	  if i==(inflen-1):
	    if self.traceValInf[operands[0]][0]=='1':
	      self.traceValInf[signal].append('1')
	    else:
	      self.traceValInf[signal].append('0')
	  else:
	    if self.traceValInf[operands[0]][i+1]=='1':
	      self.traceValInf[signal].append('1')
	    else:
	      self.traceValInf[signal].append('0')
      
      else:   # Abbreviation for nested X e.g. X[5] - nesting level is stored in second operand
        level=int(operands[1],10)
        if finlen>0:
          for i in range(finlen): # The computation is forward - 1. finite part

	    pos=i+level  # nesting level 
	    if pos>=finlen: # if we come to the inf part check there
	      pos=pos-finlen
	      pos=pos%inflen
	      if self.traceValInf[operands[0]][pos]=='1':
		self.traceValFin[signal].append('1')
	      else:
		 self.traceValFin[signal].append('0')
	    else:  # else check in the finite part
	      if self.traceValFin[operands[0]][pos]=='1':
		self.traceValFin[signal].append('1')
	      else:
		self.traceValFin[signal].append('0')

	for i in range(inflen): # 2. infinite part
	
	  pos=i+level
	  pos=pos%inflen
	  
	  if self.traceValInf[operands[0]][pos]=='1':
	    self.traceValInf[signal].append('1')
	  else:
	    self.traceValInf[signal].append('0')
	          
           
    elif operator=='*': # And
      if len(self.traceValFin[operands[0]])>0:
	for i in range(len(self.traceValFin[operands[0]])):
          ev='1'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='0'
	  if self.traceValFin[operands[1]][i]=='0':
            ev='0'
	  self.traceValFin[signal].append(ev)
      
      for i in range(len(self.traceValInf[operands[0]])):
        ev='1'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='0'
	if self.traceValInf[operands[1]][i]=='0':
          ev='0'
	self.traceValInf[signal].append(ev)
    
    elif operator=='+': # Or
      
      if len(self.traceValFin[operands[0]])>0:
	for i in range(len(self.traceValFin[operands[0]])):
          ev='0'
	  if self.traceValFin[operands[0]][i]=='1':
	    ev='1'
	  if self.traceValFin[operands[1]][i]=='1':
            ev='1'
	  self.traceValFin[signal].append(ev)
      
      for i in range(len(self.traceValInf[operands[0]])):
        ev='0'
	if self.traceValInf[operands[0]][i]=='1':
	  ev='1'
	if self.traceValInf[operands[1]][i]=='1':
          ev='1'
	self.traceValInf[signal].append(ev)

    elif operator=='>':    # implication p->q = !p+q
      
      if len(self.traceValFin[operands[0]])>0:
	for i in range(len(self.traceValFin[operands[0]])):
          ev='0'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='1'
	  if self.traceValFin[operands[1]][i]=='1':
            ev='1'
	  self.traceValFin[signal].append(ev)
      
      for i in range(len(self.traceValInf[operands[0]])):
        ev='0'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='1'
	if self.traceValInf[operands[1]][i]=='1':
          ev='1'
	self.traceValInf[signal].append(ev)


    elif operator=='E':    # Equivalence p<->q
      
      if len(self.traceValFin[operands[0]])>0:
	for i in range(len(self.traceValFin[operands[0]])):
          ev='0'
	  if self.traceValFin[operands[0]][i]==self.traceValFin[operands[1]][i]:
	    ev='1'
	  self.traceValFin[signal].append(ev)
      
      for i in range(len(self.traceValInf[operands[0]])):
        ev='0'
	if self.traceValInf[operands[0]][i]==self.traceValInf[operands[1]][i]:
          ev='1'
	self.traceValInf[signal].append(ev)

       
    
    elif operator=='!':
    
      self.sereLength[signal]=self.sereLength[operands[0]]   # just for security reason
      if len(self.traceValFin[operands[0]])>0:
	for i in self.traceValFin[operands[0]]:
          if i=='0':
	    self.traceValFin[signal].append('1')
	  else:
	    self.traceValFin[signal].append('0')

      for i in self.traceValInf[operands[0]]:
        if i=='0':
	  self.traceValInf[signal].append('1')
	else:
	  self.traceValInf[signal].append('0')
      
    elif operator==';':

      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      self.signalIsSere[signal]=1
      
      if (self.sereLength[operands[0]] == -1)|(self.sereLength[operands[1]] == -1):
        self.sereLength[signal] = -1
      else:
        self.sereLength[signal]=self.sereLength[operands[0]]+self.sereLength[operands[1]]

      if finlen>0:      
	for i in range(finlen):

	  ev='1'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='0'
	  if (i+self.sereLength[operands[0]])>=finlen:
	    if (self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-(finlen))%inflen])=='0':
	      ev='0'
	  else:
	    if self.traceValFin[operands[1]][i+self.sereLength[operands[0]]]=='0':
	      ev='0'
	  self.traceValFin[signal].append(ev)

      for i in range(inflen):

        ev='1'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='0'
	if self.traceValInf[operands[1]][(i+self.sereLength[operands[0]])%inflen]=='0':
	  ev='0'
	self.traceValInf[signal].append(ev)

    elif operator==':':

      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      self.signalIsSere[signal]=1
      
      if (self.sereLength[operands[0]] == -1)|(self.sereLength[operands[1]] == -1):
        self.sereLength[signal] = -1
      else:
        self.sereLength[signal]=self.sereLength[operands[0]]+self.sereLength[operands[1]]-1
      
      if finlen>0:      
	for i in range(finlen):

	  ev='1'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='0'
	  if (i+self.sereLength[operands[0]]-1)>=finlen:
	    if (self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-(finlen+1))%inflen])=='0':
	      ev='0'
	  else:
	    if self.traceValFin[operands[1]][i+self.sereLength[operands[0]]-1]=='0':
	      ev='0'
	  self.traceValFin[signal].append(ev)
	
      for i in range(inflen):
        
        ev='1'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='0'
	if self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-1)%inflen]=='0':
	  ev='0'
	self.traceValInf[signal].append(ev)
      

    elif operator=='&':
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      self.signalIsSere[signal]=1
      
      if (self.sereLength[operands[0]] == -1)|(self.sereLength[operands[1]] == -1):
        self.sereLength[signal] = -1
      else:
        self.sereLength[signal]=max(self.sereLength[operands[0]], self.sereLength[operands[1]])
      
      if finlen>0:      
	for i in range(finlen):
          ev='1'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='0'
	  if self.traceValFin[operands[1]][i]=='0':
            ev='0'
	  self.traceValFin[signal].append(ev)
      
      for i in range(inflen):
        ev='1'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='0'
	if self.traceValInf[operands[1]][i]=='0':
          ev='0'
	self.traceValInf[signal].append(ev)

    
    elif operator=='|':
      
      self.sereLength[signal]=max(self.sereLength[operands[0]], self.sereLength[operands[1]])
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      self.signalIsSere[signal]=1
  
      if (self.sereLength[operands[0]] == -1)|(self.sereLength[operands[1]] == -1):
        self.sereLength[signal] = -1
      else:
        self.sereLength[signal]=max(self.sereLength[operands[0]], self.sereLength[operands[1]])
       
      if finlen>0:      
	for i in range(finlen):
          ev='0'
	  if self.traceValFin[operands[0]][i]=='1':
	    ev='1'
	  if self.traceValFin[operands[1]][i]=='1':
            ev='1'
	  self.traceValFin[signal].append(ev)
      
      for i in range(inflen):
        ev='0'
	if self.traceValInf[operands[0]][i]=='1':
	  ev='1'
	if self.traceValInf[operands[1]][i]=='1':
          ev='1'
	self.traceValInf[signal].append(ev)

    elif operator=='L':

      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      self.signalIsSere[signal]=1

      # if one of the sere's contains an unmatched 'L' (sereLength[i]==-1) or the two lengths
      # are not equal then it's unmatched and zeroed out
      if (self.sereLength[operands[0]] == -1)|(self.sereLength[operands[1]] == -1)|\
         (self.sereLength[operands[0]]!=self.sereLength[operands[1]]):
        
	self.sereLength[signal]= -1
	
	for i in range(finlen):
	  self.traceValFin[signal].append('0')
	for i in range(inflen):
	  self.traceValInf[signal].append('0')

      else: # otherwise compute the result
        if finlen>0:      
	  for i in range(finlen):
            ev='1'
	    if self.traceValFin[operands[0]][i]=='0':
	      ev='0'
	    if self.traceValFin[operands[1]][i]=='0':
              ev='0'
	    self.traceValFin[signal].append(ev)

	for i in range(inflen):
          ev='1'
	  if self.traceValInf[operands[0]][i]=='0':
	    ev='0'
	  if self.traceValInf[operands[1]][i]=='0':
            ev='0'
	  self.traceValInf[signal].append(ev)
      
    elif operator=='S': # similar to !p+q
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      if finlen>0:      
	for i in range(finlen):

	  ev='0'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='1'
	  if (i+self.sereLength[operands[0]]-1)>=finlen:
	    if (self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-(1+finlen))%inflen])=='1':
	      ev='1'
	  else:
	    if self.traceValFin[operands[1]][i+self.sereLength[operands[0]]-1]=='1':
	      ev='1'
	  self.traceValFin[signal].append(ev)
	
      for i in range(inflen):
        
        ev='0'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='1'
	if self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-1)%inflen]=='1':
	  ev='1'
	self.traceValInf[signal].append(ev)
    
    elif operator=='O': # similar to 'S', difference is that second operand is a sere
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      if finlen>0:      
	for i in range(finlen):

	  ev='0'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='1'
	  if (i+self.sereLength[operands[0]]-1)>=finlen:
	    if (self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-(finlen+1))%inflen])=='1':
	      ev='1'
	  else:
	    if self.traceValFin[operands[1]][i+self.sereLength[operands[0]]-1]=='1':
	      ev='1'
	  self.traceValFin[signal].append(ev)
	
      for i in range(inflen):
        
        ev='0'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='1'
	if self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-1)%inflen]=='1':
	  ev='1'
	self.traceValInf[signal].append(ev)
    
    elif operator=='I': # similar to 'O' but not overlapping
      
      finlen=len(self.traceValFin[operands[0]])
      inflen=len(self.traceValInf[operands[0]])
      
      if finlen>0:      
	for i in range(finlen):

	  ev='0'
	  if self.traceValFin[operands[0]][i]=='0':
	    ev='1'
	  if (i+self.sereLength[operands[0]])>=finlen:
	    if (self.traceValInf[operands[1]][(i+self.sereLength[operands[0]]-(finlen))%inflen])=='1':
	      ev='1'
	  else:
	    if self.traceValFin[operands[1]][i+self.sereLength[operands[0]]]=='1':
	      ev='1'
	  self.traceValFin[signal].append(ev)
	
      for i in range(inflen):
        
        ev='0'
	if self.traceValInf[operands[0]][i]=='0':
	  ev='1'
	if self.traceValInf[operands[1]][(i+self.sereLength[operands[0]])%inflen]=='1':
	  ev='1'
	self.traceValInf[signal].append(ev)
    
    else: # might be operator 'D' which is a dummy value for unsupported evaluation op provided by vis
      #print "Visualization Error: encountered unexpected operand during signal computation:"
      #print operator
      #print "\n"
      return False    
    return  True
