##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


from gtkmvc.model import Model
from gtkmvc.controller import Controller

import views.dialogs
import utils.globals

import gtk
import os.path

# ----------------------------------------------------------------------
class PopupDialogCtrl (Controller):
    def __init__(self):
        Controller.__init__(self, Model())
        return
    
    pass # end of class


# ----------------------------------------------------------------------
def open_invalid_name_dialog():
    c =  PopupDialogCtrl()
    v = views.dialogs.InvalidName(c)
    v.run()
    return

# ----------------------------------------------------------------------
def open_message_dialog(message, details):
    c =  PopupDialogCtrl()
    v = views.dialogs.DialogMessage(c, message, details)
    v.run()
    return


# ----------------------------------------------------------------------
def open_warning_request_cancel_ok(message, details):
    """Returns True if the user confirmed the request, False otherwise"""
    c = PopupDialogCtrl()
    v = views.dialogs.WarningRequestOkCancel(c, message, details)
    res = v.run()
    return res == gtk.RESPONSE_OK
    
# ----------------------------------------------------------------------
def open_confirm_action_dialog(action_text, message, details):
    """Returns True if the user confirmed the action, False otherwise"""
    c = PopupDialogCtrl()
    v = views.dialogs.QuestionConfirmAction(c, action_text, message,
                                            details)
    res = v.run()
    return res == gtk.RESPONSE_OK
    

# ----------------------------------------------------------------------
def open_project_dialog(default_path=None):
    """Returns the selected file name, or None"""
    
    title = "Open %s Project..." % utils.globals.SHORT_APPL_NAME
    
    if gtk.pygtk_version >= (2,4,0):
        # uses the new standard dialog
        dlg = gtk.FileChooserDialog(title,
                                    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                               gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        dlg.set_default_response(gtk.RESPONSE_OK)

        filter = gtk.FileFilter()
        filter.set_name("%s Project" % utils.globals.SHORT_APPL_NAME)
        filter.add_mime_type("text/xml")
        filter.add_pattern("*%s" % utils.globals.PROJECT_FILE_EXTENSION)
        dlg.add_filter(filter)

        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        dlg.add_filter(filter)

        if default_path: dlg.set_filename(os.path.abspath(default_path))
    else:
        # uses the old style dialog
        dlg = gtk.FileSelection(title)
        dlg.complete("*%s" % utils.globals.PROJECT_FILE_EXTENSION)
        if default_path: dlg.set_filename(os.path.abspath(default_path))
        pass

    if dlg.run() == gtk.RESPONSE_OK: res = dlg.get_filename()
    else: res = None

    dlg.destroy()
    return res


def save_project_dialog(default_path=None, extra_info=True):
    """Returns the selected file name, or None"""
    
    title = "Save %s Project..." % utils.globals.SHORT_APPL_NAME

    c =  PopupDialogCtrl()
    dlg = views.dialogs.SaveAsCustomFileChooserDialog(c, title, extra_info)
    if dlg.run() == gtk.RESPONSE_OK:
        
        # double assignment
        res = \
            (filename, extra_info) = (dlg.get_filename(), dlg.get_extra_info())

        if os.path.isfile(filename):
            # requests to confirm overwrite:
            ovrw = open_confirm_action_dialog("Overwrite",
                                              'Overwrite "%s"?' % os.path.basename(filename),
                                              """A file with this name already exists.
If you choose to overwrite this file, the contents will be lost.""")
            if not ovrw: res = ( None, extra_info )
            pass
        pass
    else: res = ( None, extra_info )
            
    dlg.destroy()
    return res



def save_synthesis_results_dialog(default_path=None):
    """Returns the selected file name, or None"""
    
    title = "Save Synthesis Results..."
    
    if gtk.pygtk_version >= (2,4,0):
        # uses the new standard dialog
        dlg = gtk.FileChooserDialog(title,
                                    action=gtk.FILE_CHOOSER_ACTION_SAVE, 
                                    buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                               gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        
        dlg.set_default_response(gtk.RESPONSE_OK)

        if default_path: dlg.set_filename(os.path.abspath(default_path))

    else:
        # uses the old style dialog
        dlg = gtk.FileSelection(title)
        #dlg.complete("*%s" % utils.globals.PROJECT_FILE_EXTENSION)
        if default_path: dlg.set_filename(os.path.abspath(default_path))
        pass
    
    if dlg.run() == gtk.RESPONSE_OK:
        res = dlg.get_filename()
        if os.path.isfile(res):
            # requests to confirm overwrite:
            ovrw = open_confirm_action_dialog("Overwrite",
                                              'Overwrite "%s"?' % os.path.basename(res),
                                              """A file with this name already exists.
If you choose to overwrite this file, the contents will be lost.""")
            if not ovrw: res = None
            pass
        pass
    else: res = None
            
    dlg.destroy()
    return res
