##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import re
import string
import os

def normalize_string(s):
    """@return s without double spaces, lines, tabs, etc."""
    return re.sub("\s+", " ", s)


def make_paragraph(s, line_width):
    """Splits a string over multiple lines whose length is given"""
    s = normalize_string(s)

    words = s.split()

    res = ""; l = 0
    for w in words:
        if l + len(w) >= line_width:
            res = res.strip()
            res += os.linesep
            l = 0
            pass
        
        l += len(w)
        res += w + " "
        pass

    return res.strip()


def indent_formula(f, line_width):
    """@return a string containing the indented formula"""

    added_space = False
    indent_size = 2
    indent = 0
    res = ""; l = 0
    for i in range(len(f)):
        w = f[i]

        if w in "([{":
            # checks previous char            
            if res and res.rstrip()[-1] == w: res = res.rstrip()
            
            # does it need to go to a new line
            res += w; l += len(w)
            indent += indent_size
            
            p = __search_matching_paren(f, i)
            if p-i+l >= line_width:
                res += os.linesep + (" " * indent)
                l = 0
                continue                     
            pass
        
        elif w in ")]}":
            if res and res.rstrip()[-1] == w: res = res.rstrip()
            res += w; l += len(w)
            indent = max(0, indent-indent_size)
            
        elif w in " \t\r\n":
            if l != 0 and not added_space: res += " "; l += len(w)
            added_space = True            

        else:
            # a normal character            
            res += w; l += len(w)
            added_space = False
            continue

        # go to a new line if needed
        if l + len(w) >= line_width:
            res += os.linesep + (" " * indent)
            l = 0
            pass

        pass
    
    return res



def __search_matching_paren(s, fro=0):
    """s is the string, from is the starting index at which the opening
    paren is found. Returns -1 if not found"""
    ops = "({[<"
    cps = ")}]>"
    
    op = s[fro]
    assert op in ops
    cp = cps[ops.find(op)]
    
    pos = -1; paren = 0; l = len(s)
    for i in range(fro, l):
        c = s[i]
        if (c == op): paren += 1
        elif (c == cp):
            paren -= 1
            if paren == 0: return i
            pass
        pass

    return -1
