##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================


# This file contains general utilities

import os.path
import utils.globals


def normalize_project_filename(fn):
    """Normalizes the project filename, adding the extension if needed"""
    if fn is None: fn = ""

    base, ext = os.path.splitext(fn)
    if ext.lower() != utils.globals.PROJECT_FILE_EXTENSION.lower() and \
           not os.path.isfile(fn):
        if len(ext) < 2: fn = base + utils.globals.PROJECT_FILE_EXTENSION
        else: fn = fn + utils.globals.PROJECT_FILE_EXTENSION
        pass
    
    return os.path.abspath(fn)


def string_normalize_whitespaces(the_string):
    """Removes tabs, double spaces, newlines, etc. Encodes to ascii."""
    import string

    tmp = the_string.encode("utf-8")
    tmp = tmp.strip()
    
    tmp = tmp.translate(string.maketrans(string.whitespace,
                                         " " * len(string.whitespace)))
    
    # removes double spaces:
    while (True):
        tmp2 = tmp.replace("  ", " ")
        if tmp2 != tmp: tmp = tmp2; continue
        break                        
    return tmp2

def make_temp_file():
    """@return a pair (file_descriptor, filename)"""
    try:
        from tempfile import mkstemp
        fn = mkstemp()[1]
        res = (open(fn, "w"), fn)
        
    
    except ImportError:
        from tempfile import mktemp
        fn = mktemp()
        res = (open(fn, "w"), fn)
        pass

    return res
    

def get_conjuction(props, delim=" "):
    """@param props is a list of string of properties. 
    @return the conjuction of given properties, as a string."""
    if len(props) == 0: return ""
    if len(props) == 1: return props[0]

    def transf(x):
        s = x.strip()
        if s[0] == "(" and s[-1] == ")": return x
        return "(%s)" % x
    
    p = map(transf, props)
    return (" &"+delim).join(p)
