##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2007 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import os.path

from views.trace_compact import TraceCompactView
import utils.globals

from gtkmvc.view import View 

import gtk


## ----------------------------------------------------------------------
class OutcomesView (View):

    """The checking outcome window/panel"""

    GLADE_FILE = os.path.join(utils.globals.GLADE_DIR, "outcome.glade")

    def __init__(self, ctrl, parent_view):
        """parent_view is used when detaching the view to its own window"""
        View.__init__(self, ctrl, self.GLADE_FILE, "hb_top")
        self.is_embedded = True
        self.__handle_tabs()
        self.parent_view = parent_view
        return

    def __handle_tabs(self):
        nb = self['notebook_outputs']
        if nb.get_n_pages() > 1:
            if self.is_embedded: nb.set_tab_pos(gtk.POS_BOTTOM)
            else: nb.set_tab_pos(gtk.POS_TOP)
            nb.set_show_tabs(True)
        else: nb.set_show_tabs(False)
        return        

    def add_trace(self, compact_trace_ctrl, page_name):
        """Adds a compact view TraceCompactView for the given TraceCompactCtrl.
        @return the number of the page the trace has been added"""

        v = TraceCompactView(compact_trace_ctrl)
        nb = self['notebook_outputs']
        nb.append_page(v.get_top_widget(), gtk.Label(page_name))
        pnum = len(nb.get_children())-1        
        self.switch_to_page(pnum)

        self.__handle_tabs()        
        return pnum

    def switch_to_page(self, pnum): self['notebook_outputs'].set_current_page(pnum)

    def remove_page(self, pnum):
        nb = self['notebook_outputs']
        nb.remove_page(pnum)
        self.__handle_tabs()
        return        
    
    def set_page_title(self, pnum, text):
        nb = self['notebook_outputs']
        child = nb.get_nth_page(pnum)
        nb.set_tab_label_text(child, text)
        return        

    def show_outcomes(self, show=True):
        self.switch_to_page(0)
        if self.is_embedded: self['expander_out'].set_expanded(show)
        return

    def to_window(self, exp, title):
        """Returns the window the view went"""
        if not self.is_embedded: return
        hb = self.parent_view.detach_outcomes()

        hb.remove(exp)
        nb = self['notebook_outputs']
        rec = nb.get_allocation()
        exp.remove(nb)

        w = gtk.Window()
        w.set_title(title)
        w.set_default_size(rec.width, int(rec.height*1.20))
        w.add(nb)
        w.show_all()
        self.is_embedded = False
        self.__exp = exp
        self.__handle_tabs()        
        return w

    def to_embedded(self, win):
        if self.is_embedded: return
        nb = win.get_child()
        win.remove(nb)

        self.__exp.add(nb)
        self["hb_top"].add(self.__exp)
        self.parent_view.attach_outcomes()
        
        self.is_embedded = True
        self.__handle_tabs()        
        return


    pass # end of class
    
    
    

