##  ===========================================================================
##  Author: Ingo Pill <ipill@ist.tugraz.at>
##
##  Copyright (c) 2005 by Graz University of Technology
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import os.path
import gtk

from utils import globals
from gtkmvc.view import View 

from views.trace import TraceView

class PropertySimulationView (View):

    """This is the view of the frame about Property Simulation.
    A part from a basic almost empty widget tree built with glade.
    """

    GLADE_FILE = os.path.join(globals.GLADE_DIR, "prop_simulation.glade")

    def __init__(self, ctrl):
        # sw_top is the name of the top level widget (see the glade file)
        View.__init__(self, ctrl, self.GLADE_FILE, "vbox_top")

        self.ap = TraceView(ctrl.apCtrl)
        self.evl = TraceView(ctrl.evlCtrl)
	# the two views should have the same horizontal adjustment, vertical adjustment always visible
	#   horizontal one only the lower one ...
        self.evl.get_top_widget().set_hadjustment(self.ap.get_top_widget().get_hadjustment())  
        self.ap.get_top_widget().set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
        self.evl.get_top_widget().set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)        
        
        self['vpaned'].add(self.ap.get_top_widget())
        self['vpaned'].add(self.evl.get_top_widget())
	ctrl.setStatusMessage()
	return
    
    def create_PSA_subview(self, psaCtrl):
        self.psaView=PropertySimulationAnalysisView(psaCtrl)
	self.psaView.set_parent_view(self)
	self.psaView.hide_view()
        return
	
    def showMessagePopup(self, message_type, message):
        top_window=self['vbox_top'].get_toplevel()
	self.popup = gtk.MessageDialog(
                top_window,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
		message_type,
                gtk.BUTTONS_CLOSE, 
		message)
	self.popup.connect('delete_event',self.closeMessagePopup)
	self.popup.connect('close', self.closeMessagePopup)
	self.popup.connect('response', self.closeMessagePopup)
	ret=self.popup.show() #run()
        return
	
    def closeMessagePopup(self, response_id, *args):
	self.popup.destroy()
	return 
		
	
    pass # end of class

class PropertySimulationAnalysisView (View):
    
    """ This is the view popping up by pressing the Analysis Button in Property Simulation.
    """
    
    GLADE_FILE = os.path.join(globals.GLADE_DIR, "propsim_analysis.glade")
    
    def __init__(self, ctrl):
        
	View.__init__(self, ctrl, self.GLADE_FILE)
		
	return
	
    def show_view(self):
        self['PSAWindow'].show_all()
        return
	
    def hide_view(self):
        self['PSAWindow'].hide_all()
	return
	
    pass # end of class
