##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2006 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

from gtkmvc.view import View 

from views.outcomes import OutcomesView
from views.check_panel import CheckingPanelView
from views.synthesis_panel import SynthesisPanelView
import utils.globals

import gtk
import os.path


## ----------------------------------------------------------------------
class RealizabilityView (View):

    """The main view for property assurance. This view contains two
    sub-views for the possibilities and for the assertions.
    Also, it contains a view for checking options and control"""

    GLADE_FILE = os.path.join(utils.globals.GLADE_DIR, "realizability.glade")

    def __init__(self, ctrl):        
        View.__init__(self, ctrl, self.GLADE_FILE, "vbox_top",
                      register=False)

        self.outs_is_attached = False 
        ctrl.registerView(self)

        # to store initial shadow type (cosmetics)
        self.shadow = self['sw_result'].get_shadow_type()
        return

    def create_sub_views(self, outcome_ctrl, panel_ctrl, synthesis_ctrl):
        """This is called by the controller during view's registration.
        @param ctrl_poss is the sub-controller of the Possbilities set
        @param ctrl_ass is the sub-controller of the Assertions set
        @param panel_ctrl is the controller for the checking options
        """
        self.panel = CheckingPanelView(panel_ctrl)

        self.panel['cb_check_reqs_consistency'].set_sensitive(False)
        self.panel['cb_coi'].set_sensitive(False)
        
        nb = self.panel['notebook_options']
        nb.set_show_tabs(False)
        nb.set_current_page(1)
        
        self['hbox_top'].pack_end(self.panel.get_top_widget(),
                                  expand=False, fill=True)

        self.synthesis_panel = SynthesisPanelView(synthesis_ctrl)
        self.panel['vbox_top'].pack_end(self.synthesis_panel.get_top_widget(), expand=False, fill=True)
        
        self.outcomes = OutcomesView(outcome_ctrl, self)
        self.attach_outcomes()
        return

    def detach_outcomes(self):        
        """Detaches the outcomes view and returns the top-level
        widget. Used to send the outcomes view to its own window"""
        res = None
        if self.outs_is_attached:
            res = self.outcomes.get_top_widget()
            self['vbox_top'].remove(res)
            self.outs_is_attached = False
            pass
        
        return res
    
    def attach_outcomes(self):
        """Attaches the outcomes view. Used as initial state and whenever
        the view is embedded from the in-window state"""
        if not self.outs_is_attached:
            wid = self.outcomes.get_top_widget()
            self['vbox_top'].pack_end(wid, expand=False, fill=True)
            self.outs_is_attached = True
            pass
        
        return
            

    pass # end of class
    
    
    

