##  ===========================================================================
##  Author: Roberto Cavada <cavada@fbk.eu>
##
##  Copyright (c) 2005 by FBK 
##
##  This is free software; you can redistribute it and/or
##  modify it under the terms of the GNU Lesser General Public
##  License as published by the Free Software Foundation; either
##  version 2 of the License, or (at your option) any later version.
##
##  This software is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
##  Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public
##  License along with this library; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
##
##  For more information about this software see <http://rat.fbk.eu>
##  or email to the RAT Team <rat@fbk.eu>.
##  Please report bugs to <rat@fbk.eu>.
##
##  ===========================================================================

import os.path
from utils import globals
from gtkmvc.view import View

import gtk
import gtk.gdk

from views.trace import TraceView


## ----------------------------------------------------------------------
class TracesView (View):
    """The container of traces"""

    GLADE_FILE = os.path.join(globals.GLADE_DIR, "traces.glade")

    def __init__(self, ctrl):        
        View.__init__(self, ctrl, self.GLADE_FILE, "window_top_level", register=False)

        self.embedded_cats = {}
        self.saved_position = None

        ctrl.registerView(self)
        return

    def add_embedded_category_view(self, embedded_cat_ctrl):
        if not self.embedded_cats.has_key(embedded_cat_ctrl):
            v = EmbeddedCategoryView(embedded_cat_ctrl)
            self['vbox_traces'].pack_start(v.get_top_widget())
            self.embedded_cats[embedded_cat_ctrl] = v
            pass
        
        return

    def remove_embedded_category_view(self, embedded_cat_ctrl):
        if self.embedded_cats.has_key(embedded_cat_ctrl):
            v = self.embedded_cats[embedded_cat_ctrl]
            self['vbox_traces'].remove(v.get_top_widget())
            del self.embedded_cats[embedded_cat_ctrl]
            pass
        
        return

    def set_embedded_category_pos(self, embedded_cat_ctrl, pos):
        if self.embedded_cats.has_key(embedded_cat_ctrl):
            v = self.embedded_cats[embedded_cat_ctrl]
            self['vbox_traces'].reorder_child(v.get_top_widget(), pos)
            pass
        
        return


    def show(self):
        View.show(self)
        if self.saved_position:
            self.get_top_widget().move(*self.saved_position)
            self.saved_position = None
            pass
        
        return

    def hide(self):
        self.saved_position = self.get_top_widget().get_position()
        View.hide(self)
        return    

    pass # end of class



## ----------------------------------------------------------------------
class EmbeddedCategoryView (View):
    """A category embedded within a TracesView"""

    GLADE_FILE = os.path.join(globals.GLADE_DIR, "traces.glade")

    def __init__(self, ctrl):        
        View.__init__(self, ctrl, self.GLADE_FILE, "vbox_cat_top", register=False)
        
        self['eventbox_label_cat_text'].modify_bg(gtk.STATE_NORMAL,
                                                  gtk.gdk.color_parse("gray80"))

        self.embedded_traces = {}

        ctrl.registerView(self)
        return

    def add_embedded_trace_view(self, embedded_trace_ctrl):
        if not self.embedded_traces.has_key(embedded_trace_ctrl):
            v = EmbeddedTraceView(embedded_trace_ctrl)
            self['vbox_traces'].pack_start(v.get_top_widget())
            self.embedded_traces[embedded_trace_ctrl] = v
            pass
        
        return

    def remove_embedded_trace_view(self, embedded_trace_ctrl):
        if self.embedded_traces.has_key(embedded_trace_ctrl):
            v = self.embedded_traces[embedded_trace_ctrl]
            self['vbox_traces'].remove(v.get_top_widget())
            del self.embedded_traces[embedded_trace_ctrl]
            pass
        
        return

    def set_embedded_trace_pos(self, embedded_trace_ctrl, pos):
        if self.embedded_traces.has_key(embedded_trace_ctrl):
            v = self.embedded_traces[embedded_trace_ctrl]
            self['vbox_traces'].reorder_child(v.get_top_widget(), pos)
            pass
        
        return
    
    

    pass # end of class


## ----------------------------------------------------------------------
class EmbeddedTraceView (View):
    """A single trace embedded within an embedded category"""
    
    GLADE_FILE = os.path.join(globals.GLADE_DIR, "traces.glade")

    def __init__(self, ctrl):
        """It is assumed ctrl contains the controller of the trace
        called 'trace'"""
        
        View.__init__(self, ctrl, self.GLADE_FILE, "vbox_trace_top")

        self.trace = TraceView(ctrl.trace)
        self.trace.set_horizontal_scroll(False)
        self.trace.set_vertical_scroll(False)
        
        self['vbox_trace_top'].pack_start(self.trace.get_top_widget())        
        return

    pass # end of class
