#!/usr/bin/env python

# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



"""Deadlock Checking

Verify whether a model (optionally extended with a fault and/or FDIR
model) is deadlock free or not.

Inputs      M : an operational model.
            FM : a fault model (optional).
            FDM : an FDIR model (optional).

Outputs     "FSM is deadlock free" shall be output if there are no deadlocks detected
            "FSM contains deadlocks" otherwise

Processing  Using model checking techniques, the COMPASS toolset
            shall check if the model contains deadlocks.
"""

import booter # import this as first module in scripts

from utils import log

from runscripts.model_checking__srs_015 import NuSMVModelChecking

class NuSMVDeadlockChecking (NuSMVModelChecking):
    """This is the script that can be used to perform model
    checking with nusmv"""
    
    def check_options(self, opt_parser, options):
        # NOTE: super(NuSMVModelChecking) is intentional,
        # as the model checking check_options needs to be overridden
        super(NuSMVModelChecking, self).check_options(opt_parser, options)

    def action(self):
        """Builds nusmv and performs deadlock checking"""
        # NOTE: super(NuSMVModelChecking) is intentional,
        # as the model checking action needs to be overridden
        super(NuSMVModelChecking, self).action()
        deadlockFree = self.nusmv.check_fsm_deadlocks()
        if not deadlockFree:
            log.user.info("FSM contains deadlocks")
        else:
            log.user.info("FSM is deadlock free")
        return

    pass # end of class
# ----------------------------------------------------------------------
        
if __name__ == "__main__":
    
    main = NuSMVDeadlockChecking()
    main.run()
    
