# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import sys
from antlr3 import *
from antlr3.compat import set, frozenset

from antlr3.tree import *
from slim_compiler.src.generated.SlimLexer import SlimLexer

from slim_compiler.src.errorManagement.errors import *
from slim_compiler.src.errorManagement.exceptions import *

class DerivedLexer(SlimLexer):
    
    def __init__(self, compile_information, input=None, state=None):
        SlimLexer.__init__(self, input, state)
        self.compile_information = compile_information
        self.lexing_error = False
        
    def reportError(self, error):
        if isinstance(error, MismatchedTokenException):            
            self.add_error(MismatchedTokenError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c),
                                        self.getCharErrorDisplay(error.expecting)))        
        elif isinstance(error, NoViableAltException):
            self.add_error(NoViableAltError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c)))
            
        elif isinstance(error, EarlyExitException):
            self.add_error(EarlyExitError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c)))
            
        elif isinstance(error, MismatchedSetException):
            self.add_error(MismatchedSetError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c),
                                        repr(error.expecting)))
            
        elif isinstance(error, MismatchedNotSetException):
            self.add_error(MismatchedNotSetError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c),
                                        repr(error.expecting)))

        elif isinstance(error, MismatchedRangeException):
            self.add_error(MismatchedRangeError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        "expression",
                                        self.getCharErrorDisplay(error.c),
                                        self.getCharErrorDisplay(error.a),
                                        self.getCharErrorDisplay(error.b)))
        elif isinstance(error, FailedPredicateException):
            self.add_error(FailedPredicateError(
                                        self.compile_information.current_file,
                                        error.line,
                                        error.charPositionInLine + 1,
                                        error.ruleName,
                                        error.predicateText))

        else:
            assert False
            
        self.lexing_error = True
                
    
    def has_lexing_errors(self):
        return self.lexing_error
    
    def add_error(self, error):
        self.compile_information.add_error(error)
