# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import sys
sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../')
from invokers import slim
import antlr3
import antlr3.tree
from slim_compiler.src.dataStructure.derivedlexer import DerivedLexer
from slim_compiler.src.dataStructure.derivedparser import DerivedParser
from slim_compiler.src.generated.SlimWalkerIdentCheck import SlimWalkerIdentCheck
from dataStructure.compile import CompileInformation
from slim_compiler.src import globals
import os

show_help = False
suppress_check_output = False
slim_input_files = []
export_to_sxml = False
sxml_file = None
check_expr = False
expression = None
expr_error_file = None
check_guard = False
guard = None
guard_error_file = None
check_effects = False
effects = None
effects_error_file = None
error_file = None
root_set = False
root = None

def process_arguments():
    global show_help, suppress_check_output, export_to_sxml, sxml_file, root_set, root, slim_input_files, error_file, check_expr, expr, expr_error_file, check_effects, effects, effects_error_file, check_guard, guard, guard_error_file
    i = 1
    while i < len(sys.argv):
        if sys.argv[i] == "-sxml":
            # force check prior to export
            check = True
            export_to_sxml = True
        elif sys.argv[i] == "-sxmlf":
            if (i < len(sys.argv)):
                i += 1
                sxml_file = sys.argv[i]
            else:
                print "Malformed arguments."
        elif sys.argv[i] == "-nco":
            suppress_check_output = True
        elif sys.argv[i] == "-root":
            if i < len(sys.argv) - 1:
                root_set = True
                i += 1
                root = sys.argv[i]
            else:
                print "Malformed arguments."
        elif sys.argv[i] == "-o":
            if i < len(sys.argv) - 1:
                i += 1
                error_file = sys.argv[i]
            else:
                print "Malformed arguments."
        elif sys.argv[i] == "-g":
            if i < len(sys.argv) - 1:
                check_guard = True
                i += 1
                guard = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-og":
            if i < len(sys.argv) - 1:
                i += 1
                guard_error_file = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-ef":
            if i < len(sys.argv) - 1:
                check_effects = True
                i += 1
                effects = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-oef":
            if i < len(sys.argv) - 1:
                i += 1
                effects_error_file = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-e":
            if i < len(sys.argv) - 1:
                check_expr = True
                i += 1
                expr = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-oe":
            if i < len(sys.argv) - 1:
                i += 1
                expr_error_file = sys.argv[i]
            else:
                print "Malformed arguments"
        elif sys.argv[i] == "-help" or sys.argv[i] == "-h":
            show_help = True
        else:
            slim_input_files.append(sys.argv[i])
        i += 1

def printTextToFile(text, filename):
    file_handle = open(filename, "w")
    file_handle.write(text)
    file_handle.close()

def process_model():
    if check_guard:
        cinfo = CompileInformation()
        result = parseExpr(cinfo, guard)
        if guard_error_file is not None:
            printTextToFile(cinfo.error_collector.to_SXML(), guard_error_file)
        else:
            print cinfo.error_collector.to_SXML()
            
    if check_effects:
        cinfo = CompileInformation()
        result = parseEffects(cinfo, effects)
        if effects_error_file is not None:
            printTextToFile(cinfo.error_collector.to_SXML(), effects_error_file)
        else:
            print cinfo.error_collector.to_SXML()
            
    if check_expr:
        cinfo = CompileInformation()
        result = parseExpr(cinfo, expr)
        if expr_error_file is not None:
            printTextToFile(cinfo.error_collector.to_SXML(), expr_error_file)
        else:
            print cinfo.error_collector.to_SXML()
            
    if (len(slim_input_files) > 0):
        cinfo = CompileInformation()
        if not parse(cinfo, slim_input_files):
            if not suppress_check_output:
                if error_file is not None:
                    printTextToFile(cinfo.error_collector.to_SXML(), error_file)
                else:
                    print cinfo.error_collector.to_SXML()
        else:
            slim.check(cinfo)
            
            if not suppress_check_output:
                if error_file is not None:
                    printTextToFile(cinfo.error_collector.to_SXML(), error_file)
                else:
                    print cinfo.error_collector.to_SXML()
            
            if export_to_sxml:
                name = slim_input_files[0]
                low = name.rfind("/") + 1
                high = name.rfind(".")
                name = name[low:high]
                    
                if (sxml_file is not None):
                    printTextToFile(cinfo.to_SXML(name, True), sxml_file)
                else:
                    print cinfo.to_SXML(name, True)

def parseExpr(cinfo, expression):
    assert cinfo != None
    
    globals.PARSER_EXPRESSIONS_ONLY = True
    char_stream = antlr3.ANTLRStringStream(expression)
    
    lexer = DerivedLexer(cinfo, char_stream)
        
    tokens = antlr3.CommonTokenStream(lexer)
    parser = DerivedParser(cinfo, tokens)

    r = parser.expressionsOnly()

    globals.PARSER_EXPRESSIONS_ONLY = False
    return len(cinfo.error_collector.errors) == 0
    
def parseEffects(cinfo, expression):
    assert cinfo != None

    globals.PARSER_EFFECTS_ONLY = True

    char_stream = antlr3.ANTLRStringStream(expression)
    
    lexer = DerivedLexer(cinfo, char_stream)
        
    tokens = antlr3.CommonTokenStream(lexer)
    parser = DerivedParser(cinfo, tokens)

    r = parser.effectsOnly()
    
    globals.PARSER_EFFECTS_ONLY = False
    return len(cinfo.error_collector.errors) == 0

def parse(cinfo, files):
    """Parses a set of files compiling them into given compile info
    structure.

    @param cinfo: CompileInformation instance
    @param files: iterable sequence of slim filenames
    @return False if an error occurs during parsing"""
    assert cinfo != None

    for filename in files:         
        # sanity checks 
        assert os.path.exists(filename), "File %s not found" % filename
                     
        # let the compile_info structure know we are going to process a new file
        cinfo.current_file = os.path.abspath(filename)
        cinfo.files.append(filename)
    
        char_stream = antlr3.ANTLRFileStream(filename)
    
        lexer = DerivedLexer(cinfo, char_stream)
        
        tokens = antlr3.CommonTokenStream(lexer)
        parser = DerivedParser(cinfo, tokens)

        r = parser.specification()
    
        # start decorating
        if not parser.has_parsing_errors() and not lexer.has_lexing_errors():
            nodes = antlr3.tree.CommonTreeNodeStream(r.tree)
            nodes.setTokenStream(tokens)
            SlimWalkerIdentCheck(nodes).specification(cinfo)
        pass
    pass # end of loop

    return len(cinfo.error_collector.errors) == 0

def show_usage():
    print "Usage: python invoke.py [options] [slim files]"
    print "Options:"
    print "  -sxml \t creates the SXML representation of the SLIM files and prints it to the standard output"
    print "  -sxmlf FILE \t redirects the SXML output to the given file"
    print
    print "  -o FILE \t redirects the error report for the SLIM files to the given file"
    print
    print "  -g EXPR \t checks the given guard for syntax errors and prints the error report to the standard output"
    print "  -og FILE \t redirects the error report for the guard to the given file"
    print
    print "  -ef EXPR \t checks the given effect list for syntax errors and prints the error report to the standard output"
    print "  -oef FILE \t redirects the error report for the effect list to the given file"
    print
    print "  -e EXPR \t checks the given expression for syntax errors and prints the error report to the standard output"
    print "  -oe FILE \t redirects the error report for the expression to the given file"
    print
    print "  -nco \t suppresses the output of error checking on the given SLIM files"
    print
    print "  -h, -help \t shows this help message"

if __name__ == "__main__":
    process_arguments()
    
    if show_help:
        show_usage()
        quit(-1)
    
    process_model()
