# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


## Names of variables and definitions.
MODE_VAR = "mode"
ROOT_VAR = "root"
ACTIVE_DEF = "active"
TAU_EVENT = "#tau"
DELTA_VAR = "t_#delta"
TIME_VAR = "t_#time"  # this is introduced only when requested
DONE_DELTA_VAR = "done_delta"
STUTTER_EVENT = "stutter"
TIMED_EVENT = "timed"

## Prefixes.
IMPL_PREFIX = "Impl_"
SUBCOMP_PREFIX = "sc_"
EVENT_PREFIX = "event_"
EVENT_PAR_PREFIX = "__"
DATA_PREFIX = "data_"
DATA_PAR_PREFIX = "__"
CONST_PREFIX = "const_"
MODE_LIT_PREFIX = "mode_"
ERROR_LIT_PREFIX = "error_"
NONDET_PREFIX = "__nondet_"
ENUM_DEFAULT_PREFIX = "__enum_default_"
CANDO_PREFIX = "cando_"
RESOLVED_PREFIX = "resolved_"
DO_PREFIX = "do_"
VAL_PREFIX = "val_"

## Types.
BOOL_TYPE = "boolean"
REAL_TYPE = "real"
SIGNED_WORD_TYPE = "signed word"
UNSIGNED_WORD_TYPE = "unsigned word"

# Banner information for the resulting SMV model.
banner = \
    """======================================================================
  This file was generated by ${progname} version ${progver}
  Date: ${date}
  Command line: ${cmdline}
======================================================================
"""

# Translation unit.
afile = \
    """${banner}
    ${implementation_list}
    ${main_module}
    """

# this is the template has to be applied to every component implementation
implementation = \
    """
    ----------------------------------------------------------------------
    -- BEGIN OF ${implementation_name}
    ----------------------------------------------------------------------

    MODULE ${implementation_name} (${module_parameters})

    -- Event definitions.
    ${event_definition_list}

    -- Input data definitions.
    ${input_data_definition_list}

    -- Active definitions.
    ${active_definition}

    -- Mode and id declarations.
    ${mode_declaration}
    ${id_declaration}

    -- Mode and id invariants.
    ${mode_invariant_list}
    ${event_id_invariant_list}

    -- Output data ports declarations and definitions.
    ${output_data_port_list}

    -- Data subcomponent declarations.
    ${data_subcomponent_declaration_list}

    -- Control subcomponent declarations.
    ${control_subcomponent_declaration_list}

    -- Initialization.
    ${initialization_list}

    -- Mode transition cando definitions.
    ${cando_mode_trans_definition_list}

    -- Event cando definitions.
    ${cando_event_definition_list}

    -- Transitions.
    ${transition_list}

    -- Resolved definitions.
    ${resolved_definition_list}

    -- Control subcomponent (re)activation definitions.
    ${control_subcomponent_active_definition_list}

    -- Subcomponent do event definitions.
    ${do_control_subcomponent_event_definition_list}

    -- Subcomponent input data definitions.
    ${input_data_subcomponent_definition_list}

    ----------------------------------------------------------------------
    ---- END OF ${implementation_name}
    ----------------------------------------------------------------------
    """

# Parent active status.
PARENT_ACTIVE = "parent_active"

# Activation status.
ACTIVATED = "activated"

# Reactivation status.
REACTIVATED = "reactivated"

# Event definition that is visible at module level.
event_definition = \
    """DEFINE ${event_name} := ${event_par_name};
    """

# Timed event definition.
timed_event_definition = \
    """DEFINE %s := %s > 0;
    """ % (TIMED_EVENT, DELTA_VAR)

# Stutter definition.
stutter_definition = \
    """DEFINE %s := !(%s | ${event_list});
    """ % (STUTTER_EVENT, REACTIVATED)

# Input data definition that is visible a module level.
input_data_definition = \
    """DEFINE ${in_data_name} := ${in_data_par_name};
    """
# Mode declaration.
mode_declaration = \
    """VAR %s : {${mode_list}};
    """ % MODE_VAR

# Mode invariant.
mode_invariant = \
    """INVAR %s = ${mode} -> ${invar};
    """ % MODE_VAR

# Module parameters.
module_parameters = \
    "%s, %s, %s, ${tau_event_par}${in_event_par_list}${out_event_par_list}"\
    "${in_data_par_list}${delta}" % (PARENT_ACTIVE, ACTIVATED, REACTIVATED)

# Active definition.
active_definition = \
    """DEFINE %s := %s & %s;
    """ % (ACTIVE_DEF, PARENT_ACTIVE, ACTIVATED)

# ID variable.
ID_VAR = "id"

# ID declaration.
id_declaration = \
    """IVAR %s : 1 .. ${max_id};
    """ % ID_VAR

# Invariant of event-associated id.
event_id_invar = \
    """TRANS ${event} -> (%s <= ${max_event_id});
    """ % ID_VAR

# ID equality.
id_equal = "%s = ${id}" % ID_VAR

# Potentially-assigned output data port declaration.
assigned_odp_declaration = \
    """VAR ${name} : ${type};
    """

# Unassigned output data port definition.
odp_nondet = "%s${name}" % NONDET_PREFIX

odp_nondet_declaration = \
    """IVAR %s${name} : ${type};
    """ % NONDET_PREFIX

unassigned_odp_definition = \
    """DEFINE ${name} :=
         case
           ${odp_connection_values}
           ${odp_flow_values}
           TRUE : ${default_value};
         esac;
    """

unassigned_odp_definition_wo_default = \
    """DEFINE ${name} :=
         case
           ${odp_connection_values}
           ${odp_flow_values}
         esac;
    """

unassigned_odp_definition_opt = \
    """DEFINE ${name} := ${default_value};
    """

odp_enum_default = "%s${name}" % ENUM_DEFAULT_PREFIX

odp_enum_default_declaration = \
    """VAR %s${name} : ${type};
    """ % ENUM_DEFAULT_PREFIX

odp_enum_default_invar = \
    """INVAR %s${name} = ${value};
    """ % ENUM_DEFAULT_PREFIX


# Ouput data port value caused by a connection.
odp_connection_value = \
    """${mode_cond} : ${source};
    """

# Output data port value caused by flow.
odp_flow_value = \
    """${mode_cond} : ${flow_expr};
    """

# Mode condition.
mode_cond = "${mode_var} in {${modes}}"
mode_cond_single = "${mode_var} = ${mode}"
mode_cond_all = "TRUE"

# Declaration of data subcomponent.
data_subcomponent_declaration = \
    """VAR ${name} : ${type};
    """
# Subcomponent do event.
sc_do_event = "${sc_name}_%s${event_name}" % DO_PREFIX

# Subcomponent activated and reactivated.
sc_activated = "${sc_name}_%s" % ACTIVATED
sc_reactivated = "${sc_name}_%s" % REACTIVATED

# Subcomponent delta value.
sc_delta = "${sc_name}_%s" % DELTA_VAR

# Subcomponent input data.
sc_in_data = "${sc_name}_%s${data_name}" % VAL_PREFIX

# Declaration of control subcomponent.
control_subcomponent_declaration = \
    """VAR ${name} : ${impl_name}(%s, ${activated}, ${reactivated}, ${do_tau_event}${do_in_event_list}${do_out_event_list}${in_data_val_list}${delta});
    """ % ACTIVE_DEF

# Definition of activation and reactivation.
control_sc_activation_definition = \
    """DEFINE ${sc_active_name} := ${mode_cond};
    """

control_sc_reactivation_definition = \
    """DEFINE ${sc_reactive_name} := !${sc_name}.%s & next(%s) & next(${sc_active_name});
    """ % (ACTIVE_DEF, ACTIVE_DEF)

# Input data port definition.
idp_nondet = "%s${name}" % NONDET_PREFIX

idp_nondet_declaration = \
    """IVAR %s${name} : ${type};
    """ % NONDET_PREFIX

# Input data port value definition.
idp_value_definition = \
    """DEFINE ${val_name} :=
         case
           ${idp_connection_values}
           ${idp_flow_values}
           TRUE : ${default_value};
         esac;
    """

idp_value_definition_wo_default = \
    """DEFINE ${val_name} :=
         case
           ${idp_connection_values}
           ${idp_flow_values}
         esac;
    """

idp_value_definition_opt = \
    """DEFINE ${name} := ${default_value};
    """

idp_enum_default = "%s${name}" % ENUM_DEFAULT_PREFIX

idp_enum_default_declaration = \
    """VAR %s${name} : ${type};
    """ % ENUM_DEFAULT_PREFIX

idp_enum_default_invar = \
    """INVAR %s${name} = ${value};
    """ % ENUM_DEFAULT_PREFIX

# Input data port value caused by a connection.
idp_connection_value = \
    """${mode_cond} : ${source};
    """

# Input data port value caused by flow.
idp_flow_value = \
    """${mode_cond} : ${flow_expr};
    """

### INIT TEMPLATES ################################################

# Mode initialization
mode_init = \
    """INIT %s = ${init_mode};
    """ % MODE_VAR

# Data subcomponent initialization.
data_subcomponent_init = \
    """INIT ${name} = ${initial_value};
    """

# Inactive clock component initialization.
data_clock_init_inactive = \
    """INIT ${name} = 0;
    """
# Assigned output data port initialization.
output_data_port_init = \
    """INIT ${name} = ${initial_value};
    """

### CANDO TEMPLATES ################################################

# Definition of mode transition cando.
cando_mode_trans_definition = \
    """DEFINE ${cando_name} := ${mode_cond}${guard};
    """
#cando_mode_trans_definition = \
#    """DEFINE ${cando_name} := ${mode_cond}${guard}${next_invar};
#    """

# Definition of event transition cando.
cando_event_trans_definition = \
    """DEFINE ${cando_name} := %s & ${cando_cond};
    """ % ACTIVE_DEF

# Definition of cando timed transition.
cando_timed_transition_definition = \
    """DEFINE %s%s := %s;
    """ % (CANDO_PREFIX, TIMED_EVENT, ACTIVE_DEF)

### TRANSITION TEMPLATES ############################################

# Event transition.
event_transition = \
    """TRANS ${event} & %s = ${id} -> ${constraint};
    """ % ID_VAR

# Timed transition.
timed_transition = \
    """TRANS %s -> %s%s & %s = 1 & next(%s) = %s${effects};
    """ % (TIMED_EVENT, CANDO_PREFIX,
           TIMED_EVENT, ID_VAR, MODE_VAR, MODE_VAR)

# Trajectories.
trajectories = \
    """(case
      ${mode_trajectory_list}
      TRUE : ${stuttering_list};
    esac)
    """

mode_trajectory = \
    """%s = ${mode} : ${effects};
      """ % MODE_VAR

# Stutter transition.
stutter_transition = \
    """TRANS %s -> %s = 1 & ${effects};
    """ % (STUTTER_EVENT, ID_VAR)

# Reactivation transition
reactivation_transition_act = \
    """TRANS %s -> %s = 1 &
    case
      -- reactivation transitions:
      ${reactivation_trans_cases}
      -- default reactivation:
      TRUE: ${reactivation_def_trans};
    esac;
    """ % (REACTIVATED, ID_VAR)

reactivation_trans_case = \
    """%s = ${mode}: ${mode_reactivation_trans_or};""" % MODE_VAR

mode_reactivation_trans = \
    """next(%s) = ${mode}${effects}""" % MODE_VAR

reactivation_def_trans = \
    """next(%s) = ${mode}${effects}""" % MODE_VAR

# Mode transition.
mode_transition = \
    "(${cando_name} & next(${mode_var}) = ${next_mode}${effects})"

# Non mode transition.
non_mode_transition = "(next(${mode_var}) = ${mode_var}${effects})"

# Resolved definition.
resolved_definition = \
    """DEFINE ${resolved_name} := ${event} & ${id_cond};
    """

# Event mode transition (used by guided sim).
event_mode_transition = \
    "${event} & ${id_var} = ${id} & ${mode_trans}"

# stutter
stutter_var_transition = \
    "next(${var_name}) = ${var_name}"

# var assign
var_ass_transition = \
    "next(${var_name}) = ${value}"

# trajectory in time transitions
traj_time_transition = \
    "next(${var_name}) = (${var_name} + ${delta})"

### DO TEMPLATES ####################################################

# Do definition.
do_definition = \
    """DEFINE ${do_sc_event} := ${cando_sc_event} & ${conds};
    """

do_wo_cando_definition = \
    """DEFINE ${do_sc_event} := ${conds};
    """

# Do tau definition.
do_tau_definition = \
    """DEFINE ${do_sc_event} := ${conds};
    """

# Do timed definition.
do_timed_definition = \
    """DEFINE ${sc_delta} :=
         case
           %s & ${sc_cando} : %s;
           TRUE : 0;
         esac;
    """ % (TIMED_EVENT, DELTA_VAR)

### MAIN MODULE #####################################################

main_module = \
    """
    ----------------------------------------------------------------------
    ----------------------------------------------------------------------
    -- Top level (main) module
    ----------------------------------------------------------------------

    MODULE main

    -- Event and input data declarations.
    ${root_event_declaration_list}
    ${root_input_data_declaration_list}

    -- Event and input data constraints.
    ${root_event_constraint_list}
    ${root_input_data_constraint_list}

    -- Timed transition.
    ${root_timed_transition}

    -- At-most-one-event constraint.
    ${root_at_most_one_event_constraint}

    -- Root component declaration.
    ${root_component_declaration}

    -- Pending constants.
    ${pending_constants}

    ----------------------------------------------------------------------
    ----------------------------------------------------------------------
"""

# Declaration of root event.
root_event_declaration = \
    """IVAR ${event} : %s;
    """ % BOOL_TYPE

# Declaration of input data.
root_input_data_declaration = \
    """VAR ${ivar} : ${type};
    """

# Root event constraint.
root_event_constraint = \
    """TRANS ${event} -> %s.${cando_event};
    """ % ROOT_VAR

# Root input data constraint.
root_input_data_constraint = \
    """INVAR ${ivar} = ${val};
    """

# Declaration of root component.
root_component_declaration = \
    """VAR %s : ${name}(TRUE, TRUE, FALSE,
    ${do_tau_event}${do_in_event_list}${do_out_event_list}${in_data_val_list}${delta});
    """ % ROOT_VAR

# Root timed transitions.
root_timed_transition = \
    """IVAR {0} : {1};
    $root_time_var
    TRANS {0} >= 0;
    DEFINE {2} := {0} > 0;
    $root_time_fairness_alt
    """.format(DELTA_VAR, REAL_TYPE, TIMED_EVENT)


# time variable is introduced only when requested so
root_time_var = \
    """VAR {0} : {1};

    INIT {0} = 0;
    TRANS next({0}) = {0} + {2};
    """.format(TIME_VAR, REAL_TYPE, DELTA_VAR)

# additional constraints for timed transitions note: these should be
# seen as hacks, as now zenoness and time divergence could be handled
# at engine level
root_time_fairness_alt = \
    """VAR {0} : {1};

    FAIRNESS {2};

    -- No two consecutive timed transitions.
    INIT !{0}
    TRANS {2} -> !{0} & next({0});
    TRANS !{2} -> !next({0});
    """.format(DONE_DELTA_VAR, BOOL_TYPE, TIMED_EVENT)


# At-most-one-event constraint.
root_atmost_one_event_constraint = \
    """TRANS count(${event_list}) = 1;
    """

# Pending constants.
pending_constants = \
    """CONSTANTS ${constant_list};
    """
