# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
import sys
sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../')

import unittest

from invokers import slim
from slim_compiler.src.dataStructure.compile import CompileInformation
from slim_compiler.src.dataStructure.nominalbehaviour import ComponentImplementation
from slim_compiler.src.dataStructure.errorbehaviour import ErrorModelImplementation
import tempfile

class BasicTestCaseNeg(unittest.TestCase):

    def _parse_only(self, slim_snippet):
        # generate temp file
        f = tempfile.NamedTemporaryFile()
        try:
            f.write(slim_snippet)
            f.flush()
            # set up new compile info object
            cinfo = CompileInformation()
            slim.parse(cinfo, [f.name])
            slim.check(cinfo)
            ComponentImplementation.reset()
            ErrorModelImplementation.reset()   
        finally:
            f.close()
        return cinfo

    def _expect_errors(self, cinfo, *error_ids):
        return cinfo.error_collector.contains_only(error_ids)

    def _has_no_errors(self, cinfo):
        no_error = cinfo.error_collector.has_no_errors()
        if not no_error:
            print cinfo.error_collector.get_error_id_set()
        return no_error

class ErrorClockTests(BasicTestCaseNeg):
    def test_Positive(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._has_no_errors(cinfo))
 
    def test_ClockUnknown(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk2>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "K-22"))
        pass
 
    def test_ClockUnknown_2(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk2:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "K-26"))
        pass
 
    def test_ErrorStateDefinitionMutual(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
                e2 : initial state;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "K-15"))
        pass
 
    def test_ErrorTransitionNonlinear(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk*clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "C-5"))
        pass
 
    def test_InvariantOnProbabilisticErrorState(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;
 
        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;
 
        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;
 
        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state while clk<2;
            transitions
                e1 -[evt1 when clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 then clk:=0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "K-27"))
        pass

    def test_GuardAtProbabilisticTransition(self):
        slim_snippet = """
package test
    public
        system sys
            features
                evt1 : in event port;
        end sys;

        system implementation sys.impl
            subcomponents
                clk : data clock;
            modes
                m0 : initial mode while clk<4.0;
                m1 : mode;
            transitions
                m0 -[evt1 when clk>2.0 and clk<3.0]-> m1;
        end sys.impl;

        error model err_timed
            features
                p1 : out error propagation;
        end err_timed;

        error model implementation err_timed.impl
            events
                evt1: error event;
                evt2: error event occurrence poisson 0.5;
            clocks
                clk : data clock;
            states
                e1 : initial state while clk<4.0;
                e2 : error state;
            transitions
                e1 -[evt1 when clk>2.0]-> e2;
                e1 -[p1 when clk>3.0]-> e2;
                e2 -[evt2 when clk>1.0]-> e1;
        end err_timed.impl;
end test;
"""
        cinfo = self._parse_only(slim_snippet)
        self.assertTrue(self._expect_errors(cinfo, "K-29"))
        pass

if __name__ == "__main__":
    unittest.main()
