# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
import sys
#sys.path.append('../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../../resources/antlr/antlr-python-runtime-3.1.1/')
sys.path.append('../../')

import unittest

import os
from invokers import slim
from slim_compiler.src.dataStructure.compile import CompileInformation
from slim_compiler.src import globals
import tempfile
from slim_compiler.src.dataStructure.visitors.compileVisitor import compareVisitor
from slim_compiler.src.dataStructure.visitors.printVisitor import toSLIMVisitor

def compare(self, slimFiles, faultFile):
    # Parse slim model and extend it  
    globals.PARSER_ACCEPT_EXTENDED_MODEL = False
    
    for f in slimFiles:
        self.assertTrue(os.path.exists(f),"\"%s\" does not exist"%f)
        
    cinfo = CompileInformation()
    slim.parse(cinfo, slimFiles)
    slim.check(cinfo)
    ext_cinfo = slim.do_fault_injection(cinfo, faultFile)
    
    # Parse extended model and compare it to the previously extended one
    globals.PARSER_ACCEPT_EXTENDED_MODEL = True
    f = tempfile.NamedTemporaryFile()
    f.write(toSLIMVisitor().visit(ext_cinfo))
    f.flush()
    
    cinfoFromExtended = CompileInformation()
    slim.parse(cinfoFromExtended, [f.name])
    slim.check(cinfoFromExtended)
    
    self.assertTrue(compareVisitor().visit(ext_cinfo, cinfoFromExtended))
    
    f.close()

class Test(unittest.TestCase):
    def test_1(self):
        compare(self, ['../../examples/Adder.slim','../../examples/Adder_err.slim'],'../../examples/Adder.fixml')

    def test_2(self):
        compare(self, ['../../examples/Adder-2.slim','../../examples/Adder_err-2.slim'],'../../examples/Adder-2.fixml')

#    def test_3(self):
#        compare(self, ['../../examples/sensorfilter.slim','../../examples/sensorfilterErr.slim'],'../../examples/sensorfilter.fixml')

#    def test_4(self):
#        compare(self, ['../../examples/sensorfilter-hybrid.slim','../../examples/sensorfilterErr-hybrid.slim'],'../../examples/sensorfilter.fixml')

if __name__ == "__main__":
    unittest.main()
