# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================

import networkx as nx
import os
from gtkmvc import Model
from src.models.XMLParser import XMLParser
from src.models.ConcreteSyntaxParser import ConcreteSyntaxParser


class TFPGModel(Model):
    """Model for the TFPG graph visualization"""

    def __init__(self):
        """Class constructor"""
        Model.__init__(self)
        return

    def build_graph(self, structural_file):
        """Builds the corresponding graph for a structural file.

        The structural file can be in xml syntax or human readable.
        Depending on its extension ('xml' or 'tfpg'), it calls
        the proper function

        :param structural_file: the input structural file
        :type structural_file: string

        """
        file_name, file_extension = os.path.splitext(structural_file)

        if file_extension in ".xml .txml".split():
            self.xml_parser = XMLParser()
            return self.xml_parser.build_graph(structural_file)

        elif file_extension == ".tfpg":
            self.concrete_syntax_parser = ConcreteSyntaxParser()
            return self.concrete_syntax_parser.build_graph(structural_file)           


    def build_trace(self, xml_file):
        """Builds the corresponding trace for the xml file

        :param xml_file: the xml file containing the tfpg trace
        :type xml_file: string

        """

        self.xml_parser = XMLParser()
        return self.xml_parser.build_trace(xml_file)

pass  # end of class
