# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================


import os
import options
import utils.process


# A possible value of the attribute "returncode" of Dft2MrmcProcess objects,
# indicating that the resulting model contains action transitions, so the
# output in MRMC format is not meaningful:
ACTION_TRANSITIONS_EXIST = 100


class Dft2MrmcProcess (utils.process.InteractiveProcess):
    """Process handler for running dft2mrmc.
    """

    def __init__(self, fault_tree_filename, rates_filename, labels_filename,
                 tra_output_filename, lab_output_filename, init_output_filename, 
                 stdout=None, stdin=None, stderr=None, dumpers=[]):        

        # The specified input files must exist.
        if not os.path.isfile(fault_tree_filename):
            raise IOError("File not found: " + fault_tree_filename)
        if not os.path.isfile(rates_filename):
            raise IOError("File not found: " + rates_filename)
        if not os.path.isfile(labels_filename):
            raise IOError("File not found: " + labels_filename)
        
        # Set runtime options. Set verbosity to 0 (last command line
        # argument).
        path = utils.process.get_exec_name(options.DFT2MRMC)
        args = [fault_tree_filename, rates_filename, labels_filename,
                tra_output_filename, lab_output_filename,
                init_output_filename, "0"]

        super(Dft2MrmcProcess, self).__init__(path, args, stdout=stdout, 
                                              stdin=stdin, stderr=stderr, 
                                              dumpers=dumpers)

    
pass # end of class
