# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================
class AnalysisResults(object):
    """Results of property analysis
    """

    def __init__(self, property, tool,
                 probability=None,
                 probability_steps=None,
                 expected_time=None,
                 lra=None):

        if not any([probability!=None, probability_steps!=None,
                    expected_time!=None, lra!=None]):
            raise Exception("No results passed to AnalysisResults object")

        self.property = property
        self.tool = tool
        self.probability = probability
        self.probability_steps = probability_steps
        self.expected_time = expected_time
        self.lra = lra


    def is_prob_analysis(self):
        return self.probability != None or self.probability_steps != None

    def has_prob_result(self):
        return self.probability != None

    def has_stepwise_results(self):
        return self.probability_steps != None
    
    def is_et_analysis(self):
        return self.expected_time != None

    def is_lra_analysis(self):
        return self.lra != None

    def invertProbabilities(self):
        if not self.is_prob_analysis():
            raise Exception("Trying to invert probability results," /
                            " but none set")

        def invert(p): return (1-p[1],1-p[0]) if type(p) is tuple else 1-p

        if self.has_stepwise_results():
            self.probability_steps = self.probability_steps[0], map(invert,self.probability_steps[1])
        else:
            self.probability = invert(self.probability)

    def __str__(self):
        if self.probability!=None:
            res = "prob = " + str(self.probability)

        elif self.probability_steps!=None:
            res = "probs = " + str(zip(self.probability_steps[0],self.probability_steps[1]))

        elif self.expected_time!=None:
            res = ("expected time = " + str(self.expected_time[0])
                   if self.expected_time[0] == self.expected_time[1]
                   else "(min, max) expected time = " + str(self.expected_time))

        else:
            res = ("LRA = " + str(self.lra[0])
                   if self.lra[0] == self.lra[1]
                   else "(min, max) LRA = " + str(self.lra))

        return res
