# =============================================================================
#  This file is part of the ``HASDEL'' Toolset.
#  Copyright (C) 2014 by Fondazione Bruno Kessler
#  Copyright (C) 2014 by RWTH Aachen University
#
#
#  The HASDEL Toolset can be copied, modified and/or redistributed
#  under the terms of the HASDEL General Public License, either
#  version 1 of the License, or (at your option) any later version.
#
#  You should have received a copy of the HASDEL General Public
#  License along with this software. If not, please contact the
#  copyright owners.
#
#  The HASDEL Toolset is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  HASDEL General Public License for more details.
#
#  For more information on the HASDEL Project see the Project Web
#  site at <http://compassinternal.informatik.rwth-aachen.de/hasdel-site/> 
#  or email to <hasdel-support@lists.rwth-aachen.de>.  
#
#  Please report bugs to <hasdel-support@lists.rwth-aachen.de>.
# =============================================================================



import os.path
import options

import slim_compiler.src.smv.api as smvapi
import utils.nusmv_process
import utils.process
from utils import log
import slim_compiler.src.smv.api as smvapi

class Smv2SigrefProcess (utils.nusmv_process.NuSMVShellProcess):
    """Process handler for running Smv2Sigref"""

    def __init__(self, stdout=None, stderr=None, dumpers=[]):

        path = utils.process.get_exec_name(options.SMV2SIGREF)
        super(Smv2SigrefProcess, self).__init__(path, need_msat=False, args=["-int"], 
                                                stdout=stdout, stderr=stderr, 
                                                dumpers=dumpers)        

        # these are used internally for memoization
        self.__input = (None, None, None, None)
        self.__res = (None, None)
        return
    

    def build_sigref_inputs(self, smv_fname, prob_fname, 
                            ord_fname=None, label_fname=None,
                            out_fname=None):
        """Returns the name of the generated xml file to be used as
        input file, and the name of the generated BDD variables
        ordering file, as a pair of strings"""
        
        basename = os.path.splitext(smv_fname)[0]        
        if ord_fname is None: 
            # try to see if there exists a possible ordering file
            ord_fname = basename + ".ord"
            if not os.path.isfile(ord_fname): ord_fname = None
            pass

        new_ord_fname = basename + ".ord"

        if out_fname is None: out_fname = basename + "_smv2sigref.xml"
        
        input_files = (smv_fname, prob_fname, ord_fname, label_fname)
        
        # checks if previously calculated (memoization)        
        if input_files == self.__input: return self.__res
        self.__input = input_files

        # checks that given files exist:
        for fname in input_files:
            if fname is not None and not os.path.isfile(fname): raise IOError("File not found: " + fname)
            pass

        labelarg = ('-a "%s"' % label_fname) if label_fname else ''

        for cmd in ['set on_failure_script_quits', 
                    'set input_file "%s"' % smv_fname, 
                    ('set input_order_file "%s"' % ord_fname) if ord_fname else None, 
                    'reset',
                    'dynamic_var_ordering -e sift',
                    'go',
                    'echo "Generating output in sigref format..."',
                    'compass_gen_sigref -i "%s" -t "%s" %s -o "%s"' % \
                        (prob_fname, smvapi.get_top_tau_event(), labelarg, out_fname),
                    'echo "Generated output in sigref format in file %s"' % out_fname,
                    'write_order -b -o "%s"' % new_ord_fname, 
                    ]:                                        
            if cmd: self.send_cmd(cmd)
            pass

        # now waits for the result
        self.wait_for_command_finish()

        self.__res = (out_fname, new_ord_fname)
        return self.__res


    pass # end of class
# ----------------------------------------------------------------------
